** FILE:
**    SWAUTIL.FOR - part of program SWAP
** FILE IDENTIFICATION:
**    $Id: SWAUTIL.FOR 2.4 1999/11/18 16:12:23 Kroes Exp $
** DESCRIPTION:
**    This file contains program units of SWAP. The program units 
**    in this file utilities and are sorted alphabetically
**    with the first letter of the program unit ranging from A - O.
**    The following program units are in this file:
**        AFGEN, ANALIN, DATONR, ERRHANDL, FINDUNIT, FROMTO, JMPLBL,
**        LIMIT, NRTODA, RADATA, RACHAR, RADOUR, RAINTR, RAREAR,
**        RFCHAR, RFDOUR, RFINTR, RFREAR, RSCHAR, RSDATA, RSDOUR,
**        RSINTR, RSREAR, SHIFTL, SHIFTR, SKPLBL, STEPNR
************************************************************************
C ----------------------------------------------------------------------
      REAL*8 FUNCTION AFGEN(TABLE,ILTAB,X)
C ----------------------------------------------------------------------
C     Source             : Kees Rappoldt, 1/86
C     Purpose            : Linear interpolation in table TABLE with
C                          length ILTAB for a given value of the
C                          independent variable X.
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER I,ILTAB
      REAL*8  TABLE(ILTAB),SLOPE,X
C ----------------------------------------------------------------------
      IF (TABLE(1).GE.X)  GOTO 40
      DO 10 I = 3,ILTAB-1,2
        IF (TABLE(I).GE.X) GOTO 30
        IF (TABLE(I).LT.TABLE(I-2)) GOTO 20
 10   CONTINUE
C --- table fully filled, argument larger then last X in table
      AFGEN = TABLE(ILTAB)
      RETURN
C --- table partly filled, argument larger then last X in table
 20   AFGEN = TABLE(I-1)
      RETURN
C --- argument between first and last X in table, interpolation
 30   SLOPE = (TABLE(I+1)-TABLE(I-1))/(TABLE(I)-TABLE(I-2))
      AFGEN = TABLE(I-1) + (X-TABLE(I-2))*SLOPE
      RETURN
C --- argument less or equal to first X in table
 40   AFGEN = TABLE(2)
      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE ANALIN (LINE,BLANK,COMMENT)
C ----------------------------------------------------------------------
C     Date               : 27/10/1997
C     Purpose            : check if LINE is blank or COMMENT line
C                          COMMENT lines start with '*' or '!'
C     Subroutines called :
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      CHARACTER LINE*80
      LOGICAL BLANK,COMMENT
C ----------------------------------------------------------------------
C --- local
      INTEGER I
C ----------------------------------------------------------------------
      BLANK = .TRUE.                     
      I = 1
  30  IF (LINE(I:I) .NE. ' ') THEN
        BLANK = .FALSE.
      ELSE IF (I .LT. 80) THEN
        I = I+1
        GOTO 30
      ENDIF

      COMMENT = .FALSE.
      I = 1
40    IF (LINE(I:I).NE.' ') THEN
        IF (LINE(I:I).EQ.'*'.OR.LINE(I:I).EQ.'!') COMMENT = .TRUE.
      ELSEIF (I.LT.80) THEN
        I = I+1
        GOTO 40
      ENDIF

      RETURN 
      END

C ----------------------------------------------------------------------
      SUBROUTINE DATONR (YEAR,MONTH,DAY,DAYNR)
C ----------------------------------------------------------------------
C     Date               : 11/91
C     Purpose            : Conversion of date (YEAR,MONTH,DAY) to    
C                          daynumber (DAYNR)
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE 
      
      INTEGER ILEAP,YEAR,MONTH,DAY,DAYNR
      INTEGER FDM365(12),FDM366(12)

      DATA FDM365/1,32,60,91,121,152,182,213,244,274,305,335/
      DATA FDM366/1,32,61,92,122,153,183,214,245,275,306,336/
C ----------------------------------------------------------------------

C --- is YEAR a leap-year?
      ILEAP=0
      IF (((MOD(YEAR,4).EQ.0).AND.(MOD(YEAR,100).NE.0)).OR.
     $     (MOD(YEAR,400).EQ.0)) ILEAP=1   

C --- calculate daynumber
      IF (ILEAP.EQ.0) THEN
        DAYNR=FDM365(MONTH)+DAY-1
      ELSEIF (ILEAP.EQ.1) THEN
        DAYNR=FDM366(MONTH)+DAY-1
      ENDIF

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE ERRHANDL(UOER,SWER,KIER,MONA,NAER,PANA1,PAVA1,PADN1,
     &                    PANA2,PAVA2,PADN2,PANA3,PAVA3,PADN3)
************************************************************************
** SUBPROGRAM:
**    subroutine errhandl - handling of errors
** SYNOPSIS:
**    SUBROUTINE ERRHANDL
**    IN
**       integer      
**          UOER = unit number of output file with error messages (-)
**          SWER = switch for error output; 1=complete,2=partial (-)
**          KIER = kind of error (-)
**          MONA = module name that produced error (-)
**          NUPA = number of parameters as real type (-)
**       real
**          PAVA(3) = 3 parameter values of real type (-)
**       character    
**          NAER = name of error message (error description) (-)
**          PADN1,PADN2,PADN3 = the dimensions(units) of parameters 1,2,3 (-)
**          PANA1,PANA2,PANA3 = the names of parameters 1,2,3 of real type (-)
**    OUT: 
**       integer
**          UOER = unit number for file with error messages (-)
**       file usage:  UOER
**    USE OF SUBPROGRAMS:   -
**    USE OF COMMON BLOCKS: -
** DESCRIPTION:
**    This subroutine deals with all error handling. A file is opened
**    and 3 types of messages are written to this file: 
**    1. warning: these messages are informational only
**    2. recoverable error: these messages are informational only
**    3. fatal error: these messages indicate a severe problem, one that
**                    prevents a continuation of the simulation. This 
**                    message always results in interrupting simulation.
** HISTORY:
**    May 1995       J.G. Kroes
**                   creation
************************************************************************
C---- Declarations
      IMPLICIT NONE
C --- formal arguments
      INTEGER  KIER, SWER, UOER
      CHARACTER*(*) MONA, NAER, PADN1, PADN2, PADN3, PANA1, PANA2, PANA3
      REAL*8   PAVA1,PAVA2,PAVA3
C --- local variables
      CHARACTER*18 HV(3)
      INTEGER  I
C --- retain values of local variables
      SAVE
C --- local data
      DATA   (HV(I),I=1,3)    /'Warning           ',
     &    'Recoverable error ','Fatal error       '/
C ----------------------------------------------------------------------      
      IF (SWER.EQ.2 .AND. KIER.NE.3)  THEN
C--       Suppress warning and recoverable errors
        CONTINUE 

      ELSE
C--       Write Warning, Recoverable Error or Fatal error
        WRITE(UOER,11)  HV(KIER), MONA, NAER

        IF (PANA1.NE.' ') THEN
          IF (ABS(PAVA1).LT. 1.0d0 .OR. ABS(PAVA1).GT.100000.0d0) THEN
            WRITE(UOER,22)  PANA1, PAVA1, PADN1
          ELSE
            WRITE(UOER,33)  PANA1, PAVA1, PADN1
          END IF
        END IF
        IF (PANA2.NE.' ') THEN
          IF (ABS(PAVA2).LT. 1.0d0 .OR. ABS(PAVA2).GT.100000.0d0) THEN
            WRITE(UOER,22)  PANA2, PAVA2, PADN2
          ELSE
            WRITE(UOER,33)  PANA2, PAVA2, PADN2
          END IF
        END IF
        IF (PANA3.NE.' ') THEN
          IF (ABS(PAVA3).LT. 1.0d0 .OR. ABS(PAVA3).GT.100000.0d0) THEN
            WRITE(UOER,22)  PANA3, PAVA3, PADN3
          ELSE
            WRITE(UOER,33)  PANA3, PAVA3, PADN3
          END IF
        END IF
      END IF

   
C--     Fatal error
      IF (KIER.EQ.3) THEN
        WRITE(UOER,55)
        WRITE(*,11)  HV(3), MONA, NAER
        WRITE(*,44)
        WRITE(*,55)
        STOP
      END IF


      RETURN

   11 FORMAT(1X,A19,'   Produced by module  ',A,/,6X,A)
   22 FORMAT(6X,A,' = ',E10.3,5X,A)
   33 FORMAT(6X,A,' = ',F12.3,5X,A)
   44 FORMAT(6X,'verify output file MESSAGE.OUT')
   55 FORMAT(6X,'*** SIMULATION INTERRUPTED ***')
      END

C ----------------------------------------------------------------------
      SUBROUTINE FINDUNIT(UI,UO)
C ----------------------------------------------------------------------
C     Date               : 22/10/97
C     Purpose            : finds first free even unit number
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
C---- Declarations
      IMPLICIT NONE
      INTEGER  UI,UO
C ----------------------------------------------------------------------                                                 
C --- local
      LOGICAL  INUSE
C ----------------------------------------------------------------------      
C---- Initialise
      UO = UI
      IF (MOD(UO,2).EQ.1) UO = UO +1
      INQUIRE (UNIT = UO, OPENED = INUSE)

C---- Search for available unit number
      IF (INUSE) THEN
        DO 10 WHILE (INUSE) 
          INQUIRE (UNIT = UO, OPENED = INUSE)
          UO = UO + 2
   10   CONTINUE
        UO = UO - 2
      END IF
   
      RETURN
      END

C ----------------------------------------------------------------------
      INTEGER FUNCTION FROMTO (DAY1,YEAR1,DAY2,YEAR2)
C ----------------------------------------------------------------------
C     Date               : 2/92
C     Purpose            : Calculates number of days between DAY1, YEAR1
C                          and DAY2, YEAR2
C     Subroutines called : DATONR
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER I,J,DAY1,YEAR1,DAY2,YEAR2,DAYS
C ----------------------------------------------------------------------
      IF (YEAR1.EQ.YEAR2) THEN
        FROMTO = DAY2 - DAY1
      ELSEIF (YEAR2.GT.YEAR1) THEN 
        DAYS=0
        DO 10 I = YEAR1,YEAR2-1
          CALL DATONR (I,12,31,J)
          DAYS = DAYS + J
10      CONTINUE
        FROMTO = DAYS+DAY2-DAY1
      ENDIF

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE JMPLBL (INF,FNAME,LOGF,NAME)  
C ----------------------------------------------------------------------
C     Date               : 17/04/98
C     Purpose            : jump to label NAME
C     Subroutines called :
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   INF,LOGF
      CHARACTER *(*) NAME
      CHARACTER*12 FNAME
C ----------------------------------------------------------------------
C --- local
      INTEGER NLB,J,K,NL,NN,IOST

      CHARACTER LINE*80

      LOGICAL BLANK,COMMENT
C ----------------------------------------------------------------------
C --- tijdelijk
      LOGF = LOGF

C --- always start from BOF
      REWIND (INF)

10    READ (INF,'(A)',IOSTAT=IOST) LINE
      IF (IOST.LT.0) then
C ---   EOF reached, label not found
        WRITE (*,12) NAME,FNAME
        WRITE (LogF,12) NAME,FNAME
12      FORMAT ('Label: ',A,' not found in file: ',A)
        STOP
      ENDIF 
      CALL ANALIN (LINE,BLANK,COMMENT)
      IF (BLANK.OR.COMMENT) GOTO 10

C --- count number ofleading blanks
      NLB = 1
  35  IF (LINE(NLB:NLB) .EQ. ' ') THEN
        NLB = NLB+1
        GOTO 35
      ENDIF

      J = 1
      DO 40 K = NLB,NLB+LEN(NAME)-1

C --- verify case insensitive label  
        NL = ICHAR(LINE(K:K))
        IF (NL.GT.90) NL = NL-32
        NN = ICHAR(NAME(J:J))
        IF (NN.GT.90) NN = NN-32
        IF (NL.NE.NN) THEN
C ---   no match, next line
          GOTO 10
        ENDIF
	  J = J+1 
40    CONTINUE

C --- found
      BACKSPACE (INF)

      RETURN
      END

C ----------------------------------------------------------------------
      REAL*8 FUNCTION LIMIT(P1,P2,X)
C ----------------------------------------------------------------------
C --- Limits X to values within the range between lower limit P1 and
C --- upper limit P2
C ----------------------------------------------------------------------
      IMPLICIT NONE

      REAL*8 P1,P2,X
C ----------------------------------------------------------------------
      LIMIT=X
      IF (X.GT.P2) LIMIT=P2
      IF (X.LT.P1) LIMIT=P1
      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE NRTODA (YEAR,DAYNR,MONTH,DAY)
C ----------------------------------------------------------------------
C     Date               : 11/91
C     Purpose            : Conversion of daynumber (YEAR,DAYNR) to    
C                          date (MONTH,DAY)
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER ILEAP,YEAR,MONTH,DAY,DAYNR
      INTEGER FDM365(12),FDM366(12)

      DATA FDM365/1,32,60,91,121,152,182,213,244,274,305,335/
      DATA FDM366/1,32,61,92,122,153,183,214,245,275,306,336/
C ----------------------------------------------------------------------

C --- is YEAR a leap-year?
      ILEAP=0
      IF (((MOD(YEAR,4).EQ.0).AND.(MOD(YEAR,100).NE.0)).OR.
     $     (MOD(YEAR,400).EQ.0)) ILEAP=1   

C --- calculate month, day
      IF (ILEAP.EQ.0) THEN
        MONTH=12
10      IF (DAYNR.LT.FDM365(MONTH)) THEN
          MONTH=MONTH-1
          GOTO 10
        ENDIF
        DAY=DAYNR-FDM365(MONTH)+1
      ELSEIF (ILEAP.EQ.1) THEN
        MONTH=12
20      IF (DAYNR.LT.FDM366(MONTH)) THEN
          MONTH=MONTH-1
          GOTO 20
        ENDIF
        DAY=DAYNR-FDM366(MONTH)+1
      ENDIF

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RADATA (DTYPE,INF,FNAME,LOGF,NAME,IVALUE,RVALUE,DVALUE,
     &  CVALUE,LENGTH,CITEMS,IVALS,ERROR)
C ----------------------------------------------------------------------
C     Date               : 06/02/1998
C     Purpose            :    
C     Subroutines called :
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER        DTYPE,INF,LOGF,LENGTH,IVALUE(LENGTH)
      REAL           RVALUE(LENGTH)
      REAL*8         DVALUE(LENGTH)
      CHARACTER *(*) CVALUE(LENGTH)
      CHARACTER *(*) NAME,FNAME

C ----------------------------------------------------------------------
C --- local
      INTEGER IP,I,J,BLN,ELN,ITEMS,CITEMS,IVALS,TMP

      CHARACTER LINE*120,BUFLIN*120

      LOGICAL   BLANK,COMMENT,BLMIN1,ERROR
C ----------------------------------------------------------------------
      ERROR = .false. 

      CALL FINDUNIT (80,TMP)
      OPEN (TMP,FILE='SWAP.TMP',STATUS='UNKNOWN')
      REWIND (TMP)

C --- read line from input file
10    READ (INF,'(A)') LINE
      CALL ANALIN (LINE, BLANK,COMMENT) 
      IF (BLANK.OR.COMMENT) GOTO 10

      CALL SKPLBL (LOGF,FNAME,LINE,NAME,IP) 

      CITEMS = 0

C --- analyze line
1000  I = IP+1

      BLN = I
      ELN = 120
      DO 12 J = I,120
        IF (LINE(J:J).EQ.'!') THEN
          ELN = J-1
          GOTO 13
        ENDIF
12    CONTINUE
13    CONTINUE

C --- area between BLN and ELN may contain data
      ITEMS = 0
      BLMIN1 = .TRUE.

      IF (DTYPE.LE.3) THEN
        DO 14 J = BLN,ELN
          IF (LINE(J:J).NE.' ') THEN
            IF (BLMIN1) THEN
              BLMIN1 = .FALSE.
              ITEMS  = ITEMS+1
            ENDIF
          ELSE
            BLMIN1 = .TRUE.
          ENDIF
14      CONTINUE
      ELSE
        DO 15 J = BLN,ELN
          IF (LINE(J:J).EQ.'''') THEN
            IF (BLMIN1) then
              BLMIN1 = .FALSE.
              ITEMS  = ITEMS+1
            ELSE
              BLMIN1 = .TRUE.
            ENDIF
          ENDIF
15      CONTINUE
      ENDIF           

C --- extract data
      IF (ITEMS.GT.0) THEN
        BUFLIN = ' '
        BUFLIN(BLN:ELN) = LINE(BLN:ELN)
        WRITE (TMP,'(A)') BUFLIN
        BACKSPACE (TMP)
        IF (DTYPE.EQ.1) THEN
          READ (TMP,*) (IVALUE(J), J = CITEMS+1,CITEMS+ITEMS)
        ELSEIF (DTYPE.EQ.2) THEN
          READ (TMP,*) (RVALUE(J), J = CITEMS+1,CITEMS+ITEMS)
        ELSEIF (DTYPE.EQ.3) THEN
          READ (TMP,*) (DVALUE(J), J = CITEMS+1,CITEMS+ITEMS)
        ELSEIF (DTYPE.EQ.4) THEN
          READ (TMP,*) (CVALUE(J), J = CITEMS+1,CITEMS+ITEMS)
        ENDIF
C ---   update total number of items
        CITEMS = CITEMS+ITEMS
        IF (CITEMS.EQ.LENGTH.OR.CITEMS.GE.IVALS) GOTO 30
      ENDIF

C --- next line...
20    READ (INF,'(A)') LINE
      CALL ANALIN (LINE,BLANK,COMMENT)
      IF (BLANK) GOTO 20
      IF (.NOT.COMMENT) THEN
       IP = 0
       GOTO 1000
      ENDIF

      IF (CITEMS.EQ.0) ERROR = .TRUE.

30    CLOSE (TMP,status='delete')

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RACHAR (INF,FNAME,LOGF,NAME,VALUE,LENGTH,CITEMS)
C ----------------------------------------------------------------------
C     Date               : 3/2/99
C     Purpose            : Reads an array of CHARACTER values from a 
C                          data file and carries out a range check on
C                          the returned values. The number of values on
C                          file is returned as CITEMS.
C     Subroutines called :
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER        INF,LOGF,LENGTH,CITEMS
      CHARACTER *(*) FNAME,NAME,VALUE(LENGTH)

C ----------------------------------------------------------------------
C --- local
      INTEGER DTYPE,IVALUE(1)
      REAL    RVALUE(1)
      REAL*8  DVALUE(1)
      LOGICAL ERROR
C ----------------------------------------------------------------------
      DTYPE = 4
      CALL RADATA (DTYPE,INF,FNAME,LOGF,NAME,IVALUE,RVALUE,DVALUE,VALUE,
     &  LENGTH,CITEMS,10000,ERROR)
      IF (ERROR) THEN
        WRITE (*,5) FNAME,NAME
        WRITE (LOGF,5) FNAME,NAME
        STOP 'Reading error - RADATA' 
5       Format (' Fatal error in file: ',A,' ,no items found for: ',A)
      ENDIF
      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RADOUR (INF,FNAME,LOGF,NAME,LOW,HIGH,VALUE,LENGTH,
     & CITEMS)
C ----------------------------------------------------------------------
C     Date               : 3/2/99
C     Purpose            : Reads an array of DOUBLE PRECISION values 
C                          from a data file and carries out a range 
C                          check on the returned values. The number of 
C                          values on file is returned as CITEMS.
C     Subroutines called : RADATA
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER        INF,LOGF,LENGTH,CITEMS
      REAL*8         VALUE(LENGTH)
      REAL           LOW,HIGH
      CHARACTER *(*) NAME
      CHARACTER*12 FNAME
C ----------------------------------------------------------------------
C --- local
      INTEGER DTYPE,IVALUE(1),I
      REAL    RVALUE(1)
      CHARACTER CVALUE(1)
      LOGICAL ERROR
C ----------------------------------------------------------------------
      DTYPE = 3
      CALL RADATA (DTYPE,INF,FNAME,LOGF,NAME,IVALUE,RVALUE,VALUE,CVALUE,
     &  LENGTH,CITEMS,10000,ERROR)
      IF (ERROR) THEN
        WRITE (*,5) FNAME,NAME
        WRITE (LOGF,5) FNAME,NAME
        STOP 'Reading error - RADATA' 
5       Format (' Fatal error in file: ',A,' ,no items found for: ',A)
      ENDIF

C --- range checking
      DO 100 I = 1,CITEMS
        IF (VALUE(I) .LT. LOW) THEN
          WRITE(LOGF,60) FNAME,NAME,VALUE(I),LOW
          WRITE(*,60) FNAME,NAME,VALUE(I),LOW
 60       FORMAT(' Fatal error in file: ',A,' ,the variable ',A,
     &           ' =',d10.4,' while lower limit =',d10.4)
          STOP 'Check log file'
        ELSE IF (VALUE(I) .GT. HIGH) THEN
          WRITE(LOGF,70) FNAME,NAME,VALUE(I),HIGH
          WRITE(*,70) FNAME,NAME,VALUE(I),HIGH
 70       FORMAT(' Fatal error in file: ',A,' ,the variable ',A,
     &           ' =',d10.4,' while upper limit =',d10.4)
          STOP 'Check log file'
        ENDIF
 100  CONTINUE

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RAINTR (INF,FNAME,LOGF,NAME,LOW,HIGH,VALUE,LENGTH,
     & CITEMS)
C ----------------------------------------------------------------------
C     Date               : 3/2/99
C     Purpose            : Reads an array of INTEGER values from a data
C                          file and carries out a range check on the 
C                          returned values. The number of values on file
C                          is returned as CITEMS.
C     Subroutines called : RADATA
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER        INF,LOGF,LENGTH,VALUE(LENGTH),CITEMS,LOW,HIGH 
      CHARACTER *(*) NAME
      CHARACTER*12   FNAME
C ----------------------------------------------------------------------
C --- local
      INTEGER DTYPE,I
      REAL    RVALUE(1)
      REAL*8  DVALUE(1)
c     CHARACTER *(*) CVALUE(LENGTH)
      CHARACTER CVALUE(1)
      LOGICAL ERROR
C ----------------------------------------------------------------------
      DTYPE = 1
      CALL RADATA (DTYPE,INF,FNAME,LOGF,NAME,VALUE,RVALUE,DVALUE,CVALUE,
     &  LENGTH,CITEMS,10000,ERROR)
      IF (ERROR) THEN
        WRITE (*,5) FNAME,NAME
        WRITE (LOGF,5) FNAME,NAME
        STOP 'Reading error - RADATA' 
5       Format (' Fatal error in file: ',A,' ,no items found for: ',A)
      ENDIF

C --- range checking
      DO 100 I = 1,CITEMS
        IF (VALUE(I) .LT. LOW) THEN
          WRITE(LOGF,60) FNAME,NAME,VALUE(I),LOW
          WRITE(*,60) FNAME,NAME,VALUE(I),LOW
 60       FORMAT(' Fatal error in file: ',A,' ,the variable ',A,
     &           ' =',I10,' while lower limit =',I10)
          STOP 'Check log file'
        ELSE IF (VALUE(I) .GT. HIGH) THEN
          WRITE(LOGF,70) FNAME,NAME,VALUE(I),LOW
          WRITE(*,70) FNAME,NAME,VALUE(I),LOW
 70       FORMAT(' Fatal error in file: ',A,' ,the variable ',A,
     &           ' =',I10,' while upper limit =',I10)
          STOP 'Check log file'
        ENDIF
100   CONTINUE

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RAREAR (INF,FNAME,LOGF,NAME,LOW,HIGH,VALUE,LENGTH,
     &  CITEMS)
C ----------------------------------------------------------------------
C     Date               : 3/2/99
C     Purpose            : Reads an array of REAL values from a data
C                          file and carries out a range check on the 
C                          returned values. The number of values on file
C                          is returned as CITEMS.
C     Subroutines called : RADATA
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER        INF,LOGF,LENGTH,CITEMS
      REAL           VALUE(LENGTH),LOW,HIGH
      CHARACTER *(*) NAME
      CHARACTER*12   FNAME
C ----------------------------------------------------------------------
C --- local
      INTEGER DTYPE,IVALUE(1),I
      REAL*8  DVALUE(1)
c     CHARACTER *(*) CVALUE(LENGTH)
      CHARACTER CVALUE(1)
      LOGICAL ERROR
C ----------------------------------------------------------------------
      DTYPE = 2
      CALL RADATA (DTYPE,INF,FNAME,LOGF,NAME,IVALUE,VALUE,DVALUE,CVALUE,
     &  LENGTH,CITEMS,10000,ERROR)
      IF (ERROR) THEN
        WRITE (*,5) FNAME,NAME
        WRITE (LOGF,5) FNAME,NAME
        STOP 'Reading error - RADATA' 
5       Format (' Fatal error in file: ',A,' ,no items found for: ',A)
      ENDIF

C --- range checking
      DO 100 I = 1,CITEMS 
        IF (VALUE(I) .LT. LOW) THEN
          WRITE(LOGF,60) FNAME,NAME,VALUE(I),LOW
          WRITE(*,60) FNAME,NAME,VALUE(I),LOW
 60       FORMAT(' Fatal error in file: ',A,', the variable ',A,
     &           ' =',e10.4,' while lower limit =',e10.4)
          STOP 'Check log file'
        ELSE IF (VALUE(I) .GT. HIGH) THEN
          WRITE(LOGF,70) FNAME,NAME,VALUE(I),HIGH
          WRITE(*,70) FNAME,NAME,VALUE(I),HIGH
 70       FORMAT(' Fatal error in file: ',A,', the variable ',A,
     &           ' =',e10.4,' while upper limit =',e10.4)
          STOP 'Check log file'
        ENDIF
 100  CONTINUE

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RFCHAR (INF,FNAME,LOGF,NAME,VALUE,LENGTH,IVALS)
C ----------------------------------------------------------------------
C     Date               : 3/2/1999
C     Purpose            : Reads a fixed number of elements into a
C                          CHARACTER array
C     Subroutines called :
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER        INF,LOGF,LENGTH,IVALS
      CHARACTER *(*) FNAME,NAME,VALUE(LENGTH)

C ----------------------------------------------------------------------
C --- local
      INTEGER DTYPE,IVALUE(1),CITEMS
      REAL    RVALUE(1)
      REAL*8  DVALUE(1)
      LOGICAL ERROR
C ----------------------------------------------------------------------
      DTYPE = 4
      CALL RADATA (DTYPE,INF,FNAME,LOGF,NAME,IVALUE,RVALUE,DVALUE,VALUE,
     &  LENGTH,CITEMS,IVALS,ERROR)   
      IF (ERROR) THEN
        WRITE (*,5) FNAME,NAME
        WRITE (LOGF,5) FNAME,NAME
        STOP 'Reading error - RADATA' 
5       Format (' Fatal error in file: ',A,' ,no items found for: ',A)
      ENDIF

      IF (CITEMS.LT.IVALS) THEN
        WRITE(LOGF,10) NAME
        WRITE(*,10) NAME
 10     FORMAT (' Not enough elements found in array: ',A)
        STOP 'Check log file'
      ENDIF

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RFDOUR (INF,FNAME,LOGF,NAME,LOW,HIGH,VALUE,LENGTH,
     &  IVALS)
C ----------------------------------------------------------------------
C     Date               : 3/2/99
C     Purpose            : Reads a fixed number of elements into a
C                          DOUBLE PRECISION array and carries out a 
C                          range check on the returned values
C     Subroutines called : RADATA
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER        INF,LOGF,LENGTH,IVALS
      REAL*8         VALUE(LENGTH)
      REAL           LOW,HIGH
      CHARACTER *(*) NAME
      CHARACTER*12   FNAME
C ----------------------------------------------------------------------
C --- local
      INTEGER DTYPE,IVALUE(1),I,CITEMS
      REAL    RVALUE(1)
      CHARACTER CVALUE(1)
      LOGICAL ERROR
C ----------------------------------------------------------------------
      DTYPE = 3
      CALL RADATA (DTYPE,INF,FNAME,LOGF,NAME,IVALUE,RVALUE,VALUE,CVALUE,
     &  LENGTH,CITEMS,IVALS,ERROR)   
      IF (ERROR) THEN
        WRITE (*,5) FNAME,NAME
        WRITE (LOGF,5) FNAME,NAME
        STOP 'Reading error - RADATA' 
5       Format (' Fatal error in file: ',A,' ,no items found for: ',A)
      ENDIF

      IF (CITEMS.LT.IVALS) THEN
        WRITE(LOGF,10) NAME
        WRITE(*,10) NAME
 10     FORMAT (' Not enough elements found in array: ',A)
        STOP 'Check log file'
      ENDIF

C --- range checking
      DO 100 I = 1,CITEMS
        IF (VALUE(I) .LT. LOW) THEN
          WRITE(LOGF,60) FNAME,NAME,VALUE(I),LOW
          WRITE(*,60) FNAME,NAME,VALUE(I),LOW
 60       FORMAT(' Fatal error in file: ',A,' ,the variable ',A,' ='
     &           ,d10.4,' while lower limit =',d10.4)
          STOP 'Check log file'
        ELSE IF (VALUE(I) .GT. HIGH) THEN
          WRITE(LOGF,70) NAME,VALUE(I),HIGH
          WRITE(*,70) NAME,VALUE(I),HIGH
 70       FORMAT(' Fatal error in file: ',A,' ,the variable ',A,' ='
     &           ,d10.4,' while upper limit =',d10.4)
          STOP 'Check log file'
        ENDIF
 100  CONTINUE

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RFINTR (INF,FNAME,LOGF,NAME,LOW,HIGH,VALUE,LENGTH,
     &  IVALS)
C ----------------------------------------------------------------------
C     Date               : 3/2/99
C     Purpose            : Reads a fixed number of elements into an
C                          INTEGER array and carries out a range check
C                          on the returned values
C     Subroutines called : RADATA
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER        INF,LOGF,LENGTH,VALUE(LENGTH),IVALS,LOW,HIGH 
      CHARACTER *(*) NAME
      CHARACTER*12   FNAME
C ----------------------------------------------------------------------
C --- local
      INTEGER DTYPE,I,CITEMS
      REAL    RVALUE(1)
      REAL*8  DVALUE(1)
c     CHARACTER *(*) CVALUE(LENGTH)
      CHARACTER CVALUE(1)
      LOGICAL ERROR
C ----------------------------------------------------------------------
      DTYPE = 1
      CALL RADATA (DTYPE,INF,FNAME,LOGF,NAME,VALUE,RVALUE,DVALUE,CVALUE,
     &  LENGTH,CITEMS,IVALS,ERROR)   
      IF (ERROR) THEN
        WRITE (*,5) FNAME,NAME
        WRITE (LOGF,5) FNAME,NAME
        STOP 'Reading error - RADATA' 
5       Format (' Fatal error in file: ',A,' ,no items found for: ',A)
      ENDIF

      IF (CITEMS.LT.IVALS) THEN
        WRITE(LOGF,10) NAME
        WRITE(*,10) NAME
 10     FORMAT (' Not enough elements found in array: ',A)
        STOP 'Check log file'
      ENDIF

C --- range checking
      DO 100 I = 1,CITEMS
        IF (VALUE(I) .LT. LOW) THEN
          WRITE(LOGF,60) FNAME,NAME,VALUE(I),LOW
          WRITE(*,60) FNAME,NAME,VALUE(I),LOW
 60       FORMAT(' Fatal error in file: ',A,' , the variable ',A,' ='
     &           ,I10,' while lower limit =',I10)
          STOP 'Check log file'
        ELSE IF (VALUE(I) .GT. HIGH) THEN
          WRITE(LOGF,70) FNAME,NAME,VALUE(I),LOW
          WRITE(*,70) FNAME,NAME,VALUE(I),LOW
 70       FORMAT(' Fatal error in file: ',A,' ,the variable ',A,' ='
     &           ,I10,' while upper limit =',I10)
          STOP 'Check log file'
        ENDIF
100   CONTINUE

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RFREAR (INF,FNAME,LOGF,NAME,LOW,HIGH,VALUE,LENGTH,
     &  IVALS)
C ----------------------------------------------------------------------
C     Date               : 3/2/99
C     Purpose            : Reads a fixed number of elements into a
C                          REAL array and carries out a range check
C                          on the returned values
C     Subroutines called : RADATA
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER        INF,LOGF,LENGTH,IVALS
      REAL           VALUE(LENGTH),LOW,HIGH
      CHARACTER *(*) NAME
      CHARACTER*12   FNAME
C ----------------------------------------------------------------------
C --- local
      INTEGER DTYPE,IVALUE(1),I,CITEMS

      REAL*8  DVALUE(1)
c     CHARACTER *(*) CVALUE(LENGTH)
      CHARACTER CVALUE(1)
      LOGICAL ERROR
C ----------------------------------------------------------------------
      DTYPE = 2
      CALL RADATA (DTYPE,INF,FNAME,LOGF,NAME,IVALUE,VALUE,DVALUE,CVALUE,
     &  LENGTH,CITEMS,IVALS,ERROR)   
      IF (ERROR) THEN
        WRITE (*,5) FNAME,NAME
        WRITE (LOGF,5) FNAME,NAME
        STOP 'Reading error - RADATA' 
5       Format (' Fatal error in file: ',A,' ,no items found for: ',A)
      ENDIF

      IF (CITEMS.LT.IVALS) THEN
        WRITE(LOGF,10) NAME
        WRITE(*,10) NAME
 10     FORMAT (' Not enough elements found in array: ',A)
        STOP 'Check log file'
      ENDIF

C --- range checking
      DO 100 I = 1,CITEMS 
        IF (VALUE(I) .LT. LOW) THEN
          WRITE(LOGF,60) FNAME,NAME,VALUE(I),LOW
          WRITE(*,60)FNAME,NAME,VALUE(I),LOW
 60       FORMAT(' Fatal error in file: ',A,' ,the variable ',A,' ='
     &           ,e10.4,' while lower limit =',e10.4)
          STOP 'Check log file'
        ELSE IF (VALUE(I) .GT. HIGH) THEN
          WRITE(LOGF,70) FNAME,NAME,VALUE(I),HIGH
          WRITE(*,70) FNAME,NAME,VALUE(I),HIGH
 70       FORMAT(' Fatal error in file: ',A,' ,the variable ',A,' ='
     &           ,e10.4,' while upper limit =',e10.4)
          STOP 'Check log file'
        ENDIF
 100  CONTINUE

      RETURN
      END

C ----------------------------------------------------------------------
       SUBROUTINE RSCHAR (INF,FNAME,LOGF,XNAME,VALUE)
C ----------------------------------------------------------------------
C     Date               : 3/2/1999
C     Purpose            : reads a single CHARACTER value from a data
C                          file    
C     Subroutines called :
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER INF,LOGF

      CHARACTER *(*) FNAME,XNAME,VALUE
C ----------------------------------------------------------------------
C --- local
      INTEGER   DTYPE,IVALUE
      REAL      RVALUE
      REAL*8    DVALUE
      LOGICAL   ERROR 
C ----------------------------------------------------------------------
      DTYPE = 4
      CALL RSDATA (INF,FNAME,LOGF,XNAME,IVALUE,RVALUE,DVALUE,VALUE,
     &  DTYPE,ERROR)
      IF (ERROR) THEN
        WRITE (*,5) FNAME,XNAME
        WRITE (LOGF,5) FNAME,XNAME
        STOP 'Reading error - RSDATA' 
5       Format (' Fatal error in file: ',A,' ,no items found for: ',A)
      ENDIF

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RSDATA(INF,FNAME,LOGF,NAME,IVALUE,RVALUE,DVALUE,
     & CVALUE,DTYPE,ERROR)
C ----------------------------------------------------------------------
C     Date               : 3/2/99
C     Purpose            :    
C     Subroutines called :
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER    DTYPE,INF,LOGF,IVALUE
      REAL       RVALUE
      REAL*8     DVALUE

      CHARACTER *(*) FNAME,NAME,CVALUE 
      LOGICAL    ERROR
C ----------------------------------------------------------------------
C --- local
      INTEGER    I,BLN,ELN,IP,TMP
      CHARACTER  LINE*80,BUFLIN*80
      LOGICAL    BLANK,COMMENT
C ----------------------------------------------------------------------
      CALL FINDUNIT (80,TMP)
      OPEN (TMP,FILE='SWAP.TMP',STATUS='UNKNOWN')
      REWIND (TMP)

C      CALL JMPLBL (INF,'SHIT',LOGF,NAME)

C --- read line from input file
  25  READ(INF,'(A)') LINE
      CALL ANALIN (LINE,BLANK,COMMENT) 

C --- if comment line or blank line read next line 
      IF (BLANK.OR.COMMENT) GOTO 25

      CALL SKPLBL (LOGF,FNAME,LINE,NAME,IP)  

C --- find start of value: BLN
      I = IP+1
 50   IF (LINE(I:I) .EQ. ' ') THEN
        I = I+1
        IF (I.EQ.81) THEN
          ERROR = .TRUE.
          CLOSE (TMP,status='delete')
          RETURN
        ENDIF
        GOTO 50
      ELSE IF (LINE(I:I).EQ.'!') THEN
        ERROR = .TRUE.
        CLOSE (TMP,status='delete')
        RETURN
      ENDIF

      BLN = I

C --- find end of value: ELN
      I = I+1
 
      IF (DTYPE.LE.3) then
 55     IF (LINE(I:I) .NE. ' ' .AND. LINE(I:I) .NE. '!') THEN
          I = I+1
          GOTO 55
        ENDIF
      ELSE
56      IF (LINE(I:I).NE.'''') THEN
          I = I+1
          GOTO 56      
        ENDIF     
      ENDIF
            
      ELN = I-1
      IF (DTYPE.EQ.4) ELN = ELN+1

C --- extract data 
      BUFLIN = ' '
      BUFLIN(BLN:ELN) = LINE(BLN:ELN)
      WRITE (TMP,'(A)') BUFLIN
      BACKSPACE (TMP)

      IF (DTYPE .EQ. 1) READ (TMP,*) IVALUE
      IF (DTYPE .EQ. 2) READ (TMP,*) RVALUE
      IF (DTYPE .EQ. 3) READ (TMP,*) DVALUE
      IF (DTYPE .EQ. 4) READ (TMP,*) CVALUE

      CLOSE (TMP,status='delete')

      RETURN
      END 

C ----------------------------------------------------------------------
      SUBROUTINE RSDOUR(INF,FNAME,LOGF,NAME,LOW,HIGH,VALUE)
C ----------------------------------------------------------------------
C     Date               : 3/2/99
C     Purpose            : reads a single DOUBLE PRECISION value from a 
C                          data file and carries our a range check on 
C                          the returned value
C     Subroutines called : RSDATA
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   INF,LOGF

      REAL*8    VALUE
      REAL      LOW,HIGH
      CHARACTER *(*) NAME     
      CHARACTER*12   FNAME
C ----------------------------------------------------------------------
C --- local
      INTEGER   DTYPE,IVALUE
      REAL      RVALUE
      CHARACTER CVALUE*80
      LOGICAL   ERROR
C ----------------------------------------------------------------------
      DTYPE = 3
      CALL RSDATA (INF,FNAME,LOGF,NAME,IVALUE,RVALUE,VALUE,CVALUE,
     &  DTYPE,ERROR)
      IF (ERROR) THEN
        WRITE (*,5) FNAME,NAME
        WRITE (LOGF,5) FNAME,NAME
        STOP 'Reading error - RSDATA' 
5       Format (' Fatal error in file: ',A,' ,no items found for: ',A)
      ENDIF

      IF (VALUE .LT. LOW) THEN
        WRITE(LOGF,60) FNAME,NAME,VALUE,LOW
        WRITE(*,60) FNAME,NAME,VALUE,LOW
 60     FORMAT(' Fatal error in file: ',A,', the variable ',A,' ='
     &         ,D10.4,' while lower limit =',D10.4)
        STOP 'Check log file' 
      ELSE IF (VALUE .GT. HIGH) THEN
        WRITE(LOGF,70) FNAME,NAME,VALUE,HIGH
        WRITE(*,70) FNAME,NAME,VALUE,HIGH
 70     FORMAT(' Fatal error in file: ',A,', the variable ',A,' ='
     &         ,D10.4,' while upper limit =',D10.4)
        STOP 'Check log file'
      ENDIF

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RSINTR(INF,FNAME,LOGF,NAME,LOW,HIGH,VALUE)
C ----------------------------------------------------------------------
C     Date               : 3/2/99
C     Purpose            : reads a single INTEGER value from a data file
C                          and carries out a range check on the returned
C                          value
C     Subroutines called : RSDATA
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   INF,LOGF,VALUE,LOW,HIGH
      CHARACTER *(*) NAME     
      CHARACTER*12   FNAME
C ----------------------------------------------------------------------
C --- local
      INTEGER   DTYPE
      REAL      RVALUE
      REAL*8    DVALUE

      CHARACTER CVALUE*80
      LOGICAL   ERROR
C ----------------------------------------------------------------------
      DTYPE = 1
      CALL RSDATA (INF,FNAME,LOGF,NAME,VALUE,RVALUE,DVALUE,CVALUE,
     &  DTYPE,ERROR)
      IF (ERROR) THEN
        WRITE (*,5) FNAME,NAME
        WRITE (LOGF,5) FNAME,NAME
        STOP 'Reading error - RSDATA' 
5       Format (' Fatal error in file: ',A,' ,no items found for: ',A)
      ENDIF

      IF (VALUE .LT. LOW) THEN
        WRITE(LOGF,60) FNAME,NAME,VALUE,LOW
        WRITE(*,60) FNAME,NAME,VALUE,LOW
 60     FORMAT(' Fatal error in file: ',A,', the variable ',A,' =',I10,
     &         ' while lower limit =',I10)
        STOP 'Check log file'
      ELSE IF (VALUE .GT. HIGH) THEN
        WRITE(LOGF,70) FNAME,NAME,VALUE,HIGH
        WRITE(*,70) FNAME,NAME,VALUE,HIGH
 70     FORMAT(' Fatal error in file: ',A,', the variable ',A,' =',I10,
     &         ' while upper limit =',I10)
        STOP 'Check log file'
      ENDIF

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RSREAR(INF,FNAME,LOGF,NAME,LOW,HIGH,VALUE)
C ----------------------------------------------------------------------
C     Date               : 3/2/99
C     Purpose            : reads a single INTEGER value from a data file
C                          and carries our a range check on the returned
C                          value
C     Subroutines called : RSDATA
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER INF,LOGF

      REAL      VALUE,LOW,HIGH
      CHARACTER *(*) NAME     
      CHARACTER*80 CVALUE
      CHARACTER*12 FNAME
C ----------------------------------------------------------------------
C --- local
      INTEGER   DTYPE,IVALUE
      REAL*8    DVALUE
      LOGICAL   ERROR
C ----------------------------------------------------------------------
      DTYPE = 2
      CALL RSDATA (INF,FNAME,LOGF,NAME,IVALUE,VALUE,DVALUE,CVALUE,
     &  DTYPE,ERROR)
      IF (ERROR) THEN
        WRITE (*,5) FNAME,NAME
        WRITE (LOGF,5) FNAME,NAME
        STOP 'Reading error - RSDATA' 
5       Format (' Fatal error in file: ',A,' ,no items found for: ',A)
      ENDIF

      IF (VALUE .LT. LOW) THEN
        WRITE(LOGF,60) FNAME,NAME,VALUE,LOW
        WRITE(*,60) FNAME,NAME,VALUE,LOW
 60     FORMAT(' Fatal error in file: ',A,', the variable ',A,' ='
     &         ,e10.4,' while lower limit =',e10.4)
        STOP 'Check log file'
      ELSE IF (VALUE .GT. HIGH) THEN
        WRITE(LOGF,70) FNAME,NAME,VALUE,HIGH
        WRITE(*,70) FNAME,NAME,VALUE,HIGH
 70     FORMAT(' Fatal error in file: ',A,', the variable ',A,' ='
     &         ,e10.4,' while upper limit =',e10.4)
        STOP 'Check log file'
      ENDIF

      RETURN
      END

C-----------------------------------------------------------------------
      SUBROUTINE SHIFTL(STRING)
C-----------------------------------------------------------------------
C     Date               : 26/10/95
C     Purpose            : Shift STRING to the left as far as possible
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   I,LS,empty

      CHARACTER *(*) STRING
C-----------------------------------------------------------------------
      ls  = len (string)

      empty = 0
      i     = 1
30    if (string(i:i).eq.' ') then
        empty = empty+1
        i     = i+1
        if (i.le.ls) goto 30
      endif

      if (empty.gt.0) then
        do 40 i = 1,ls-empty
          string(i:i) = string(i+empty:i+empty)
40      continue
        do 50 i = ls-empty+1,ls
          string(i:i) = ' '
50      continue
      endif

      return
      end 

C-----------------------------------------------------------------------
      SUBROUTINE SHIFTR(STRING)
C-----------------------------------------------------------------------
C     Date               : 26/10/95 
C     Purpose            : Shift STRING to the right as far as possible
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   i,ls,empty

      CHARACTER *(*) STRING
C-----------------------------------------------------------------------
      LS  = LEN (STRING)

      empty = 0
      i     = ls
30    if (string(i:i).eq.' ') then
        empty = empty+1
        i     = i-1
        if (i.ge.1) goto 30
      endif

      if (empty.gt.0) then
        do 40 i = ls,empty+1,-1
          string(i:i) = string(i-empty:i-empty)
40      continue
        do 50 i = 1,empty
          string(i:i) = ' '
50      continue
      endif

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE SKPLBL (LOGF,FNAME,LINE,NAME,POSIT)  
C ----------------------------------------------------------------------
C     Date               : 27/10/1997
C     Purpose            : verify label and return position of '='
C     Subroutines called :
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   LOGF,POSIT,NL,NN
      CHARACTER LINE*80 
      CHARACTER *(*) FNAME,NAME
C ----------------------------------------------------------------------
C --- local
      INTEGER NLB,J,K
C ----------------------------------------------------------------------
C --- count Number of Leading Blanks
      NLB = 1
2     IF (LINE(NLB:NLB).EQ.' ') THEN
        NLB = NLB+1
	  GOTO 2
	ENDIF   	 

      J = 1
      DO 10 K = NLB,NLB+LEN(NAME)-1

C ---   verify case insensitive label  
        NL = ICHAR(LINE(K:K))
        IF (NL.GT.90) NL = NL-32
        NN = ICHAR(NAME(J:J))
        IF (NN.GT.90) NN = NN-32
        IF (NL.NE.NN) THEN
          WRITE (*,5) FNAME,NAME
          WRITE (LOGF,5) FNAME,NAME
          STOP 'Reading error - SKPLBL' 
5         Format(' Fatal error in file: ',A,
     &           ' cannot read parameter: ',A)
        ENDIF 
        J = J+1
10    CONTINUE

      K = NLB+LEN(NAME)-1
20    K = K+1
      IF (K.GT.80) THEN
          WRITE (*,5) FNAME,NAME
          WRITE (LOGF,5) FNAME,NAME
          STOP 'Reading error - SKPLBL' 
      ENDIF
      IF (LINE(K:K).NE.'=') GOTO 20

      POSIT = K
                             
      RETURN
      END

C ----------------------------------------------------------------------
      INTEGER FUNCTION STEPNR(ARRAY,LENGTH,X)
C ----------------------------------------------------------------------
C --- Find stepnumber 
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER I,LENGTH
      REAL*8  X,ARRAY(LENGTH)
C ----------------------------------------------------------------------
      IF (ARRAY(1).GE.X) GOTO 30

      DO 10 I = 2,LENGTH
        IF (ARRAY(I).GT.X)          GOTO 20 
        IF (ARRAY(I).LT.ARRAY(I-1)) GOTO 20
10    CONTINUE  
C --- array fully filled, argument larger than last X in array
      STEPNR = LENGTH
      RETURN
C --- array partly filled, argument larger than last X in array or
C --- argument between first and last X in table
20    STEPNR = I-1
      RETURN
C --- argument less or equal to first X in array
30    STEPNR = 1
      RETURN
      END


