************************************************************************
** FILE:
**    SWCOMLIB.FOR - common program code for SWAP and SWAPS programs
**
**    Paired #ifdef and #endif preprocessor statements have been used
**    to enter SWAP or SWAPS specific code in the subroutines below.
**    The appropriate model is selected by defining SWAP or SWAPS at
**    preprocessing before compilation.  
**    SWAPS specific code will thus not be compiled in SWAP and vice versa.
**
**    How to achieve this:
**    - Under UNIX, use the -fpp and -DSWAP(S) flags as compiler options.
**    - Under Windows, use the fpp and /DSWAP(S) flags for compilation.
**      In Visual Fortrango to the PROJECT - SETTINGS - FORTRAN menu, go to
**      the preprocessor category, fill in SWAP or SWAPS in the 
**      "Predefined preprocessor symbols" field and enable "use FPP".
**
** FILE IDENTIFICATION:
**    $Id: SWCOMLIB.FOR 1.12 2000/02/04 16:38:45 Kroes Exp $
** DESCRIPTION:
**    This file contains subroutines of SWAP which are similar in SWAPS.
**    The program units in this file are of the 3rd level and are sorted
**    alphabetically.
**
**    The following program units are in this file:
**        BOCOBOT, BOCODRB, BOCODRE, BOCOTOP, CALCGWL, CRACK, DIVDRA,
**        DMCNODE, FLUXES, HCONODE, HEADCALC, INITSOL, OUTDRF, OUTSWB,
**        PRHNODE, QHTAB, SLTBAL, SOLUTE, THENODE, TRIDAG, UPDATE, 
**        WATCON, WBALLEV, WLEVBAL, WLEVST, WSTLEV
************************************************************************

C ----------------------------------------------------------------------
      SUBROUTINE BOCOBOT (SWBOTB,GWL,GWLTAB,T,H,Z,LAYER,
     & FLGENU,COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS,THETHI,
     & DMCH,DMCC,KTABLE,KSE99,KSAT,FMOBIL,THETA,DIMOCA,K,KMEAN,
     & QBOT,QBOTAB,AQAVE,AQAMP,AQOMEG,AQTAMX,RIMLAY,COFQHA,COFQHB,
     & HGITAB,NUMNOD,SWMOBI,FM1,PF1,SLOPFM,NODGWL,SWDRA,QDRTOT,
     & QDRA,NRLEVS,QDRAIN,HDRAIN,SHAPE,GWLINP)
C ----------------------------------------------------------------------
C     Date               : 4/2/2000
C     Purpose            : determines boundary conditions at the bottom
C                          of the soil profile at time T
C     Subroutines called : -
C     Functions called   : AFGEN,THENODE,DMCNODE,HCONODE
C     File usage         : -
C     SWAP/SWAPS differences: Use timestep T+1 instead of T for inter-
C                             polation GWL
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C ---  global
       INTEGER SWBOTB,LAYER(MACP),THETLO(MAHO),THETHI(MAHO),NUMNOD
       INTEGER SWMOBI,NODGWL,SWDRA,NRLEVS

       REAL*8  GWL,GWLTAB(732),T,Z(MACP),QBOT,QBOTAB(732),HGITAB(50)
       REAL*8  FMOBIL(MACP),FM1(MAHO),PF1(MAHO),SLOPFM(MAHO),QDRTOT
       REAL*8  AFGEN,AQAMP,AQAVE,AQOMEG,AQTAMX,COFQHA,COFQHB,RIMLAY
       REAL*8  QDRA(5,MACP),QDRAIN(5),HDRAIN,SHAPE,GWLINP

       REAL*8  H(MACP),KMEAN(MACP+1),THENODE,DMCNODE,HCONODE
       REAL*8  COFGEN(8,MAHO),HTABLE(MAHO,99),KTABLE(MAHO,99)
       REAL*8  ALFAMG(MACP),THETAR(MACP),THETAS(MACP),DMCC(MAHO,99)
       REAL*8  DMCH(MAHO,99)
       REAL*8  KSE99(MAHO),KSAT(MAHO),THETA(MACP),DIMOCA(MACP),K(MACP+1)

       LOGICAL FLGENU(MAHO)
C ----------------------------------------------------------------------
C --- local
      INTEGER  node,i,lay,level

      REAL*8   deepgw,hresis,hmax,qdrnode(macp),fluxtotal,ratio,gwlmean
C ----------------------------------------------------------------------
C --- interpolation between daily values of given groundwaterlevel
      if (swbotb.eq.1) then
        if (nodgwl.le.numnod) then
          hresis = gwl - real(h(numnod)) - real(z(numnod))
          hmax   = 0.5*(-z(numnod))
          hresis = min(hresis,hmax)
          hresis = max(hresis,-hmax)
        else
          hresis = 0.0
        endif

C ----------------------------------------------------------------------
CMS 23-JUL-99 In SWAPS: assume input GWL measured at 00:00
CMS                     thus interpolate using T+1
CMS           in SWAP:  input GWL set at 24:00 by using T
C --- Jos: Please check if this needs to be modified to T+1 in SWAP
#ifdef SWAPS
        gwlinp = AFGEN (GWLTAB,732,T+1)
#endif

#ifdef SWAP
        gwlinp = AFGEN (GWLTAB,732,T)
#endif
C ----------------------------------------------------------------------

c --- conversion to pressure head at bottom of profile
        h(numnod)      = gwlinp-z(numnod)-hresis
      endif

C --- interpolation between daily values of given flux
      if (swbotb.eq.2) qbot = AFGEN (QBOTAB,732,T)

C --- seepage or infiltration from/to deep groundwater
      if (swbotb.eq.3) then
        gwlmean = hdrain + shape*(gwl - hdrain)
        deepgw = aqave + aqamp*cos(aqomeg*(t - aqtamx))
        qbot   = (deepgw - gwlmean)/rimlay
      endif

C --- flux calculated as function of h
      if (swbotb.eq.4) qbot = cofqha*exp(cofqhb*abs(gwl))

C --- interpolation between daily values of given pressurehead
      if (swbotb.eq.5) then
        h(numnod)      = AFGEN (HGITAB,50,T)
        lay            = layer(numnod)
        i              = numnod
        if (SWMOBI.eq.1) then
          if (h(i).gt.-1.0d0) then
            fmobil(i) = fm1(lay)+(0.0-pf1(lay))*slopfm(lay)
          else
            fmobil(i) = fm1(lay)+(log10(abs(real(h(i))))-pf1(lay))*
     &                  slopfm(lay)
          endif
          fmobil(i) = min (1.0d0,fmobil(i))
          fmobil(i) = max (0.3d0,fmobil(i))
          thetar(i) = cofgen(1,lay)*fmobil(i)
          thetas(i) = cofgen(2,lay)*fmobil(i)
        endif
        theta(numnod)  = thenode (numnod,h(numnod),LAYER,FLGENU,
     &                   COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
        dimoca(numnod) = dmcnode (numnod,h(numnod),LAYER,FLGENU,COFGEN,
     &                   THETLO,THETHI,DMCH,DMCC,THETAS,THETAR,
     &                   ALFAMG,SWMOBI,FM1,PF1,SLOPFM,HTABLE,THETA)
        k(numnod)      = hconode (numnod,theta(numnod),LAYER,FLGENU,
     &                   COFGEN,THETAR,THETAS,THETLO,THETHI,KTABLE,
     &                   KSE99,KSAT,SWMOBI,FMOBIL)
        kmean(numnod)  = 0.5*(k(numnod)+k(numnod-1))
      endif

C --- zero flux at the bottom
      if (swbotb.eq.6) qbot = 0.0

C --- free drainage
      if (swbotb.eq.7) qbot = -1.0 * kmean(numnod+1)

C --- lysimeter with free drainage
      if (swbotb.eq.8) qbot = 0.0

C --- restrict bottom and drainage flux if necessary
      if (gwl.lt.998.0d0 .and. nodgwl.lt.numnod .and. swbotb.le.6) then

        if (swdra.ge.1) then

C --- calculate total drainage flux of each node
          do 230 node = 1,numnod
            qdrnode(node) = 0.0
 230      continue
          qdrtot = 0.0

          do 250 node = nodgwl,numnod
            do 270 level = 1,nrlevs
              qdrnode(node) = qdrnode(node) + qdra(level,node)
 270        continue
            qdrtot = qdrtot + qdrnode(node)
 250      continue
        endif
        fluxtotal = qbot - qdrtot

C --- search for less conductive layers and restrict fluxes if necessary
        do 300 node = nodgwl,numnod
          lay = layer(node)
          if (fluxtotal .lt. -ksat(lay)) then
            ratio = -ksat(lay)/fluxtotal*0.8
            fluxtotal = 0.0
            do 320 i = node,numnod
              do 340 level=1,nrlevs
                qdra(level,i) = ratio * qdra(level,i)
 340          continue
              qdrnode(i) = ratio * qdrnode(i)
              fluxtotal = fluxtotal - qdrnode(i)
 320        continue
            qbot = ratio * qbot
            fluxtotal = fluxtotal + qbot + qdrnode(node)
          else
            fluxtotal = fluxtotal + qdrnode(node)
          endif
 300    continue
      endif      

C --- in case of drainage always calculate total drainage flux
      if (swdra.ge.1) then
        do 350 level = 1,nrlevs
          qdrain(level) = 0.0
 350    continue
        qdrtot = 0.0
        do 360 level = 1,nrlevs
          do 370 node = 1,numnod
             qdrain(level) = qdrain(level) + qdra(level,node)
 370      continue
          qdrtot = qdrtot + qdrain(level)
 360    continue
      endif

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE BOCODRB (DRAMET,GWL,ZBOTDR,POND,BASEGW,L,QDRAIN,IPOS,
     &  KHTOP,KHBOT,KVTOP,KVBOT,ENTRES,WETPER,ZINTF,GEOFAC,SWDTYP,
     &  OWLTAB,T,SWALLO,DRARES,INFRES,QDRTAB,NRLEVS)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : determines the values of the boundary       
C                          conditions at the lateral side of the soil  
C                          profile : basic drainage routine                    
C     Subroutines called : -                                           
C     Functions called   : AFGEN                                     
C     File usage         : -                                           
C     SWAP/SWAPS differences: None
C ----------------------------------------------------------------------  
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER DRAMET,IPOS,SWDTYP(5),SWALLO(5),NRLEVS

      REAL*8  GWL,ZBOTDR(5),POND,BASEGW(2),L(5),QDRAIN(5),KHTOP,KHBOT
      REAL*8  KVTOP,KVBOT,ENTRES,WETPER(5),ZINTF,GEOFAC,T
      REAL*8  DRARES(5),INFRES(5),QDRTAB(50),AFGEN,OWLTAB(5,2*MAOWL)
C ----------------------------------------------------------------------
C --- local
      INTEGER i,lev

      REAL*8  dh,zimp,dbot,pi,totres,x,fx,eqd,rver,rhor,rrad
      REAL*8  TEMPTAB(2*MAOWL)

      LOGICAL FLDRY

      PARAMETER (pi=3.14159d0)
C ----------------------------------------------------------------------

C --- drainage flux calculated according to Hooghoudt or Ernst
      if (dramet.eq.2) then
        dh    = gwl-zbotdr(1)
        if (abs(gwl).lt.1.0d-7) dh = dh + pond

C --- contributing layer below drains limited to 1/4 L
        zimp  = max (basegw(1),zbotdr(1)-0.25*L(1))
        dbot  = (zbotdr(1)-ZIMP)  
        if (dbot.lt.0.0) STOP 'Error - Bocodrb: DBOT negative'

C --- no infiltration allowed
        if (dh.lt.1.0d-10) then
          qdrain(1) = 0.0
          return
        endif

C --- case 1: homogeneous, on top of impervious layer
        if (ipos.eq.1) then

C --- calculation of drainage resistance and drainage flux
          totres    = l(1)*l(1)/(4*khtop*abs(dh)) + entres
          qdrain(1) = dh/totres 

C --- case 2,3: in homogeneous profile or at interface of 2 layers
        elseif (ipos.eq.2.or.ipos.eq.3) then

C --- calculation of equivalent depth
          x = 2*pi*dbot/l(1)
          if (x.gt.0.5d0) then
            fx = 0.0
            do 10 i = 1,5,2
10          fx = fx + (4*exp(-2*i*x))/(i*(1.0d0-exp(-2*i*x)))
            eqd = pi*l(1)/8 / (log(l(1)/wetper(1))+fx)
          else
            if (x.lt.1.0d-6) then
              eqd = dbot
            else
              fx  = pi**2/(4*x) + log(x/(2*pi))
              eqd = pi*l(1)/8 / (log(l(1)/wetper(1))+fx)
            endif
          endif 
          if (eqd.gt.dbot) eqd = dbot

C --- calculation of drainage resistance & drainage flux
          if (ipos.eq.2) then
            totres = l(1)*l(1)/(8*khtop*eqd+4*khtop*abs(dh)) + entres 
          elseif (ipos.eq.3) then
            totres = l(1)*l(1)/(8*khbot*eqd+4*khtop*abs(dh)) + entres
          endif
          qdrain(1) = dh/totres 

C --- case 4: drain in bottom layer
        elseif (ipos.eq.4) then
          if (zbotdr(1).gt.zintf) STOP 'Error - check ZINTF and ZBOTDR' 
          rver      = max (gwl-zintf,0.0d0) / kvtop +
     &                (min (zintf,gwl) -zbotdr(1))/kvbot
          rhor      = l(1)*l(1)/(8*khbot*dbot) 
          rrad      = l(1)/(pi*sqrt(khbot*kvbot)) * log(dbot/wetper(1))
          totres    = rver+rhor+rrad+entres
          qdrain(1) = dh/totres 

C --- case 5 : drain in top layer
        elseif (ipos.eq.5) then
          if (zbotdr(1).lt.zintf) STOP 'Error - check ZINTF and ZBOTDR'
          rver      = (gwl-zbotdr(1))/kvtop
          rhor      = l(1)*l(1) / (8*khtop*(zbotdr(1)-zintf) +
     &                             8*khbot*(zintf-zimp))
          rrad      = l(1)/(pi*sqrt(khtop*kvtop))*log((geofac*
     &                (zbotdr(1)-zintf))/wetper(1))
          totres    = rver+rhor+rrad+entres
          qdrain(1) = dh/totres
        endif

C --- drainage flux calc. using given drainage/infiltration resistance
      elseif (dramet.eq.3) then
        do 1000 LEV = 1,NRLEVS
          FLDRY = .FALSE.
          if (swdtyp(lev).eq.1) then
            dh = gwl-zbotdr(lev)
          elseif (swdtyp(lev).eq.2) then

C --- first copy to 1-dimensional table TEMPTAB
            do 12 I = 1,2*MAOWL 
 12         TEMPTAB(I) = OWLTAB(LEV,I)
            if (afgen (TEMPTAB,2*MAOWL,t+1.0d0)-zbotdr(lev).lt.1.0d-3) 
     &      FLDRY = .TRUE.  
            dh = gwl- afgen (TEMPTAB,2*MAOWL,t+1.0d0)
            if (FLDRY) dh = gwl-zbotdr(lev)
          endif
          if (abs(gwl).lt.1.0d-7) dh = dh + pond
          if (dh.ge.0.0) then

C --- drainage
            qdrain(lev) = dh/drares(lev)
            if (swallo(lev).eq.2) qdrain(lev) = 0.0
          else

C --- infiltration
            qdrain(lev) = dh/infres(lev)
            if (swallo(lev).eq.3.or.FLDRY) qdrain(lev) = 0.0
          endif

 1000   continue

C --- drainage flux from table with gwlevel - flux data pairs
      elseif (dramet.eq.1) then
        qdrain(1) = afgen (qdrtab,50,abs(gwl))
      endif

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE BOCODRE (SWSEC,SWSRF,NRSRF,NRPRI,GWL,ZBOTDR,TALUDR,
     & WIDTHR,POND,PONDMX,SWDTYP,DT,WLS,WLP,DRAINL,L,RDRAIN,RINFI,
     & RENTRY,REXIT,GWLINF,WETPER,QDRAIN,QDRD,T,IMPEND,NMPER,WSCAP,WST)
C ----------------------------------------------------------------------
C     Date               : 11/8/99
C     Purpose            :        
C --- Calculate drainage/infiltration fluxes for all levels: qdrain(level).
C --- Summate fluxes to secondary system: qdrd
C --- Given present storage (wst), qdrd and wscap(imper), check if the
C --- system falls dry. If so reduce drainage fluxes proportionally in 
C --- such a way that total drainage flux equals available amount 
C --- (= wst + wscap).
C     Subroutines called : -                                           
C     Functions called   : -                                     
C     File usage         : -                                           
C     SWAP/SWAPS differences: - SWAPS uses variable "swl"
C                             - Error handling
C ----------------------------------------------------------------------  
      IMPLICIT NONE

C --- global
      INTEGER SWSEC,SWSRF,NRSRF,NRPRI,SWDTYP(5),IMPEND(10),NMPER

      REAL*8  GWL,ZBOTDR(5),TALUDR(5),WIDTHR(5),POND,PONDMX,WLS,WLP
      REAL*8  DRAINL(5),L(5),WETPER(5),RINFI(5),REXIT(5),GWLINF(5),DT,T
      REAL*8  RDRAIN(5),RENTRY(5),QDRAIN(5),QDRD,WST
      REAL*8  WSCAP(10)

C ----------------------------------------------------------------------
C --- local
      INTEGER level,imper

#ifdef SWAPS
C --- SWAPS variable swl, defined and initialised
      INTEGER swl
#endif

      REAL*8  dh,qdrdm,qdratio,swdepth,swexbrd,dvmax,wstmax,wl,rd,re

      save
C ----------------------------------------------------------------------

#ifdef SWAPS
      swl = 99
#endif

C --- summate fluxes for use by swballev and swlevbal
      qdrd = 0.0
      do 500 level = 1,nrsrf

C --- surface water level
        if (swsrf.ge.2) then
          if (level.gt.nrpri) then
            wl = wls
          else
            wl = wlp
          endif
        endif

C --- drainage fluxes are set to zero if both groundwater level and surface 
C     water level are above ponding sill (so the nonzero drainage flux 
C     is only computed if either the gwl or the wl is below pondmx)
        if (wl .lt. pondmx .or. gwl .lt. pondmx) then

C --- channel is active medium if either groundwater or surface water
C     level is above channel bottom
          if (gwl.gt.(zbotdr(level)+0.001d0) .or. 
     &        wl.gt.(zbotdr(level)+0.001d0)) then
            if (wl .le. (zbotdr(level)+0.001d0).or.swsrf.eq.1) then

C --- only groundw. level above channel bottom; bottom is dr. base
              drainl(level) = zbotdr(level)

C --- wetted perimeter only computed for open channels
C     (for drains it is input) 
              if (swdtyp(level) .eq. 0) then
                wetper(level) = widthr(level)
              endif
            else

C --- surface water level above channel bottom
              drainl(level) = wl
              if (swdtyp(level) .eq. 0) then
                swdepth = wl-zbotdr(level)
                swexbrd = (wl-zbotdr(level))/taludr(level)
                wetper(level) = widthr(level) +
     &            2*sqrt(swdepth**2 + swexbrd**2) 
              endif
            endif

C --- drainage flux (cm/d)
            dh = gwl - drainl(level)
            if (gwl.gt.-0.1d0) dh = dh+pond
            if (dh.lt.0.0 .and.gwl.lt.gwlinf(level)) then
              dh = gwlinf(level)-drainl(level)
            endif

            if (dh.gt.0.0) then
              rd = rdrain(level)
              re = rentry(level)
            else
              rd = rinfi(level)
              re = rexit(level)
            endif 
            if (swdtyp(level).eq.0) then
              qdrain(level) = dh / ((rd + re*l(level)/wetper(level)))
            else
              qdrain(level) = dh/rd
            endif 
          else
            if (swdtyp(level) .eq. 0) then
              wetper(level) = 0.0
            endif
            dh = 0.0
            qdrain(level) = 0.0
          endif
c
	  else
	    qdrain(level) = 0.0
        endif
c
        if (swsrf.ge.2.and.level.gt.nrpri) then

C --- qdrd is total flux to or from secondary system
          qdrd = qdrd + qdrain(level)
        endif
 500  continue

C ----------------------------------------------------------------------
C --- check for system falling dry (only for swsec = 2):
      if (swsec .eq. 1) return
      if (swsrf.eq.1) then
        do 10 level = 1,nrsrf
          if (qdrain(level).lt.0.0) then
            qdrain(level) = 0.0
          endif
 10     continue
      elseif (swsrf.ge.2) then

C --- determine which management period the model is in:
        imper   = 0
 800    imper   = imper + 1

#ifdef SWAPS
C --- Error handling SWAPS
        if (imper .gt. nmper) then
          call SetFatalError(9999,
     &                       'err. sw-management periods','BOCODRE')
          call SetModelStatus(100)
          return
        endif
#endif

#ifdef SWAP
C --- Error handling SWAP
        if (imper .gt. nmper) stop 'BOCODRE: err. sw-management 
     &  periods'
#endif

        if (int(T+1.0d0) .gt. impend(imper)) goto 800

C ---   determine whether the system will become empty
        dvmax  = (qdrd + wscap(imper)) * dt
        wstmax = wst + dvmax

        if (wstmax .lt. 0.0) then
C ---     storage decreases to below zero, then the surface water system 
C         falls dry; make the total infiltration exactly equal to the
C         available amount:
          qdrdm = - (wst + wscap(imper)*dt) / dt
          qdratio = qdrdm / qdrd
          if (qdratio .gt. 1.0d0 .or. qdratio .lt. 0.0d0) then

#ifdef SWAPS
C --- Error handling SWAPS
            write(swl,*) qdratio,qdrdm,wst,wscap(imper),dt
            call SetModelStatus(100)
            call SetFatalError(9999,'error algorithm swbal','BOCODRE')
            return
#endif

#ifdef SWAP
C --- Error handling SWAP
            print *, qdratio,qdrdm,wst,wscap(imper),dt
            stop 'BOCODRE: error algorithm swbal'
#endif

          endif

          do 820 level=1+NRPRI,nrsrf
            qdrain(level) = qdrain(level)*qdratio
 820       continue
          qdrd = qdrdm
        endif
      endif

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE BOCOTOP (POND,REVA,NIRD,DT,QTOP,QBOT,QDRTOT,
     & KMEAN,K,H,DISNOD,LAYER,FLGENU,COFGEN,THETA,THETIM,DZ,
     & HTABLE,THETLO,THETHI,KTABLE,ALFAMG,FTOPH,ARFLUX,THETAR,
     & THETAS,KSE99,KSAT,SWMOBI,FMOBIL,VOLSAT,VOLACT,ACRACK,QCRACK,
     & FLTSAT,HSURF,FLLAST,FLRAIC,DTMIN,WTOPLAT,FLREVA,NUMNOD,QROSUM,
     & QCOMP,FPEGWL,NPEGWL,Q,SWBOTB,GWLINP
#ifdef SWAPS
C --- SWAPS: extra parameters in this function...
     & ,hAtm,FlAdjust,revadeep
#endif
     & )
C ----------------------------------------------------------------------
C     Date               : 4/2/2000
C     Purpose            : determines the values of the boundary       
C                          conditions at the top of the profile        
C     Formal parameters  :                                             
C     Subroutines called : -                                           
C     Functions called   : hconode,thenode                                 
C     File usage         : -                                           
C     SWAP/SWAPS differences: 
C     - extra parameters SWAP
C     - Assignment pressure heads different in SWAP/SWAPS
C ----------------------------------------------------------------------
C
C --- SWAPS NOTES:
C    (1) Number of soil layers and corresponding array lengths increased
C    from 5 (SWAP) to 10 (SWAPS)
C
C    (2) Notes on arguments:
C
C     SWAPS external name       Local name
C     -------------------       ----------
C     revatop                   REVA
C     nrai                      ARFLUX
C     flrevatop                 FLREVA
C
C     hAtm [cm] input, not calculated here
C     FlAdjust set to .TRUE. if timestep decreased here.
C
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER LAYER(MACP),THETLO(MAHO),THETHI(MAHO),SWMOBI,NUMNOD,NODE
      INTEGER NPEGWL,SWBOTB

      REAL*8  POND,REVA,NIRD,DT,QTOP,QBOT,QDRTOT,DISNOD(MACP+1),WTOPLAT
      REAL*8  FMOBIL(MACP),ARFLUX,ACRACK,QCRACK,HSURF,QTOP1,DTMIN,QROSUM
      REAL*8  QCOMP,DZ(MACP),GwlInp,hAtm
      REAL*8  KSURF,HCONODE,KMEAN(MACP+1),K(MACP+1),H(MACP)
      REAL*8  COFGEN(8,MAHO),THETA(MACP),THETIM(MAHO)
      REAL*8  HTABLE(MAHO,99),KTABLE(MAHO,99),ALFAMG(MACP)
      REAL*8  THETAR(MACP),THETAS(MACP),THENODE,KSE99(MAHO),KSAT(MAHO)
      REAL*8  VOLACT,VOLSAT,Q(MACP+1)

      LOGICAL FLGENU(MAHO),FTOPH,FLTSAT,FLLAST,FLRAIC,FLREVA,FPEGWL

C --- SWAPS variables
#ifdef SWAPS
      LOGICAL FlAdjust
      REAL*8  revadeep
#endif

C ----------------------------------------------------------------------
C --- local variables
      INTEGER i,lay
      REAL*8  inflow,qsurf,vair,Emax,Imax

      save
C ----------------------------------------------------------------------

C --- Assign pressure heads, different in SWAP/SWAPS
CEM 7-7-99: 
CMA   Adjustment below is needed for SWAPS which has hAtm = f(RH), 
CMA   not constant. hAtm is used to calculate ksurf only if RH < 100%. 
CMA   If RH = 100%, hAtm = 0 and k(hAtm) = kSat would be used for 
CMA   kSurf which is incorrect. Therefore for if RH = 100% use 
CMA   kSurf = k(1).
C --- Pressure head of the atmosphere for SWAPS assigned elswhere...

#ifdef SWAP
C --- Assign pressure head of the atmosphere for SWAP
        hAtm  = -2.75d+05
#endif

C --- Calculate hydraulic conductivity corresponding with hAtm
      if (hAtm.lt.0.0) Then
         ksurf = hconode (1,THENODE(1,DBLE(HATM),LAYER,FLGENU,
     &        COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS),LAYER,
     &        FLGENU,COFGEN,THETAR,THETAS,THETLO,THETHI,KTABLE,
     &        KSE99,KSAT,SWMOBI,FMOBIL)
      else

CMJW  This only occurs if RH is 100% in SWAPS, never used for SWAP
         kSurf = k(1)
      endif

C --- determine potential flux at the top of the profile
      flreva = .false.

C --- save qtop from previous time step
      qtop1 = qtop
      if (pond.gt.1.0d-6) then
        qtop = reva-(pond-wtoplat)/dt-(ARFLUX+NIRD)*(1.0d0-acrack)+qcomp
      else
        qtop = reva-(ARFLUX+NIRD)*(1.0d0-acrack)+qcomp
      end if

C --- Adjustment for time step switched off in SWAPS, discuss with EM!!!
C     Adjustment time step for SWAP below
#ifdef SWAP
c --- decrease time step at transition evaporation --> large infiltration
      if (qtop1 .gt. 0.001d0 .and. qtop .lt. -1.0d0 .and. pond.lt.1.0d-7
     &    .and. dt .gt. (dtmin+1.d-9)) then
        dt = dtmin

C --- recalculate qtop 
        qtop = reva-(pond-wtoplat)/dt-(ARFLUX+NIRD)*(1.0d0-acrack)+qcomp
        if (fllast) fllast = .false.
        if (flraic) flraic = .false.
      endif        
#endif

C --- maximum evaporation rate according to Darcy
      kmean(1) = 0.5*(ksurf+k(1))
      Emax = kmean(1) * (h(1)-hatm) / disnod(1) - kmean(1)

C --- maximum infiltration rate according to Darcy
      kmean(1) = 0.5*(ksat(1)+k(1))
      Imax = kmean(1) * (pond-h(1)) / disnod(1) + kmean(1)
      Imax = max(ksat(1),Imax)

C --- potential flux at soil surface
      qsurf = min(qtop,Emax)
      qsurf = max(qsurf,-Imax)

C --- determine inflow
      if (fpegwl.and.npegwl.gt.0) then

C --- perched groundwater level exists
        inflow = (- qsurf + q(npegwl+1)) * dt
      else

C --- just phreatic groundwater level
CMS 19-JUL-99 SWAPS: include revadeep
#ifdef SWAPS
        inflow = (qbot - qsurf - qrosum - revadeep - qdrtot + qcrack)*dt
#endif

#ifdef SWAP
C --- Does not include revadeep
        inflow = (qbot - qsurf - qrosum - qdrtot + qcrack)*dt
#endif

      endif

C --- set flux boundary condition
      ftoph = .false.

c --- determine whether soil profile is saturated
      fltsat = .true.
      do 100 node = 1,numnod
        if (h(node) .lt. 0.0) fltsat = .false.
 100  continue      

#ifdef SWAPS
CEM 7-7-99:
C For SWAPS:
C if soil profile was saturated and becomes unsaturated
c set FlAdjust to TRUE and qtop to qtop1
      if (fltsat.and.(inflow.le.-1.0e-6).and.(dt.gt.(dtmin+0.001))) then
        FlAdjust = .true.
        qtop = qtop1
      else
#endif
        if (fltsat) then

c --- soil profile was saturated
          if (inflow .gt. -1.0d-6) then

C --- profile remains saturated, head boundary condition
            ftoph    = .true.
            hsurf    = inflow
            kmean(1) = ksat(1)
          else

C --- Code below has been commented out in SWAPS, is now only used in SWAP
#ifdef SWAP

C --- profile becomes unsaturated
            if (dt .gt. (dtmin+0.001d0)) then

C --- decrease time step for numerical stability
              dt = 0.001d0

C --- qtop recalculate
              qtop = reva - (pond - wtoplat)/dt 
     &               - (ARFLUX+NIRD)*(1.0d0-acrack) + qcomp
              if (fllast) fllast = .false.
              if (flraic) flraic = .false.
            endif
#endif

            if (qtop .ge. 0.0) then

c --- maximum evaporation flux soil
              if (qtop.gt.Emax) then
                ftoph  = .true.

#ifdef SWAPS
CMS 19-JUL-99 SWAPS: only adjust reva if reva > 0
                if (reva.gt.0.0) flreva = .true.
#endif

#ifdef SWAP
CMJW SWAP: always adjust reva
                flreva = .true.
#endif

                hsurf  = hAtm
                kmean(1) = 0.5*(ksurf+k(1))
              endif             
            else
c --- maximum infiltration flux soil
              if ((qtop .lt. -ksat(1)) .and. (qtop .lt. (-Imax))) then 
                ftoph = .true.
                hsurf = pond
                kmean(1) = 0.5*(ksat(1)+k(1))
              endif
            endif
          endif
        else
c --- soil profile was unsaturated

c --- air volume in case of perched groundwater level
          if (fpegwl.and.npegwl.gt.0) then
            vair = 0.0
            do 120 i = 1,npegwl
              lay    = layer(i)
              vair = vair + thetas(i) - theta(i) - 
     &                          (1.0d0-fmobil(i))*thetim(lay)*dz(i)
 120        continue

c --- air volume in case of phreatic groundwater level
	  else
            vair = volsat-volact
	  endif

          if (inflow.ge.vair) then

C --- profile becomes saturated
            ftoph = .true.
            if (swbotb.eq.1) then
              hsurf = gwlinp
            else
              hsurf = inflow - vair
            endif
            kmean(1) = ksat(1)
          else

C --- profile remains unsaturated
            if (qtop .ge. 0.0) then

C --- maximum evaporation flux soil
CMS 22-JUL-99: abs(Emax) and if (Emax.gt.0.0) as in old version, 
CMS            probably incorrect, should be looked into!!!
              if (qtop .gt. Emax) then
                ftoph = .true.

#ifdef SWAPS
CMS 19-JUL-99   only adjust reva if reva > 0 for SWAPS
                if ((Emax.gt.0.0).and.(reva.gt.0.0)) flreva = .true.
#endif

#ifdef SWAP
C --- For SWAP, set flreva to true
                flreva = .true.
#endif

                hsurf = hAtm
                kmean(1) = 0.5*(ksurf+k(1))
              endif             
            else

C --- maximum infiltration flux soil
              if ((qtop .lt. -ksat(1)) .and. (qtop .lt. (-Imax))) then 
                ftoph = .true.
                hsurf = pond
                kmean(1) = 0.5*(ksat(1)+k(1))
              endif
            endif
          endif
        endif
     
#ifdef SWAPS
C --- Terminate upper "SWAPS-only" if-then-else statement
      endif
#endif

      return
      end


C ----------------------------------------------------------------------
      SUBROUTINE CALCGWL(GWL,GWLBAK,NUMNOD,H,Z,DZ,SWBOTB,POND,NODGWL,
     &           FPEGWL,NPEGWL,PEGWL,GWLTAB,T,NODGWLM1)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : search for the watertable and perched
C                          watertable(s) (if existing).
C     Formal parameters  :                                             
C     Subroutines called : -                                           
C     Functions called   : -                                     
C     File usage         : -
C     SWAP/SWAPS differences: - Error handling
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER  NUMNOD,SWBOTB,NODGWL,NPEGWL,NODGWLM1
     
      REAL*8   GWL,GWLBAK(4),Z(MACP),DZ(MACP),POND,T,PEGWL,GWLTAB(732)
C     REAL*8   oscil

      REAL*8   H(MACP)

      LOGICAL  FPEGWL

C ----------------------------------------------------------------------
C --- local
      INTEGER  node,i
      REAL*8   gwprod1,gwprod2,Afgen

      LOGICAL  flsat

      save
C ----------------------------------------------------------------------

C --- set initial values
        gwl     = 999.0d0
      flsat     = .false.
      nodgwl    = numnod+1
      fpegwl    = .false.

C --- search for ground watertable
      if (h(numnod).ge.0.0d0) then
        flsat  = .true.
      endif
      do 200 node = numnod-1,1,-1
          if (flsat.and.h(node).lt.0.0d0) then
            gwl = (z(node+1) + h(node+1) / (h(node+1)-h(node))
     $            * 0.5 * (dz(node) + dz(node+1))) 
            flsat   =.false.
            nodgwl  = node
          endif
 200  continue

C --- groundwater level near soil surface
      if (flsat) then
        if (pond .lt. 1.d-8) then
          gwl = z(1) + h(1)
        else
          gwl = pond
        endif
        nodgwl = 1
      endif 
 
c --- search for perched groundwater table
      do 300 node = numnod,2,-1

c ---   first search for transition to saturated part 
        if ((.not.fpegwl) .and. (h(node).lt.0.0d0)
     &	                .and. (h(node-1).gt.0.0d0)) then            
          do 400 i = node-1,2,-1

c ---       next search for transition to unsaturated part 
            if ((.not.fpegwl) .and. (h(i).gt.0.0d0)
     &	                    .and. (h(i-1).lt.0.0d0)) then            
              fpegwl  =.true.
              npegwl  = i - 1
              pegwl   = (z(i-1)*h(i)-z(i)*h(i-1))/(h(i)-h(i-1))
            endif
 400      continue
          if (.not.fpegwl) then
            fpegwl = .true.
            npegwl = 0
            pegwl  = pond
          endif
        endif

 300  continue

C --- set gwl in case gwl is prescribed below soil profile
      if (swbotb.eq.1.and.gwl.gt.998d0) gwl = afgen(gwltab,732,t)

C --- warning gwl below profile
C --- Difference in error handling SWAP/SWAPS
      if ((swbotb.eq.3.or.swbotb.eq.4).and.gwl.gt.998.0d0) then

#ifdef SWAPS
C --- Error handling SWAPS
        call SetFatalError(9999,
     &           'Groundwater table below lower boundary','CALCGWL')
        call SetModelStatus(100)
        return
#endif

#ifdef SWAP
C --- Error handling SWAP
        stop 'Groundwater table below lower boundary'
#endif

      endif

C --- updating registration of last four groundwater levels, for noting 
C     oscillation
        gwlbak(1) = gwlbak(2)
        gwlbak(2) = gwlbak(3)
        gwlbak(3) = gwlbak(4)
        gwlbak(4) = gwl

        gwprod1 = (gwlbak(2)-gwlbak(1))*(gwlbak(3)-gwlbak(2))
        gwprod2 = (gwlbak(3)-gwlbak(2))*(gwlbak(4)-gwlbak(3))

        if (gwprod1.lt.0.0d0 .and. gwprod2.lt.0.0d0) then

C --- SWAPS/SWAP Temporarily commented out!!!
C          oscil = abs(gwlbak(3)-gwlbak(2)) 
C          if (oscil.gt.osgwlm.and.(gwl.lt.-10.0.or.gwl.gt.5.0))then
C            write(6,9997) i,oscil
C 9997       format(/' >>gw-level(',i1,') oscillation of ',f4.1,' cm;',
C     &             'advise reduction of dtmax') 
c          endif

        endif

      return
      end


C ----------------------------------------------------------------------
      subroutine CRACK (layer,nncrack,thetsl,theta,shrina,shrinb,shrinc,
     & acrack,wtoplat,geomf,pond,numnod,adsflu,thetcr,z,dz,qcrack,
     & moisr1,moisrd,crackw,flgenu,cofgen,thetar,thetas,thetlo,thethi,
     & ktable,kse99,ksat,swmobi,fmobil,h,diampol,disnod,rapdra,rapcoef,
     & cralev)
C ----------------------------------------------------------------------
C      Date               : 29/9/99
C      Purpose            : infiltration into cracks, absorption to     
C                           soil matrix, rapid drainage                 
C      Subroutines called :
C      Functions called   :
C      File usage         :
C      Differences SWAP/SWAPS: CRADEP extra SWAPS variable
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER swmobi,nncrack,layer(MACP),numnod,thetlo(MAHO)
      INTEGER thethi(MAHO)

      REAL*8  shrina,shrinb,shrinc,acrack,wtoplat,geomf,pond,cralev
      REAL*8  adsflu(MACP),thetcr(MAHO),z(MACP),dz(MACP),qcrack,moisr1
      REAL*8  crackw,fmobil(MACP),diampol,disnod(MACP+1),rapdra,rapcoef
      REAL*8  moisrd

      REAL*8  thetsl(MAHO),theta(MACP),hconode,cofgen(8,MAHO)
      REAL*8  ktable(MAHO,99)
      REAL*8  ksat(MAHO),kse99(MAHO),thetar(MACP),thetas(MACP),h(MACP)

      LOGICAL flgenu(MAHO)
C ----------------------------------------------------------------------
C --- local
      INTEGER lay,node

      REAL*8  vsolid,moisr,voidr,vapore,vshrink,subsid,vcrack
      REAL*8  totalv,conduc,ratio,hcrack,cradepth

#ifdef SWAPS
C --- SWAPS variable
      REAL*8  cradep
#endif
C ----------------------------------------------------------------------

C --- shrinkage volume and area crack at soil surface
      lay = layer(nncrack)
      if (theta(nncrack) .gt. (thetsl(lay)-0.01D0)) then
        acrack  = 0.0
        wtoplat = 0.0
      else
        vsolid =1.0d0 - thetsl(lay)
        moisr  = theta(nncrack) / vsolid
        if (moisr .lt. moisr1) then
          voidr = shrina / exp(shrinb*moisr) + shrinc * moisr
        else
          voidr = moisr1 + moisrd
        endif
        vapore = voidr * vsolid
        vshrink =thetsl(lay) - vapore
        subsid = 1.0d0 - (1.0d0 - vshrink) ** (1.0 / geomf)
        vcrack = vshrink - subsid
        acrack = vcrack / (1.0d0 - subsid)

C --- lateral flow due to ponding at the soil surface 
        wtoplat = pond
      endif

C --- initialize values
      do 50 node = 1,numnod
        adsflu(node) = 0.0
 50   continue
      totalv = 0.0
      qcrack = 0.0
      cralev = 0.0
      cradepth = 0.0

C --- determine depth of crack
      node = numnod
      lay  = layer(node)
 60   if (theta(node).gt.(thetcr(lay) - 1.0d-4) .and. node.gt.1) then
        node = node - 1
        lay  = layer(node)
        goto 60
      endif

C --- skip calculation when cracks are closed until soil surface
      if (node.eq.1 .and. theta(1).gt.(thetsl(1)-0.01D0)) goto 70
        cradepth = z(node) - 0.5*dz(node)

C --- crack volume for each node 
 65   if (theta(node) .gt. (thetsl(lay)-0.01D0)) then
        if (node .gt. 1) then
          node = node -1
          lay = layer(node)
          goto 65
        endif
      else
        vsolid = 1.0d0 - thetsl(lay)
        moisr = theta(node) / vsolid
        if (moisr .lt. moisr1) then
          voidr = shrina / exp(shrinb*moisr) + shrinc * moisr
        else
          voidr = moisr1 + moisrd
        endif
        vapore = voidr * vsolid
        vshrink = thetsl(lay) - vapore
        subsid = 1.0d0 - (1.0d0 - vshrink) ** (1.0 / geomf)
        vcrack = (vshrink - subsid)*dz(node)
        totalv = totalv + vcrack

c --- determine lateral infiltration flux from crack 
        if (totalv .lt. crackw) then

c --- compartment participates with full height 
          conduc = hconode (node,theta(node),LAYER,FLGENU,COFGEN,
     &           THETAR,THETAS,THETLO,THETHI,KTABLE,KSE99,KSAT,SWMOBI,
     &           FMOBIL)
          hcrack = z(node) - cradepth
          adsflu(node) = (- conduc*24*dz(node)*(h(node)-hcrack)/
     &                       (diampol*diampol))
          qcrack = qcrack + adsflu(node)
          if (node .gt. 1) then
            node = node -1
            lay = layer(node)
            goto 65
          endif
        else

c --- compartment participates with part of total height 
          ratio  = (crackw - (totalv - vcrack)) / vcrack
          cralev = z(node) - 0.5 * disnod(node+1) + ratio * dz(node)
          conduc =hconode (node,theta(node),LAYER,FLGENU,COFGEN,
     &           THETAR,THETAS,THETLO,THETHI,KTABLE,KSE99,KSAT,SWMOBI,
     &           FMOBIL)
          hcrack = z(node) -cradepth       
          adsflu(node) = (- conduc*24*dz(node)*(h(node)-cradepth)/
     &                       (diampol*diampol)* ratio)
          qcrack = qcrack + adsflu(node)
        endif
      endif

C --- rapid drainage
 70   rapdra = rapcoef * crackw

      return
      end


C ----------------------------------------------------------------------
      SUBROUTINE DIVDRA( NumComp, NumDrain,ThickComp,KSAT,LAYER,COFANI,
     &                   GWLEV, DistDrain, FluxDr,FluxDrComp)  
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : Simulation of lateral waterfluxes in the
C                          saturated zone. In two steps:
C                          1. Calculate bottom-boundaries of
C                             model_discharge_layers;
C                          2. Distribute drainage fluxes over each
C                             model_discharge_layer.
C                          
C     Subroutines called : -                                           
C     Functions called   : -                                     
C     File usage         : -
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER DrainSequence(5),Icomp,idr,iidr
      INTEGER idum,jdr,NumComWatLev,NumComBotDislay(5),NumActDrain
      INTEGER NumDrain,NumComp,LAYER(MACP)

      REAL*8  FacAniso,CondSatHorAv,CondSatVerAV,BotDisLay(5),ThickCum
      REAL*8  CumThickCondHor,CumThickCondVer,DistDrain(5),Dum
      REAL*8  FluxDr(5),FluxDrComp(5,MACP),FlowDrDisch(5),HelpFl(5)
      REAL*8  HelpTh,HelpTr,MaxDepthDislay(5),CondSatHor(MACP)
      REAL*8  CondSatVer(MACP),Small,ThickComp(MACP),ThickCompSatWatLev
      REAL*8  ThickCompBotDislay(5),Transmissivity(5),WatLevAv,GWLEV
      REAL*8  COFANI(MAHO)

      REAL*8  KSAT(MAHO)

      LOGICAL NonExistFluxDr

      DATA    Small /1.0d-10/
C ----------------------------------------------------------------------

C --- groundwater level converted to cm below surface level 
      WatLevAv = -1*MIN(GWLEV,0.0d0)

C --- calculation of CondSatHor and CondSatVer
      do 10 icomp = 1,NumComp
        CondSatHor(icomp) = KSAT(LAYER(icomp))
        CondSatVer(icomp) = KSAT(LAYER(icomp))*COFANI(LAYER(icomp))
 10   continue

C --- Initialisation and test whether any drainflux exists
      NonExistFluxDr = .true.
      do idr=1,NumDrain
         if( abs( FluxDr(idr) ) .gt. Small ) NonExistFluxDr = .false.
         do icomp = 1, NumComp
            FluxDrComp(idr,icomp) = 0.0
         enddo
      enddo      
      if( NonExistFluxDr ) return

C --- paragraph 2 
C --- Search for compartment with groundwaterlevel (NumComWatLev)
      NumComWatLev = 1
      ThickCum     = ThickComp(1)
      do while (WatLevAv.gt.ThickCum)
         NumComWatLev = NumComWatLev + 1
         ThickCum     = ThickCum + ThickComp(NumComWatLev)
      enddo

C --- saturated part (ThickComp_unsat) of compartment with waterlevel
      ThickCompSatWatLev = ThickCum - WatLevAv

C --- paragraph 3
C --- Calculate overall anisotropic factor model profile and 
C --- cumulative transmissivity as a function of depth
      CumThickCondHor = ThickCompSatWatLev * CondSatHor(NumComWatLev)
      CumThickCondVer = ThickCompSatWatLev / CondSatVer(NumComWatLev)
      ThickCum        = ThickCompSatWatLev

      do icomp=NumComWatLev+1,NumComp
         CumThickCondHor = CumThickCondHor + ThickComp(icomp) * 
     &                       CondSatHor(icomp)
         CumThickCondVer = CumThickCondVer + ThickComp(icomp) /
     &                       CondSatVer(icomp)
         ThickCum        = ThickCum + ThickComp(icomp)
      enddo
      
      CondSatHorAv      = CumThickCondHor / ThickCum
      CondSatVerAv      = ThickCum / CumThickCondVer
      FacAniso          = sqrt( CondSatVerAv / CondSatHorAv )

C --- paragraph 4      
C --- Calculate maximum depth per drainage system and discharge 
C --- flow rate per drainage system 
      do idr=1,NumDrain
         MaxDepthDislay(idr) = 0.25*DistDrain(idr)*FacAniso+WatLevAv

C ---    beperken tot SWAP profiel dikte
         MaxDepthDislay(idr) = MIN (MaxDepthDislay(idr),
     &                              Thickcum+WatLevAv)
      enddo

C --- paragraph 5      
C --- determine sequence of order of drainage systems 

      NumActDrain = 0
      do idr=1,NumDrain
         if ( abs(FluxDr(idr)) .gt. small ) then
            NumActDrain = NumActDrain+1
            DrainSequence(NumActDrain) = NumActDrain
            HelpFl(NumActDrain)        = DistDrain(idr)
         end if 
      end do
      
      do idr=1,NumActDrain-1
         do iidr = idr+1,NumActDrain
            if ( HelpFl(idr) .lt. HelpFl(iidr) )then
             dum                 = HelpFl(iidr)
             HelpFl(iidr)        = HelpFl(idr)
             HelpFl(idr)         = dum
             idum                = DrainSequence(iidr)
             DrainSequence(iidr) = DrainSequence(idr)
             DrainSequence(idr)  = idum
            endif
         enddo
      enddo       

      idr = DrainSequence(NumActDrain)
      FlowDrDisch(idr) = abs(FluxDr(idr)*HelpFl(idr))
      do iidr=NumActDrain-1,1,-1 
         idr = DrainSequence(iidr)
         jdr = DrainSequence(iidr+1)
         FlowDrDisch(idr) = FlowDrDisch(jdr) + 
     &                      abs(FluxDr(idr)*HelpFl(idr))
      enddo

C --- paragraph 6
C --- Bottom of 1st order model_discharge_layer

      idr = DrainSequence(1)
      Transmissivity(idr)     = CumThickCondHor
      BotDisLay(idr)          = ThickCum + WatLevAv
      NumComBotDislay(idr)    = NumComp
      ThickCompBotDislay(idr) = ThickComp(NumComp)           
      
C --- correction of BotDisLay(1) if D1 < 0.25 L \/(kv/kh) 
      if (abs(FluxDr(idr)).gt.Small) then
         if (BotDisLay(idr) .gt. MaxDepthDislay(idr) )then
            BotDisLay(idr) = MaxDepthDislay(idr)

C ---       determine adjusted transmissivity and compartment 
C ---       number which contains bottom of discharge layer
            NumComBotDislay(idr) = NumComWatLev
            HelpTh               = ThickCompSatWatLev
            Transmissivity(idr)  = ThickCompSatWatLev * 
     &                             CondSatHor(NumComWatLev)
            do while ( BotDisLay(idr)-WatlevAv .gt. HelpTh )      
               NumComBotDislay(idr) = NumComBotDislay(idr) + 1       
               HelpTh               = HelpTh + 
     &                                ThickComp(NumComBotDislay(idr))
               Transmissivity(idr)  = Transmissivity(idr) + 
     &                                ThickComp(NumComBotDislay(idr)) * 
     &                                CondSatHor(NumComBotDislay(idr))              
            enddo
            Transmissivity(idr) = Transmissivity(idr) - 
     &                          (HelpTh - BotDisLay(idr)) * 
     &                          CondSatHor(NumComBotDislay(idr))              

C ---       thickness of part of bottom compartment
            ThickCompBotDislay(idr) = ThickComp(NumComBotDislay(idr)) - 
     &                              (HelpTh - BotDisLay(idr)) 
         endif      
      else 
         BotDisLay(idr) = WatLevAv
      endif

C --- paragraph 7
C --- Bottom of 2nd and higher order model_discharge_layers 
C --- for drainage system of orders 2 to NumDrain (number of drains)         
      do iidr=2,NumActDrain
         idr = DrainSequence(iidr) 
         jdr = DrainSequence(iidr-1)
         Transmissivity(idr) = Transmissivity(jdr) * 
     &                         FlowDrDisch(idr) / FlowDrDisch(jdr) 

C --- bottom of discharge layer of order i
C     and thickness of bottom compartment, and
C     number of bottom compartment
         icomp  = NumComWatLev
         HelpTr = ThickCompSatWatLev * CondSatHor(icomp)
         HelpTh = ThickCompSatWatLev
         do while ( HelpTr .lt. transmissivity(idr) )      
            icomp  = icomp + 1       
            HelpTr = HelpTr + ThickComp(icomp)*CondSatHor(icomp)      
            HelpTh = HelpTh + ThickComp(icomp)       
         enddo
         NumComBotDislay(idr)    = icomp
         ThickCompBotDislay(idr) = ThickComp(icomp) - 
     &             ( HelpTr - Transmissivity(idr)) / CondSatHor(icomp)
            BotDisLay(idr)      = WatLevAv + HelpTh +
     &             ( Transmissivity(idr) - HelpTr) / CondSatHor(icomp)

C --- paragraph 8
C     correction of BotDisLay(idr) if D(idr) < 0.25 L(idr) \/(kv/kh) 
         if (BotDisLay(idr).gt. MaxDepthDislay(idr)) then
            BotDisLay(idr) = MaxDepthDislay(idr)

C --- transmissivity of order i, thickness 
C     of bottom compartment, and number 
C     of bottom compartment
            NumComBotDislay(idr) = NumComWatLev
            HelpTh               = ThickCompSatWatLev
            Transmissivity(idr)  = ThickCompSatWatLev * 
     &                             CondSatHor(NumComWatLev)
            do while ( BotDisLay(idr)-WatLevAv .gt. HelpTh )      
               NumComBotDislay(idr) = NumComBotDislay(idr) + 1       
               HelpTh = HelpTh + ThickComp(NumComBotDislay(idr))       
               Transmissivity(idr)  = Transmissivity(idr) + 
     &                               ThickComp(NumComBotDislay(idr)) * 
     &                               CondSatHor(NumComBotDislay(idr))
            enddo
            Transmissivity(idr) = Transmissivity(idr) - 
     &                            (HelpTh - BotDisLay(idr))*
     &                            CondSatHor(NumComBotDislay(idr))
            ThickCompBotDislay(idr) = ThickComp(NumComBotDislay(idr)) - 
     &                            (HelpTh - BotDisLay(idr)) 
         endif   
      enddo

C --- paragraph 9
C     Distribute drainage fluxes as lateral fluxes over 
C     ith order model discharge layers
      do iidr=1,NumActDrain
         idr = DrainSequence(iidr) 
         FluxDrComp(idr,NumComWatLev) = FluxDr(idr)*ThickCompSatWatLev*
     &                              CondSatHor(NumComWatLev) / 
     &                              Transmissivity(idr)
         do icomp=NumComWatLev+1,NumComBotDislay(idr)-1
            FluxDrComp(idr,icomp) = FluxDr(idr)*ThickComp(icomp)*
     &                              CondSatHor(icomp) / 
     &                              Transmissivity(idr)
         enddo
         FluxDrComp(idr,NumComBotDislay(idr)) = FluxDr(idr)*
     &                              ThickCompBotDislay(idr)*
     &                              CondSatHor(NumComBotDislay(idr)) / 
     &                              Transmissivity(idr)
      enddo

      return
      end


C ----------------------------------------------------------------------
      REAL*8 FUNCTION DMCNODE (NODE,HEAD,LAYER,FLGENU,COFGEN,THETLO,
     & THETHI,DMCH,DMCC,THETAS,THETAR,ALFAMG,SWMOBI,FM1,PF1,SLOPFM,
     & HTABLE,THETA)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculate the differential moisture         
C                          capacity (as a function of pressure head)
C     Subroutines called : -                                           
C     Functions called   : thenode                                     
C     File usage         : -                                           
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER LAYER(MACP),NODE,THETLO(MAHO)
      INTEGER THETHI(MAHO),SWMOBI

      REAL*8  FM1(MAHO),PF1(MAHO),SLOPFM(MAHO) 

      REAL*8  HEAD,COFGEN(8,MAHO),DMCC(MAHO,99),DMCH(MAHO,99)
      REAL*8  THETAS(MACP)
      REAL*8  THETAR(MACP),ALFAMG(MACP),HTABLE(MAHO,99),THETA(MACP)

      LOGICAL FLGENU(MAHO)
C ----------------------------------------------------------------------
C --- local 

      INTEGER lay,posit

      REAL*8  fmobile

      REAL*8  deltah,head2,theta2,tresid,tsatu,thenode,alphah
      REAL*8  term1,term2
C ----------------------------------------------------------------------

C --- soil layer of node
      lay = layer(node)
      if (flgenu(lay)) then

C --- according to van Genuchten
        if (head.gt. -1.0d-8) then
          dmcnode = 1.0d-7
        else
          if (SWMOBI.eq.1) then

C --- use numerical evaluation of capacity
            deltah = 0.001 * (1.0e-3 * abs(head)+1.0d0)
            head2 = head + deltah
            if (head2.gt.0.0) then
              deltah = -deltah
              head2  = head + deltah
            endif
            if (head2.gt.-1.0d0) then
              fmobile = fm1(lay)+(0.0-pf1(lay))*slopfm(lay)
            else
              fmobile = fm1(lay)+(log10(abs(real(head2)))-pf1(lay))*
     &        slopfm(lay)
            endif
            fmobile = min (1.0d0,fmobile)
            fmobile = max (0.3d0,fmobile)
            tresid  = thetar(node)
            tsatu   = thetas(node)
            thetar(node) = cofgen(1,lay)*fmobile
            thetas(node) = cofgen(2,lay)*fmobile
            theta2 = thenode (node,head2,layer,flgenu,cofgen,
     &               htable,thetlo,alfamg,thetar,thetas)
            thetar(node) = tresid
            thetas(node) = tsatu           
            dmcnode = (theta2 - theta(node))/deltah
          else

C --- use analytical evaluation of capacity
            alphah = abs (alfamg(node)*head)

C --- compute |alpha * h| to the power n-1
            term1 = alphah ** (cofgen(6,lay) -1.0d0)

C --- compute |alpha*h| to the power n
            term2 = term1 * alphah

C --- add one and raise to the power m+1
            term2 = (1.0d0 + term2) ** (cofgen(7,lay) + 1.0d0)

C --- divide theta-s minus theta-r by term2
            term2 = (thetas(node)-thetar(node)) / term2

C --- calculate the differential moisture capacity
            dmcnode = abs(-1.0d0 * cofgen(6,lay) * cofgen(7,lay) * 
     $                alfamg(node) * term2 * term1)
          endif
          if (head.gt.-1.0d0.and.dmcnode.lt.1.0d-7) dmcnode = 1.0d-7
        endif
      else

C --- according to the tables
        if (head.ge.0.0d0) then
          dmcnode = 1.0d-7
        else
          do 100 posit = thetlo(lay)+1,thethi(lay)
            if (head.le.dmch(lay,posit)) go to 120
 100      continue
 120      dmcnode = dmcc(lay,posit-1)+(head-dmch(lay,posit-1))*
     $              (dmcc(lay,posit)-dmcc(lay,posit-1))/
     $              (dmch(lay,posit)-dmch(lay,posit-1))
        endif
      endif

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE FLUXES (Q,QBOT,DT,INQ,NUMNOD,THETM1,THETA,DZ,QROT,
     & QDRA,QIMMOB,ADSFLU,QTOP,QROSUM,QDRTOT,VOLACT,VOLM1,
     & SWBOTB,QCRACK
#ifdef SWAP
C --- SWAP-only parameters
     & ,FLREVA,POND,WTOPLAT,ARFLUX,NIRD,ACRACK,REVA,QCOMP
#endif
#ifdef SWAPS
C --- SWAPS-only parameters
     & ,EVP
#endif
     & )
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculates the fluxes between compartments
C     Subroutines called : -                                           
C     Functions called   : -                                     
C     File usage         : -                                           
C     Differences SWAP/SWAPS: 
C     - EVP only in SWAPS function parameter list
C     - FLREVA,POND,WTOPLAT,ARFLUX,NIRD,ACRACK,REVA,QCOMP variables
C     only used by SWAP.
C     Different calculations qbot,q and inq
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NUMNOD,SWBOTB

      REAL*8  QBOT,DT,INQ(MACP+1),DZ(MACP),QROT(MACP),QDRA(5,MACP)
      REAL*8  QIMMOB(MACP),ADSFLU(MACP),QTOP,QROSUM,QDRTOT,QCRACK

      REAL*8  Q(MACP+1),THETM1(MACP),THETA(MACP),VOLACT,VOLM1

#ifdef SWAP
C --- Only used by SWAP
      REAL*8  POND,WTOPLAT,ARFLUX,NIRD,ACRACK,REVA,QCOMP
      LOGICAL FLREVA
#endif

#ifdef SWAPS
C --- Only used by SWAPS
      REAL*8  EVP(MACP)
#endif


C ----------------------------------------------------------------------
C --- local
      INTEGER i
      REAL*8 EVPSUM

C ----------------------------------------------------------------------

C --- determine qbot if not specified
#ifdef SWAPS
C --- SWAPS qbot calculation statements
CKRO  23-NOV-1999: update in case of free drainage (swbotb=7)
CEM   12-JUL-1999: qtop is qDarcy from HEADCALC
CEM                evpsum = revadeep
      if (swbotb .eq. 1 .or. swbotb .eq. 5 .or. swbotb .eq. 7 .or. 
     &    swbotb .eq. 8) then
        EVPSUM = 0.0d0
        do i = 1,numnod
          EVPSUM = EVPSUM + EVP(i)
        enddo
        qbot = qtop+qrosum+evpsum+qdrtot-qcrack+(volact-volm1)/dt
      endif
#endif
#ifdef SWAP
C --- SWAP qbot calculation
      if (swbotb .eq. 1 .or. swbotb .eq. 5 .or. swbotb .eq. 7.or. 
     &    swbotb .eq. 8) then
        qbot = qtop + qrosum + qdrtot - qcrack+(volact-volm1)/dt
      endif
#endif

C --- calculate fluxes (cm/d) from changes in volume per compartment
      i      = numnod+1
      q(i)   = qbot
      inq(i) = inq(i) + q(i)*dt

#ifdef SWAPS
C --- SWAPS statements
CMJW EVP(i) added with respect to SWAP
CMS 22-JUL-99: SWAPS NOTE --- evp(1) = 0 when deep soil evaporation
      do 100 i = numnod,1,-1
        q(i)   = -(theta(i)-thetm1(i)+qimmob(i))*dz(i)/dt+q(i+1)-
     &                  qrot(i)-qdra(1,i)-qdra(2,i)-qdra(3,i)-qdra(4,i)-
     &                  qdra(5,i)-evp(i)+adsflu(i)
        inq(i) = inq(i) + q(i)*dt
100   continue
#endif

#ifdef SWAP
C --- SWAP q and inq statements
      do 100 i = numnod,1,-1
        q(i) = - (theta(i)-thetm1(i)+qimmob(i))*dz(i)/dt + q(i+1)-
     &                qrot(i)-qdra(1,i)-qdra(2,i)-qdra(3,i)-qdra(4,i)-
     &                qdra(5,i)+adsflu(i)
        inq(i) = inq(i) + q(i)*dt
100   continue
#endif

C --- set final value qtop
CEM 12-JUL-99: qtop is flux calculated from top comp. water balance 
      qtop = q(1)

#ifdef SWAP
c --- determine reduced evaporation rate in case of FLREVA
      if (flreva) reva = max(0.0d0,qtop + (pond - wtoplat)/dt - qcomp
     &                   + (arflux+nird)*(1.0d0 - acrack))
#endif

      return
      end

C ----------------------------------------------------------------------
      REAL*8 FUNCTION HCONODE (NODE,WCON,LAYER,FLGENU,COFGEN,THETAR,
     & THETAS,THETLO,THETHI,KTABLE,KSE99,KSAT,SWMOBI,FMOBIL)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculate hydraulic conductivity (as a     
C                          function of pressure head)
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C     Differences SWAP/SWAPS: Error handliong
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NODE,LAYER(MACP),THETLO(MAHO),THETHI(MAHO)
      INTEGER SWMOBI

      REAL*8  FMOBIL(MACP)
      REAL*8  WCON,COFGEN(8,MAHO),KTABLE(MAHO,99),KSAT(MAHO),KSE99(MAHO)
      REAL*8  THETAR(MACP),THETAS(MACP) 

      LOGICAL FLGENU(MAHO)

C ----------------------------------------------------------------------
C --- local
      INTEGER lay,posit 

      REAL*8  term1,exponl,relsat
C ----------------------------------------------------------------------

C --- soil layer of node
      lay = layer(node)
      if (flgenu(lay)) then

C --- calculation according to van Genuchten
        relsat = (wcon-thetar(node))/(thetas(node)-thetar(node))
        if (relsat.gt.0.99d0) then

C --- linear interpolation of k
          hconode = kse99(lay) + (relsat-0.99d0)/0.01
     &              *(ksat(lay)-kse99(lay))
        else
          if (relsat.lt.0.001d0) then
            hconode = 1.0d-10
          else
            exponl  = cofgen(5,lay)/cofgen(7,lay)-2.0d0
            term1   = (1.0d0-relsat**(1.0d0/cofgen(7,lay)))
     &                **cofgen(7,lay)
            hconode = cofgen(3,lay)*(relsat**exponl)*(1.0d0-term1)
     &               *(1.0d0-term1)
            if (SWMOBI.eq.1) hconode = hconode*fmobil(node)
          endif
        endif
      else

C ---   calculation according to the tables   
        if (wcon.lt.0.01*thetlo(lay)) then

C --- Error handling SWAPS and SWAP
#ifdef SWAPS
C --- SWAPS error handling
          hconode = -9999.0
          call SetFatalError(9999,
     &          'calculated water content outside range','HCONODE')
          call SetModelStatus(100)
          return
#endif

#ifdef SWAP
C --- SWAP error handling
          stop 'HCONODE - calculated water content outside range'
#endif

        else
          if (wcon.ge.(thetas(node)-0.001d0)) then
            hconode = ktable(lay,thethi(lay))
          else
            posit   = int(100.0*wcon)
            hconode = ktable(lay,posit)+(ktable(lay,posit+1)-
     &                ktable(lay,posit))*(100.0*wcon-real(posit))
          endif
        endif
      endif

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE HEADCALC (NUMNOD,HM1,H,THETM1,THETA,NUMBIT,
     & FTOPH,DT,T,
     & KMEAN,DZ,DISNOD,DIMOCA,QROT,QDRA,POND,QTOP,LAYER,FLGENU,
     & COFGEN,HTABLE,THETLO,THETHI,ALFAMG,THETAR,THETAS,DMCH,DMCC,
     & SWBOTB,SWNUMS,THETOL,REVA,NIRD,QBOT,QDRTOT,K,KTABLE,Q,
     & ARFLUX,KSE99,KSAT,SWMOBI,FMOBIL,VOLSAT,VOLACT,DTMIN,DNOCON,
     & QIMMOB,FM1,PF1,SLOPFM,THETIM,FLLAST,ADSFLU,ACRACK,QCRACK,FLTSAT,
     & HSURF,FLRAIC,WTOPLAT,FLREVA,QROSUM,QCOMP,QDARCY,FPEGWL,NPEGWL,
#ifdef SWAPS
C --- SWAPS parameters
     & FlAdjust,FldtMin,RevaDeep,thetaSurf,NodeEvapFront,hEvapFront,
     & hAtm,Evp
#endif
#ifdef SWAP
C --- SWAP parameter
     & GWLINP
#endif
     & ,ErrorCode)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculate pressure heads, water contents,
C                          and conductivities for next time step
C     Subroutines called : bocotop,TriDag (SWAP only)
C     Functions called   : hconode,dmcnode,thenode                 
C     File usage         : -
C     Differences SWAP/SWAPS: see notes below
C
C --- SWAPS notes
C     (1) Number of soil layers and corresponding array lengths increased
C     from 5 (SWAP) to 10 (SWAPS)
C     (2) Notes on arguments:
C     SWAPS external name       Local name
C     -------------------       ----------
C     revatop #                 REVA     
C     nrai                      ARFLUX
C     flrevatop                 FLREVA
C     # revatop input to local REVA so is NOT included in arguments 
C       (unlike revadeep).
C     (3) FlAdjust set to .TRUE. if timestep decreased here
C     (4) REVA (externally revatop) is parsed to BOCOTOP and is 
C         adjusted HERE if BOCOTOP returns FLREVA (externally flrevatop)
C         = .TRUE. 
C     (5) RevaDeep is used by SoilEvap to get the deep soil evaporation 
C         given by the array Evp
C     (6) Only one of REVA/revatop and EDed may be non-zero.
C     (7) qDarcy = qTop = the flux according to DARCY for the top 
C         compartment
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- common global
      INTEGER NUMNOD,NUMBIT,LAYER(MACP),THETLO(MAHO)
      INTEGER THETHI(MAHO)
      INTEGER SWBOTB,SWNUMS,DNOCON,SWMOBI,NPEGWL

      REAL*8  DTMIN,DT,T,DZ(MACP),DISNOD(MACP+1),QROT(MACP)
      REAL*8  QDRA(5,MACP),POND,QTOP,QROSUM,THETOL,REVA,NIRD,QBOT
      REAL*8  QDRTOT,ARFLUX,FMOBIL(MACP),QIMMOB(MACP),FM1(MAHO)
      REAL*8  PF1(MAHO),SLOPFM(MAHO),ADSFLU(MACP),ACRACK,QCRACK,HSURF
      REAL*8  WTOPLAT,QCOMP,QDARCY,GWLINP

      REAL*8  HM1(MACP),H(MACP),THETM1(MACP),THETA(MACP),KMEAN(MACP+1)
      REAL*8  DIMOCA(MACP),Q(MACP+1),COFGEN(8,MAHO),HTABLE(MAHO,99)
      REAL*8  ALFAMG(MACP),THETAR(MACP),THETAS(MACP),DMCH(MAHO,99)
      REAL*8  DMCC(MAHO,99),K(MACP+1),KTABLE(MAHO,99),KSE99(MAHO)
      REAL*8  KSAT(MAHO),VOLSAT,VOLACT,THETIM(MAHO)

      LOGICAL FTOPH,FLGENU(MAHO),FLLAST,FLRAIC,FLTSAT,FLREVA,FPEGWL
      LOGICAL FlAdjust

#ifdef SWAPS
C --- SWAPS global
      INTEGER NodeEvapFront
      REAL*8  RevaDeep
      REAL*8  thetaSurf,hAtm,hEvapFront
      LOGICAL FldtMin
#endif

#ifdef SWAP
C --- SWAP global
C      REAL*8 GWLINP
#endif

C ----------------------------------------------------------------------
C --- local
      INTEGER ErrorCode,i,j,lay 

      REAL*8  qimold(MACP),fmobm1(MACP),deno
      REAL*8  thenode,hconode,thoma(MACP),thomb(MACP),thomc(MACP)
      REAL*8  thomf(MACP)
      REAL*8  capold(MACP),dmcnode,hold(MACP),theold(MACP)

#ifdef SWAPS
C --- Local SWAPS
      REAL*8  Evp(1:40),qthetaEF
      REAL*8  greeka(40),greekb(40),greekc(40)
C --- error handling SWAPS
      CHARACTER*80 ErrTxt,ErrSrc    
#endif

      save
C ----------------------------------------------------------------------

#ifdef SWAPS
C --- SWAPS: Source (routine name) of potential error     
      ErrSrc = 'HeadCalc'
C     SWAPS: set FlAdjust = .FALSE.
      FlAdjust = .FALSE.
#endif

C --- store values of h, theta and fmobil
      do 400 i = 1,numnod
        hm1(i)    = h(i)
        thetm1(i) = theta(i)
        fmobm1(i) = fmobil(i)
 400   continue

#ifdef SWAPS
***** SWAPS modification - assign values to Evp [cm/d] where 
*     Evp(NumNod) = soil evaporation array  
      call SoilEvap(NumNod,NodeEvapFront,dt,dz,hEvapFront,
     &              REvaDeep,theta,qRot,
     &              LAYER,FLGENU,COFGEN,HTABLE,
     &              THETLO,ALFAMG,THETAR,THETAS,
     &              Evp,qthetaEF)
#endif

#ifdef SWAP
 410  numbit = 1
#endif

#ifdef SWAPS
      numbit = 1
#endif

C --- first step :
C --- ============
C --- calculation of coefficients for node = 1
      i = 1
      if (ftoph) then

C --- h at soil surface prescribed
        thomc(i) = - dt*kmean(i+1)/dz(i)/disnod(i+1)
        thomb(i) = - thomc(i)+dimoca(i)+dt*kmean(1)/disnod(i)/dz(i)
        thomf(i) = dimoca(i)*h(i)-qimmob(i) +
     &             dt/dz(i)*(+kmean(i)-kmean(i+1)-qrot(i)+adsflu(i)
     &             - qdra(1,i) - qdra(2,i) - qdra(3,i) - qdra(4,i)
     &             - qdra(5,i)
#ifdef SWAPS
C --- add EVP(i): only defined in SWAPS
     &             - Evp(i)
#endif
     &             )
     &             + dt*kmean(i)*hsurf/disnod(i)/dz(i)
      else

C --- q at soil surface prescribed
        thomc(i) = - dt*kmean(i+1)/dz(i)/disnod(i+1)
        thomb(i) = - thomc(i) + dimoca(i)
        thomf(i) = dimoca(i)*h(i) -qimmob(i)+
     &             dt/dz(i)*(-qtop-kmean(i+1)-qrot(i)+adsflu(i)
     &             - qdra(1,i) - qdra(2,i) - qdra(3,i) - qdra(4,i)
     &             - qdra(5,i)
#ifdef SWAPS
C --- add EVP(i): only defined in SWAPS
     &             - Evp(i)
#endif
     &             )
      endif

C --- calculation of coefficients for 2 < node < numnod
      do 500 i = 2, numnod-1
        thoma(i) = - dt*kmean(i)/dz(i)/disnod(i)
        thomc(i) = - dt*kmean(i+1)/dz(i)/disnod(i+1)
        thomb(i) = - thoma(i) - thomc(i) + dimoca(i)
        thomf(i) = dimoca(i)*h(i) -qimmob(i)+
     &             dt/dz(i)*(kmean(i)-kmean(i+1)-qrot(i)+adsflu(i)
     &             - qdra(1,i) - qdra(2,i) - qdra(3,i) - qdra(4,i)
     &             - qdra(5,i)
#ifdef SWAPS
C --- add EVP(i): only defined in SWAPS
     &             - Evp(i)
#endif
     &             )
 500   continue

C --- calculation of coefficients for node = numnod
      i = numnod
      if (swbotb .eq. 1 .or. swbotb .eq. 5) then
        thoma(i) = 0.0
        thomb(i) = 1.0d0
        thomf(i) = h(i)
      else
        thoma(i) = - dt*kmean(i)/dz(i)/disnod(i)
        thomb(i) = - thoma(i) + dimoca(i)
        thomf(i) = dimoca(i)*h(i) -qimmob(i)+
     &             dt/dz(i)*(kmean(i)+qbot-qrot(i)+adsflu(i)
     &             - qdra(1,i) - qdra(2,i) - qdra(3,i) - qdra(4,i)
     &             - qdra(5,i)
#ifdef SWAPS
C --- add EVP(i): only defined in SWAPS
     &             - Evp(i)
#endif
     &             )
      endif

C --- SWAPS: Use tridag to be efficient!!!
C     Save old values of h, Theta and DiMoCa in Old.
      do i = 1, NumNod
        hOld(i)   = h(i)
        TheOld(i) = Theta(i)
        CapOld(i) = DiMoCa(i)
      enddo

C --- Solve the tridiagonal matrix
      call TriDag(NumNod, thoma, thomb, thomc, thomf, h,ErrorCode)
      if (ErrorCode .eq. 0) then

C --- first solution
CMA/JH Because TriDag is used this has been simplified to one loop
      do 530 i = 1,numnod
        lay     = layer(i)
        if (SWMOBI.eq.1) then
          if (h(i).gt.-1.0d0) then
            fmobil(i) = fm1(lay)+(0.0-pf1(lay))*slopfm(lay)
          else
            fmobil(i) = fm1(lay)+(log10(abs(real(h(i))))-pf1(lay))*
     &                  slopfm(lay)
          endif
          fmobil(i) = min (1.0d0,fmobil(i))
          fmobil(i) = max (0.3d0,fmobil(i))
          thetar(i) = cofgen(1,lay)*fmobil(i)
          thetas(i) = cofgen(2,lay)*fmobil(i)
          qimold(i) = qimmob(i)
          qimmob(i) = (fmobm1(i)-fmobil(i))*thetim(lay)
        endif
        theta(i)  = thenode(i,h(i),LAYER,FLGENU,COFGEN,HTABLE,
     &              THETLO,ALFAMG,THETAR,THETAS)
        dimoca(i) = dmcnode(i,h(i),LAYER,FLGENU,COFGEN,THETLO,THETHI,
     &              DMCH,DMCC,THETAS,THETAR,ALFAMG,SWMOBI,FM1,PF1,
     &              SLOPFM,HTABLE,THETA)
 530  continue

C --- iteration step : 
C --- ================
 700   continue

#ifdef SWAPS
C --- SWAPS modification - assign values to Evp [cm/d] where 
C     Evp(NumNod) = soil evaporation array
       call SoilEvap(NumNod,NodeEvapFront,dt,dz,hEvapFront,
     &               REvaDeep,theta,qRot,
     &               LAYER,FLGENU,COFGEN,HTABLE,
     &               THETLO,ALFAMG,THETAR,THETAS,
     &               Evp,qthetaEF)
#endif

C --- updating coefficients tridiagonal Thomas matrix
      i = 1
      if (ftoph) then

C ---   head controlled boundary 
        thomb(i) = - thomc(i)+dimoca(i)+dt*kmean(1)/disnod(i)/dz(i)
        thomf(i) = -theta(i)+thetm1(i)-qimmob(i)+qimold(i)+
     &             dimoca(i) * h(i) +
     &             dt/dz(i)*(+kmean(i)-kmean(i+1)-qrot(i)+adsflu(i)
     &             - qdra(1,i) - qdra(2,i) - qdra(3,i) - qdra(4,i)
     &             - qdra(5,i)
#ifdef SWAPS
C --- add EVP(i): only defined in SWAPS
     &             - Evp(i)
#endif
     &             ) + dt*kmean(i)*hsurf/disnod(i)/dz(i)
      else

C ---   flux controlled boundary
        thomb(i) = - thomc(i) + dimoca(i)
        thomf(i) = -theta(i) + thetm1(i)-qimmob(i)+qimold(i)+
     &             dimoca(i) * h(i) +
     &             dt/dz(i)*(-qtop-kmean(i+1)-qrot(i)+adsflu(i)
     &             - qdra(1,i) - qdra(2,i) - qdra(3,i) - qdra(4,i)
     &             - qdra(5,i)
#ifdef SWAPS
C --- add EVP(i): only defined in SWAPS
     &             - Evp(i)
#endif
     &             )
      endif
          
      do 550 i = 2,numnod
        thomb(i) = thomb(i) - capold(i) + dimoca(i)
        thomf(i) = thomf(i) - capold(i) * hold(i) + dimoca(i) * h(i)
     &             - theta(i) + theold(i)-qimmob(i)+qimold(i)
 550  continue

C --- update qbot in case of free drainage (JVD,JK 18/11/1999)
      if (swbotb.eq.7) then 
        i = numnod
        k(i) = hconode(i,theta(i),LAYER,FLGENU,COFGEN,
     &                 THETAR,THETAS,THETLO,THETHI,KTABLE,
     &                 KSE99,KSAT,SWMOBI,FMOBIL)
	  qbot = -1.0d0 * k(i)
        thomf(i) = dimoca(i)*h(i) -qimmob(i)+
     &             dt/dz(i)*(kmean(i)+qbot-qrot(i)+adsflu(i)
     &             -qdra(1,i)-qdra(2,i)-qdra(3,i)-qdra(4,i)-qdra(5,i))
      endif

      i = numnod
      if (swbotb .eq. 1 .or. swbotb .eq. 5) then
        thomb(i) = 1.0d0
        thomf(i) = h(i)
      endif

      if (swbotb .eq. 8 .and. h(i) .gt. (-disnod(i+1)+0.001d0)) then
        thoma(i) = 0.0d0
        thomb(i) = 1.0d0
        thomf(i) = - disnod(i+1)
      endif

C --- SWAPS: Use tridag to be efficient!!!
C --- Save old values of h, Theta and DiMoCa in Old.
      do i = 1, NumNod
        hOld(i)   = h(i)
        TheOld(i) = Theta(i)
        CapOld(i) = DiMoCa(i)
      enddo

C --- Solve the tridiagonal matrix.
      call TriDag(NumNod, thoma, thomb, thomc, thomf, h,ErrorCode)
      if (ErrorCode .eq. 0) then

C --- first solution
CMA/JH Because TriDag is used this has been simplified to one loop
      do 590 i = 1,numnod
        lay     = layer(i)
        if (SWMOBI.eq.1) then
          if (h(i).gt.-1.0d0) then
            fmobil(i) = fm1(lay)+(0.0-pf1(lay))*slopfm(lay)
          else
            fmobil(i) = fm1(lay)+(log10(abs(real(h(i))))-pf1(lay))*
     &                  slopfm(lay)
          endif
          fmobil(i) = min (1.0d0,fmobil(i))
          fmobil(i) = max (0.3d0,fmobil(i))
          thetar(i) = cofgen(1,lay)*fmobil(i)
          thetas(i) = cofgen(2,lay)*fmobil(i)
          qimold(i) = qimmob(i)
          qimmob(i) = (fmobm1(i)-fmobil(i))*thetim(lay)
        endif
        theta(i) = thenode(i,h(i),LAYER,FLGENU,COFGEN,HTABLE,
     &             THETLO,ALFAMG,THETAR,THETAS)
        dimoca(i) = dmcnode(i,h(i),LAYER,FLGENU,COFGEN,THETLO,THETHI,
     &              DMCH,DMCC,THETAS,THETAR,ALFAMG,SWMOBI,FM1,PF1,
     &              SLOPFM,HTABLE,THETA)
 590  continue

C --- only one iteration if SWNUMS = 1
      if (swnums .eq. 1) goto 640

C --- check on convergence of solution
      do 600 i = 1,numnod
        if (abs(h(i)) .lt. 10.0d0) then
          deno = 10.0d0
        else
          deno = h(i)
        endif
        if (abs(theold(i)-theta(i)) .gt. thetol
     &      .or. abs( (h(i)-hold(i)) / deno ) .gt. 0.01d0
     &      .or. h(1) .gt. 50.0d0) then

C --- SWAP: do this iteration regardless of the value of FlAdjust...
C --- SWAPS: FlAdjust is set in BOCOTOP, iterate only when not set (= false)
#ifdef SWAP
          FlAdjust = .false.
#endif

          if ((numbit .lt. 6).and.(.NOT.FlAdjust)) then

C --- perform the iteration
            numbit = numbit + 1

C --- NOTE: kmean(1) calculated here
            call BOCOTOP(POND,REVA,NIRD,DT,QTOP,QBOT,QDRTOT,
     & KMEAN,K,H,DISNOD,LAYER,FLGENU,COFGEN,THETA,THETIM,DZ,
     & HTABLE,THETLO,THETHI,KTABLE,ALFAMG,FTOPH,ARFLUX,THETAR,
     & THETAS,KSE99,KSAT,SWMOBI,FMOBIL,VOLSAT,VOLACT,ACRACK,QCRACK,
     & FLTSAT,HSURF,FLLAST,FLRAIC,DTMIN,WTOPLAT,FLREVA,NUMNOD,QROSUM,
     & QCOMP,FPEGWL,NPEGWL,Q,SWBOTB,GWLINP
#ifdef SWAPS
C --- SWAPS variables in BOCOTOP call, not used in SWAP
     & ,hAtm,FlAdjust,revadeep
#endif
     & )
            goto 700
          else

#ifdef SWAPS
C --- SWAPS: return control to Timer by setting FlAdjust = .TRUE.
C     and reset all values to previous SOIL time step values
C     and goto 410 (which is now at bottom) to exit routine
C     However, if timestep is already dtMin (FldtMin=.TRUE.) then
C     no convergence is reached and the program continues to next
C     timestep.
            FlAdjust = .TRUE.
            if (.NOT.FldtMin) then
C --- Note, this loop is closed later with an endif, same as for SWAP
#endif

#ifdef SWAP
            if (dt .gt. 3.0*dtmin) then
C --- SWAP: start headcalc again with smaller timestep
              dt = dt / 3.0
              if (FLLAST) FLLAST=.FALSE.
              if (FLRAIC) FLRAIC=.FALSE.
C --- Note, this loop is closed later with an endif, same as for SWAPS
#endif

              if (SWMOBI.eq.1) then
                qimmob(1) = 0.0
                fmobil(1) = fmobm1(1)
                thetar(1) = cofgen(1,1)*fmobil(1)
                thetas(1) = cofgen(2,1)*fmobil(1)
              endif
              h(1)      = hm1(1)
              theta(1)  = thetm1(1)
              k(1)      = hconode(1,theta(1),LAYER,FLGENU,COFGEN,
     &                    THETAR,THETAS,THETLO,THETHI,KTABLE,
     &                    KSE99,KSAT,SWMOBI,FMOBIL)
              dimoca(1) = dmcnode(1,hm1(1),LAYER,FLGENU,COFGEN,THETLO,
     &                    THETHI,DMCH,DMCC,THETAS,THETAR,ALFAMG,
     &                    SWMOBI,FM1,PF1,SLOPFM,HTABLE,THETA)
              do 620 j = 2,numnod
                lay = layer(j)
                if (SWMOBI.eq.1) then
                  qimmob(j) = 0.0
                  fmobil(j) = fmobm1(j)
                  thetar(j) = cofgen(1,lay)*fmobil(j)
                  thetas(j) = cofgen(2,lay)*fmobil(j)
                endif
                h(j)      = hm1(j)
                theta(j)  = thetm1(j)
                k(j)      = hconode(j,theta(j),LAYER,FLGENU,COFGEN,
     &                      THETAR,THETAS,THETLO,THETHI,KTABLE,
     &                      KSE99,KSAT,SWMOBI,FMOBIL)
                dimoca(j) = dmcnode(j,h(j),LAYER,FLGENU,COFGEN,THETLO,
     &                      THETHI,DMCH,DMCC,THETAS,THETAR,ALFAMG,
     &                      SWMOBI,FM1,PF1,SLOPFM,HTABLE,THETA)
                kmean(j)  = 0.5*(k(j) + k(j-1))
 620          continue
              kmean(numnod+1) = k(numnod)
         call BOCOTOP (POND,REVA,NIRD,DT,QTOP,QBOT,QDRTOT,
     &          KMEAN,K,H,DISNOD,LAYER,FLGENU,COFGEN,THETA,THETIM,DZ,
     &          HTABLE,THETLO,THETHI,KTABLE,ALFAMG,FTOPH,ARFLUX,THETAR,
     &          THETAS,KSE99,KSAT,SWMOBI,FMOBIL,VOLSAT,VOLACT,ACRACK,
     &          QCRACK,FLTSAT,HSURF,FLLAST,FLRAIC,DTMIN,WTOPLAT,FLREVA,
     &          NUMNOD,QROSUM,QCOMP,FPEGWL,NPEGWL,Q,SWBOTB,GWLINP
#ifdef SWAPS
C --- Extra SWAPS Parameters in BOCOTOP call
     &          ,hAtm,FlAdjust,revadeep
#endif
     &          )
              goto 410
            else

C --- no convergence !! continue, but store daynumber
#ifdef SWAPS
              FlAdjust = .FALSE.
#endif

              if (dnocon .eq. 0) dnocon = INT(T)

#ifdef SWAPS
C --- SWAPS error handling
              write(ErrTxt,10)T
 10   format('No convergence of soil moisture calculations at',
     &       ' time: ',F8.4)           
              call ErrorSetError(5010,ErrTxt,1,ErrSrc)
#endif

C --- following endif closes both SWAPS - SWAP loops defined in #ifdef
C     code above
            endif
          endif
        endif
 600  continue
 640  continue

C --- SWAPS: Skip these calculations if returning control to Timer 
C     (FlAdjust=TRUE and Fldtmin=FALSE)
C --- flux at soil surface according to Darcy.
      if (ftoph) then
        qDarcy = -kmean(1)*((hsurf - h(1))/disnod(1)+1.0d0)

#ifdef SWAPS
CMS 20-JUL-99 qDarcy is invalid if no convergence or hAtm close to zero
CMS 20-JUL-99 may cause negative reva, therefore reset to zero here
         if ((qDarcy.lt.0.0).and.(qTop.gt.0.0)) qDarcy = 0.0
#endif

      else
        qDarcy = qtop
      endif

C --- Determine top boundary flux in case of ponding or limited evaporation
      if (ftoph) qtop = qDarcy

#ifdef SWAPS
      if (flreva) then
        reva = qtop + (pond - wtoplat)/dt
     &                   + (arflux+nird)*(1.0d0-acrack)
      endif
#endif

C --- adjust conductivities
      k(1) = hconode(1,theta(1),LAYER,FLGENU,COFGEN,THETAR,THETAS,
     &       THETLO,THETHI,KTABLE,KSE99,KSAT,SWMOBI,FMOBIL)
      do 650 i = 2,numnod
        k(i)     = hconode(i,theta(i),LAYER,FLGENU,COFGEN,THETAR,
     &             THETAS,THETLO,THETHI,KTABLE,KSE99,KSAT,SWMOBI,
     &             FMOBIL)
        kmean(i) = 0.5*(k(i)+k(i-1))
 650  continue
      kmean(numnod+1) = k(numnod)

C --- SWAPS modification - conceptual theta at surface calculated here
C     for output only
#ifdef SWAPS
      thetaSurf  = thenode(1,hAtm,LAYER,FLGENU,COFGEN,HTABLE,
     &            THETLO,ALFAMG,THETAR,THETAS)
      REvaDeep = qthetaEF
 410  continue
#endif

      end if ! ErrorCode above 590
      end if ! ErrorCode


      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE INITSOL(
#ifdef SWAP
C --- specific SWAP parameter list for subroutine INITSOL
     & NUMLAY,FLGENU,THETHI,THETLO,HTABLE,KTABLE,THETSL,
     & KSAT,DMCH,DMCC,COFGEN,KSE99,NUMNOD,DZ,Z,
     & DISNOD,INPOLB,INPOLA,BOTCOM,LAYER,THETAS,THETAR,SWHYST,INDEKS,
     & ALFAMG,SWINCO,THETA,THETAI,H,SWBOTB,HGITAB,T,HI,GWLTAB,GWLI,GWL,
     & GWLBAK,SWSOLU,CMLI,CML,CIL,SAMINI,PSAND,PSILT,PCLAY,ORGMAT,CL,
     & CMSY,KF,FREXP,CISY,VOLM1,VOLACT,VOLINI,VOLSAT,CREF,THETM1,HM1,
     & DIMOCA,K,KMEAN,THETIM,SWMOBI,FMOBIL,KMOBIL,PF1,FM1,PF2,FM2,
     & SLOPFM,QIMMOB,SWCRACK,SHRINA,MOISR1,MOISRD,ZNCRACK,SHRINB,SHRINC,
     & NNCRACK,ACRACK,QCRACK,RAPDRA,WTOPLAT,ADSFLU,CKWM1,DT,
     & POND,BDENS,FQUARTZ,FCLAY,FORG
#endif
#ifdef SWAPS
C --- specific SWAPS parameter list for subroutine INITSOL
     & NUMLAY,FLGENU,THETHI,THETLO,HTABLE,KTABLE,THETSL,KSAT,DMCH,DMCC,
     & COFGEN,KSE99,NUMNOD,DZ,Z,DISNOD,INPOLB,INPOLA,BOTCOM,LAYER,
     & THETAS,THETAR,SWHYST,INDEKS,ALFAMG,SWINCO,THETA,THETAI,H,SWBOTB,
     & HGITAB,T,HI,GWLTAB,GWLI,GWL,GWLBAK,SWSOLU,CMLI,CML,CIL,SAMINI,CL,
     & CMSY,BDENS,KF,FREXP,CISY,VOLM1,VOLACT,VOLINI,VOLSAT,CREF,THETM1,
     & HM1,DIMOCA,K,KMEAN,THETIM,SWMOBI,FMOBIL,KMOBIL,PF1,FM1,PF2,FM2,
     & SLOPFM,QIMMOB,SWCRACK,SHRINA,MOISR1,MOISRD,ZNCRACK,SHRINB,SHRINC,
     & THETCR,NNCRACK,ACRACK,QCRACK,RAPDRA,WTOPLAT,ADSFLU,CKWM1,ARFLUX,
     & NIRD,PCLAY,PSAND,SwTherm,PercentClay,PercentSand,ZSpec,NSpec,
     & SpecFrac,VolPart,VolRoot,VolRes,VolPartRes,VolPartSat,crackw,DT,
     & POND
***** End SWAPS variables
***** NOTE: crackw is NOT a SWAPS variable but it is NOT initialised in
***** SWAP so assume it needs initialising and add to arguments
#endif
     & )
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : initialize soil profile data
C     Subroutines called : RDSOL,WATCON
C     Functions called   : PRHNODE,THENODE,DMCNODE,HCONODE,AFGEN
C     File usage         : -
C     Differences SWAP/SWAPS:
C     - function parameter list + calculations
C     - Error handling
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
C --- Common INTs
      INTEGER THETHI(MAHO),THETLO(MAHO),BOTCOM(MAHO),LAYER(MACP)
      INTEGER NUMLAY,NUMNOD,SWHYST,SWINCO,SWBOTB,SWSOLU,NNCRACK
      INTEGER SWMOBI,SWCRACK, INDEKS(MACP)

C --- Common REALs
      REAL*8 HGITAB(50),T,GWLTAB(732),GWL,KF,SHRINA,MOISR1,MOISRD
      REAL*8 SHRINB,SAMINI,SHRINC,ACRACK,QCRACK,RAPDRA,WTOPLAT,KMOBIL
      REAL*8 AFGEN,FREXP,CKWM1,GWLI,CREF,GWLBAK(4),POND,DT,ARFLUX,NIRD
      REAL*8 CRACKW,ZNCRACK
      REAL*8 FMOBIL(MACP),DZ(MACP),Z(MACP),DISNOD(MACP+1),INPOLA(MACP)
      REAL*8 INPOLB(MACP),CMLI(MACP),CML(MACP),CIL(MACP),CL(MACP)
      REAL*8 CMSY(MACP),CISY(MACP),PF1(MAHO),FM1(MAHO),SLOPFM(MAHO)
      REAL*8 ADSFLU(MACP),PF2(MAHO),FM2(MAHO),QIMMOB(MACP),PCLAY(MAHO)
      REAL*8 PSAND(MAHO),BDENS(MAHO)

C --- Common REAL*8s
      REAL*8 PRHNODE,THENODE,DMCNODE,HCONODE,VOLM1,VOLACT,VOLINI,VOLSAT
      REAL*8 HTABLE(MAHO,99),KTABLE(MAHO,99),THETSL(MAHO),KSAT(MAHO)
      REAL*8 DMCC(MAHO,99),DMCH(MAHO,99),COFGEN(8,MAHO),KSE99(MAHO)
      REAL*8 THETAS(MACP),THETAR(MACP),ALFAMG(MACP),THETA(MACP)
      REAL*8 THETAI(MACP),H(MACP),HI(MACP),THETM1(MACP),HM1(MACP)
      REAL*8 DIMOCA(MACP),K(MACP+1),KMEAN(MACP+1),THETIM(MAHO)

      LOGICAL FLGENU(MAHO)

#ifdef SWAP
C -- SWAP variables
      REAL*8 PSILT(MAHO),FQUARTZ(MACP),FCLAY(MACP),FORG(MACP)
      REAL*8 ORGMAT(MAHO)
#endif

#ifdef SWAPS
C --- SWAPS variables
      INTEGER SwTherm,NSpec,NRoot
      REAL*8 THETCR(5),PercentClay(1:40),PercentSand(1:40),ZSpec
      REAL*8 VolPart,VolRoot,VolRes,VolPartRes,VolPartSat,SpecFrac
      LOGICAL FlError
#endif

C ----------------------------------------------------------------------
C --- local vars
      INTEGER LAY,NODE,I,I2
      REAL*8  beta1,beta2,func,deri

#ifdef SWAP
C --- SWAP variables
      REAL*8 DUMMY,GMINERAL
#endif

#ifdef SWAPS
C --- Variables for error handling SWAPS
      Character*80 ErrTxt,ErrSrc
#endif

      save
C ----------------------------------------------------------------------

#ifdef SWAPS
C --- Define source (routine name) of potential error in SWAPS     
      ErrSrc = 'InitSol'
#endif

C --- position of nodal points and distances between them
      Z(1)      = - 0.5*DZ(1)
      DISNOD(1) = - Z(1)
      do 20 NODE=2,NUMNOD
        Z(NODE)      = Z(NODE-1) - 0.5*(DZ(NODE-1)+DZ(NODE))
        DISNOD(NODE) = Z(NODE-1)-Z(NODE)
 20   continue
      DISNOD(NUMNOD+1) = 0.5*DZ(NUMNOD)

#ifdef SWAPS
C --- SWAPS specific call
      call SpecDep(ZSpec,NumNod,dz,z,NSpec,SpecFrac)
#endif

C --- interpolation values for calculation solute transport
      INPOLB(1) = 0.5*DZ(1)/DISNOD(2)
      do 22 NODE = 2,NUMNOD-1
        INPOLA(NODE) = 0.5*DZ(NODE)/DISNOD(NODE)
        INPOLB(NODE) = 0.5*DZ(NODE)/DISNOD(NODE+1)
22    continue
      INPOLA(NUMNOD) = 0.5*DZ(NUMNOD)/DISNOD(NUMNOD)

c --- immobile soil fractions
      if (SWMOBI.eq.1) then
        do 24 I = 1,NUMLAY
24      SLOPFM(I) = (FM2(I)-FM1(I))/(PF2(I)-PF1(I))
      else
        do 26 I = 1,NUMNOD
26      FMOBIL(I) = 1.0d0
        KMOBIL    = 0.0
      endif
      do 28 I = 1,NUMNOD
28    QIMMOB(I) = 0.0

C --- SWAP: per node: layer, saturated watercontent, dry bulk density
C --- volume fractions sand, clay and organic matter, hysteresis data
C --- SWAPS: layer of each node, saturated watercontent per nodalpoint
      LAY = 1
      do 40 NODE = 1,NUMNOD
        if (NODE.gt.BOTCOM(LAY)) LAY = LAY+1
        LAYER(NODE) = LAY

#ifdef SWAPS
C --- SWAPS only - get soil compostition for each comp
        if (SwTherm.eq.1) Then
           PercentClay(Node) = PCLAY(LAY)
           PercentSand(Node) = PSAND(LAY)
        endif
#endif

        if (FLGENU(LAY)) then
          THETAS(NODE) = COFGEN(2,LAY)
          THETAR(NODE) = COFGEN(1,LAY)
          if (SWHYST.eq.1) then

C --- wetting curve  
            INDEKS(NODE)  = 1
            ALFAMG(NODE)  = COFGEN(8,LAY)
          elseif (SWHYST.eq.0.OR.SWHYST.eq.2) then

C --- drying branch or simulation without hysteresis
            INDEKS(NODE)   = -1
            ALFAMG(NODE)  = COFGEN(4,LAY)
          endif
        else
          THETAS(NODE) = THETSL(LAY)
        endif

#ifdef SWAP
C --- SWAP calculations
          DUMMY = ORGMAT(LAY)/(1.0d0 - ORGMAT(LAY))
          GMINERAL = (1.0d0 - THETAS(NODE)) / (0.370d0 + 0.714*DUMMY)
          BDENS(LAY) = (1.0d0 + DUMMY) * GMINERAL
          FQUARTZ(NODE) = (PSAND(LAY) + PSILT(LAY))*GMINERAL/2.7
          FCLAY(NODE) = PCLAY(LAY)*GMINERAL/2.7
          FORG(NODE) = DUMMY*GMINERAL/1.4
#endif
 40   continue

C --- Set initial conditions
#ifdef SWAP
      POND = 0.0
#endif

      if (SWINCO.eq.0) then

C --- compute initial pressure heads from moisture content
        do 60 NODE = 1,NUMNOD
          THETA(NODE)  = THETAI(NODE)
          H(NODE) = PRHNODE (NODE,THETA(NODE),LAYER,DISNOD,H,
     &    FLGENU,THETAR,THETAS,COFGEN,ALFAMG,HTABLE)
 60     continue

C --- change h for last node
        if (SWBOTB.eq.5) then
          H(NUMNOD)     = AFGEN (HGITAB,50,T+1.0d0)
          THETA(NUMNOD) = THENODE (NUMNOD,H(NUMNOD),LAYER,
     &    FLGENU,COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
        endif
      elseif (SWINCO.eq.1) then

C --- pressure head profile is given 
        do 70 NODE = 1,NUMNOD
 70     H(NODE) = HI(NODE)

C --- in case of preferential flow, adjust MVG parameters
        if (SWMOBI.eq.1) then
          do 170 i = 1,numnod
            lay = layer(i)
            if (h(i).gt.-1.0d0) then
              fmobil(i) = fm1(lay)+(0.0d0-pf1(lay))*slopfm(lay)
            else
              fmobil(i) = fm1(lay)+(log10(abs(h(i)))-pf1(lay))*
     &                    slopfm(lay)
            endif
            fmobil(i) = min (1.0d0,fmobil(i))
            fmobil(i) = max (0.3d0,fmobil(i))
            thetar(i) = cofgen(1,lay)*fmobil(i)
            thetas(i) = cofgen(2,lay)*fmobil(i)
 170      continue
        endif
        do 180 node = 1,numnod

#ifdef SWAP
C --- SWAP uses THETA
          THETA(NODE) = THENODE (NODE,H(NODE),LAYER,FLGENU,
     &                  COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
#endif

#ifdef SWAPS
C --- SWAPS uses THETAI
          THETAI(NODE) = THENODE (NODE,H(NODE),LAYER,FLGENU,
     &                  COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
#endif

#ifdef SWAPS
CMS 16-JUL-99: SWAPS uses THETAI for error check
          THETA(NODE) = THETAI(NODE)
#endif

 180    continue
      elseif (SWINCO.eq.2) then

C --- pressure head profile is calculated 
        if (SWBOTB.eq.1) then  
          GWL = AFGEN (GWLTAB,732,T+1.0d0)
        else
          GWL = GWLI
        endif
        if (GWL.gt.0.0) POND = GWL
        do 80 I = 1,NUMNOD
          H(I) = GWL - Z(I)

C --- in case of preferential flow, adjust MVG parameters
          if (SWMOBI.eq.1) then
            lay = layer(i)
            if (h(i).gt.-1.0d0) then
              fmobil(i) = fm1(lay)+(0.0d0-pf1(lay))*slopfm(lay)
            else
              fmobil(i) = fm1(lay)+(log10(abs(h(i)))-pf1(lay))*
     &                    slopfm(lay)
            endif
            fmobil(i) = min (1.0d0,fmobil(i))
            fmobil(i) = max (0.3d0,fmobil(i))
            thetar(i) = cofgen(1,lay)*fmobil(i)
            thetas(i) = cofgen(2,lay)*fmobil(i)
          endif

#ifdef SWAP
          THETA(I) = THENODE (I,H(I),LAYER,FLGENU,
     &               COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
#endif

#ifdef SWAPS
          THETAI(I) = THENODE (I,H(I),LAYER,FLGENU,
     &               COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
          THETA(I) = THETAI(I)
#endif

 80     continue 
      endif

C --- water volume in totally saturated profile
      VOLSAT = 0.0

#ifdef SWAPS
      do NODE = 1,NUMNOD
 82   VOLSAT = VOLSAT + DZ(NODE)*THETSL(LAYER(NODE))
      enddo
#endif

#ifdef SWAP
      do 82 NODE = 1,NUMNOD
 82   VOLSAT = VOLSAT + DZ(NODE)*THETSL(LAYER(NODE))
#endif

C --- save initial moisture contents and pressure heads
      do 90 NODE = 1,NUMNOD
        THETM1(NODE) = THETA(NODE)
        HM1(NODE)    = H(NODE)
 90   continue

C --- hydr. conductivities, differential moisture capacities
C     and geometrical mean hydr. conductivities for each node
      DIMOCA(1) = DMCNODE(1,H(1),LAYER,FLGENU,COFGEN,THETLO,THETHI,
     &            DMCH,DMCC,THETAS,THETAR,ALFAMG,SWMOBI,FM1,
     &            PF1,SLOPFM,HTABLE,THETA)
      K(1) = HCONODE (1,THETA(1),LAYER,FLGENU,COFGEN,THETAR,THETAS,
     &       THETLO,THETHI,KTABLE,KSE99,KSAT,SWMOBI,FMOBIL)
      do 50 NODE = 2,NUMNOD
        DIMOCA(NODE) = DMCNODE (NODE,H(NODE),LAYER,FLGENU,COFGEN,
     &                 THETLO,THETHI,DMCH,DMCC,THETAS,THETAR,
     &                 ALFAMG,SWMOBI,FM1,PF1,SLOPFM,HTABLE,THETA)
        K(NODE) = HCONODE (NODE,THETA(NODE),LAYER,FLGENU,COFGEN,
     &            THETAR,THETAS,THETLO,THETHI,KTABLE,KSE99,KSAT,
     &            SWMOBI,FMOBIL)
        KMEAN(NODE) = 0.5*(K(NODE)+K(NODE-1))
 50   continue
      KMEAN(NUMNOD+1) = K(NUMNOD)

C --- initial solute profiles
      if (SWSOLU.eq.1) then
        SAMINI = 0.0
        do 4 I = 1,NUMNOD
          LAY = LAYER(I)
          if (FMOBIL(I).gt.0.995d0) then
            CML(I) = CMLI(I)
            CIL(I) = 0.0   
            CL(I)  = CML(I)
            CMSY(I) = (THETA(I)*CML(I)+BDENS(LAY)*KF*CREF*(CML(I)
     &                /CREF)**FREXP)
            CISY(I) = 0.0
          else
            CML(I) = CMLI(I)
            CIL(I) = CML(I)
            CL(I)  = CML(I)
            CMSY(I) = (THETA(I)*CML(I)+BDENS(LAY)*KF*CREF*((CML(I)
     &                /CREF)**FREXP) * FMOBIL(I))
            CISY(I) = ((THETIM(LAY)*CIL(I)+BDENS(LAY)*KF*CREF
     &                *(CIL(I)/CREF)**FREXP)* (1.0d0-FMOBIL(I)))
          endif
          SAMINI = SAMINI + (CMSY(I)+CISY(I)) * DZ(I)
 4      continue
      endif

C --- flow through soil cracks
      if (SWCRACK.eq.1) then
        shrinc = 0.7d0
        beta2  = -1.0/moisr1*log((moisrd+moisr1-shrinc*moisr1)/shrina)      
        beta1  = beta2+1.0d0

 445    if (abs(beta2-beta1).gt.0.001d0) then
          beta1 = beta2
          func  = (shrina+shrina*moisr1*beta1)*exp(-beta1*moisr1)-moisrd
          deri  = (2.0*shrina*moisr1-shrina*moisr1*moisr1*beta1)*
     &            exp(-beta1*moisr1)
          beta2 = beta1-func/deri
          goto 445
        endif
        shrinb = beta2
        shrinc = 1.0d0+shrina*shrinb*exp(-shrinb*moisr1) 

C --- number node for evaluating shrinkage characteristic
        i = 1
 460    if (z(i).gt.zncrack) then
          i = i+1
          goto 460
        endif
        nncrack = i
      else

C ---   set variables to zero
        acrack  = 0.0
        qcrack  = 0.0
        rapdra  = 0.0
        wtoplat = 0.0
        do 443 i = 1,numnod
 443    adsflu(i) = 0.0
      endif         

C --- initial total volume
#ifdef SWAP
C --- SWAP only
      ARFLUX = 0.0
      NIRD   = 0.0
      CRACKW = 0.0
#endif

#ifdef SWAPS
C --- SWAPS only        
      NRoot = 0
#endif

#ifdef SWAP
C --- call WATCON with SWAP parameters
      call WATCON (VOLM1,VOLACT,NUMNOD,THETA,DZ,FMOBIL,LAYER,
     & THETIM,SWCRACK,CKWM1,CRACKW,WTOPLAT,QCRACK,ARFLUX,NIRD,ACRACK,
     & RAPDRA,DT)
#endif

#ifdef SWAPS
C --- call WATCON with SWAPS parameters
      call WATCON (VOLM1,VOLACT,NUMNOD,THETA,DZ,FMOBIL,LAYER,
     & THETIM,SWCRACK,CKWM1,CRACKW,WTOPLAT,QCRACK,ARFLUX,NIRD,ACRACK,
     & RAPDRA,DT,NSpec,SpecFrac,NRoot,thetaR,thetaS,VolPart,VolRoot,
     & VolRes,VolSat,VolPartRes,VolPartSat)
#endif

      VOLINI = VOLACT+POND

C --- initialize registration of last four groundwaterlevels, for noting
C --- oscillation
       do 454 i2=1,4
         gwlbak(i2)= 0.0
 454   continue

#ifdef SWAPS
C --- SWAPS error checking
      FlError = .FALSE.
      do Node = 1,NumNod
         if (thetaI(Node).gt.THETAS(Node)) Then
            write(ErrTxt,1000) Node,thetaI(Node),THETAS(Node)
 1000 format('Node: ',I2,
     &' Initial moisture ',F6.4,' > saturated ',F6.4)
            FlError = .TRUE.
            call ErrorSetError(5001,ErrTxt,3,ErrSrc) 
         endif
         if (thetaI(Node).lt.thetaR(Node)) Then
            write(ErrTxt,1001) Node,thetaI(Node),thetaR(Node)
 1001 format('Node: ',I2,
     &'Initial moisture ',F6.4,' < residual ',F6.4)
            FlError = .TRUE.
            call ErrorSetError(5002,ErrTxt,3,ErrSrc)
         endif
      enddo
      if (flError) call SetModelStatus(100)
#endif

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE OUTDRF (OUTNAM,DRF,HDRFD,ENVSTR,STASTR,DAYNR,YEAR,
     &  NRPRI,NRSRF,CQDRAIN,CQDRD,CRUNO,CQDRAR,T)
C ----------------------------------------------------------------------
C     Date               : 10/6/99
C     Purpose            : write drainage fluxes, surface runoff, rapid
C                          drainage to  OUTNAM.DRF file
C     Subroutines called : 
C     Functions called   : -
C     File usage         : OUTNAM.DRF
C     Differences SWAP/SWAPS: 
C     - Extra SWAPS integer in file handling
C     - Output format (for GUI SWAP)
C ---------------------------------------------------------------------
      IMPLICIT NONE

C --- Global
      INTEGER   DRF,DAYNR,YEAR,D,M,NRPRI,NRSRF
      REAL*8    CQDRAIN(5),CQDRD,CRUNO,CQDRAR,T
      CHARACTER OUTNAM*8,FILNAM*12,ENVSTR*50,STASTR*7

C --- Local
      INTEGER   LEVEL
      REAL*8    C1QDRAIN(5),C1QDRD,C1RUNO,C1QDRAR
      LOGICAL   HDRFD

c
c IJke
c
      INTEGER ios

      save
C ----------------------------------------------------------------------
      if (.NOT.HDRFD) then

C --- compose filename
        call SHIFTR (OUTNAM)
        FILNAM = OUTNAM//'.DRF'
        call SHIFTL (FILNAM)
        call SHIFTR (ENVSTR)

C --- open output file
#ifdef SWAP
        call Findunit (10,DRF)
#endif
        open (DRF,FILE = ENVSTR//FILNAM, STATUS = STASTR, 
     &  ACCESS = 'SEQUENTIAL',iostat=ios)

C --- write header
        if (nrpri .eq. 1) then
          write (DRF,100) NRSRF
        else
          write (DRF,101) NRSRF
        endif


C --- Output format
 100    format (
     &'* CUMULATIVE DRAINAGE FLUXES, SURFACE RUNOFF AND RAPID',
     &' DRAINAGE:',/,'*',/,
     &'* Total number of levels for drainage fluxes :',I3,/,'*',/
     &'* First level (primary system) NOT included in sw-reservoir',/,
     &'*',/)

 101    format (
     &'* CUMULATIVE DRAINAGE FLUXES, SURFACE RUNOFF AND RAPID',
     &' DRAINAGE:',/,'*',/,
     &'* Total number of levels for drainage fluxes :',I3,/,'*',/
     &'* First level (primary system) IS included in sw-reservoir',/,
     &'*')

        write (DRF,11)
 11     format (
     &'* Meaning of symbols:',/,
     &'* - CQD(*)  : Drainage fluxes all levels',/,
     &'* - CQDRD   : Total of drainage fluxes into surface',
     &            ' water reservoir (SWSRF>=1)',/,
     &'* - CRUNO   : Surface runoff (increment can be <0.0)',/,
     &'* - CQDRAR  : Rapid drainage (increment always >= 0.0)',/,
     &'* - cum     : cumulative value',/,
     &'* - inc     : incremental value',/,'*',/, 
     &'* DATE            CQD(1)  CQD(2)  CQD(3)  CQD(4)  CQD(5)',
     &'   CQDRD   CRUNO  CQDRAR',/,
     &'* dd/mm/yyyy  nr    inc/    inc/    inc/    inc/    inc/',
     &'    inc/    inc/    inc/',/,
     &'*                    cum     cum     cum     cum     cum',
     &'     cum     cum     cum',/,
     &'*                   [cm]    [cm]    [cm]    [cm]    [cm]',
     &'    [cm]    [cm]    [cm]',/,
     &'* <============><======><======><======><======><======>',
     &'<======><======><======>')

        do 30 level=1,5
          c1qdrain(level)=0.0
 30     continue
        c1qdrd   = 0.0
        c1runo   = 0.0
        c1qdrar  = 0.0
        hdrfd    = .true.
      endif

C --- conversion of daynumber to date
      call NRTODA (YEAR,DAYNR,M,D)

C --- write output record
      write (DRF,20) D,M,YEAR,NINT(T),((cqdrain(level)-c1qdrain(level)),
     &  level=1,5),(cqdrd-c1qdrd),(cruno-c1runo),(cqdrar-c1qdrar),
     &  D,M,YEAR,NINT(T),(cqdrain(level),level=1,5),cqdrd,cruno,cqdrar

 20   format ('1',1X,I2,'/',I2.2,'/',2I4,8f8.2,/,
     &        '2',1X,I2,'/',I2.2,'/',2I4,8f8.1)

C --- store cumulative values
      do 50 level=1,5
          c1qdrain(level)=cqdrain(level)
 50   continue
      c1qdrd   = cqdrd
      c1runo   = cruno
      c1qdrar  = cqdrar

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE OUTSWB (OUTNAM,SWB,HSWBD,ENVSTR,STASTR,DAYNR,YEAR,
     & GWL,POND,wlstar,wls,WSTINI,wst,CQDRD,CRUNO,CQDRAR,CWSUPP,CWOUT,
     & NUMADJ,HWLMAN,VTAIR,OVERFL,SWSEC,SWMAN,NMPER,IMPEND,IMPER,
     & HBWEIR,T)
C ----------------------------------------------------------------------
C     Date               : 21/08/99
C     Purpose            : write surface water balance data to 
C                          OUTNAM.SWB file, surface water management
C                          data to OUTNAM.MAN. The files overlap
C     Subroutines called : 
C     Functions called   : -
C     File usage         : OUTNAM.SWB, OUTNAM.MAN
C     Differences SWAP/SWAPS: 
C     - SWAPS: extra parameter MAN + integer ios, file handling
C     - Output format
C     - Error handling
C ---------------------------------------------------------------------

      IMPLICIT NONE

C --- Global
      INTEGER   SWB,MAN,DAYNR,YEAR,D,M,swsec,swman(10),nmper,impend(10)
      INTEGER   imper,numadj,ios
      REAL*8    GWL,POND,WLSTAR,WSTINI,WLS,WST,CQDRD,CRUNO
      REAL*8    CQDRAR,CWSUPP,CWOUT,hwlman,vtair,hbweir(nmper),t
      CHARACTER OUTNAM*8,FILNAM*12,ENVSTR*50,STASTR*7
      LOGICAL   OVERFL

C --- Local
      REAL*8    GWLEV,CQDRF1,C1WSUPP,C1WOUT,DELBAL
      CHARACTER SPC*1
      LOGICAL   HSWBD

#ifdef SWAPS
      PARAMETER (MAN=99)
c
c IJke
c
      INTEGER swl
#endif

      save
C ----------------------------------------------------------------------

C ---   determine which management period the model is in:
      if (SWSEC .eq. 2) then
        imper   = 0
   5    imper   = imper + 1

#ifdef SWAP
C --- Error handling SWAP
        if (imper .gt. nmper) stop 'OUTSWB: err. sw-management periods'
#endif

#ifdef SWAPS
C --- Error handling SWAPS
        if (imper .gt. nmper) then
            call SetModelStatus(100)
* gui - ma       stop 'OUTSWB: err. sw-management periods'
            call SetFatalError(9999,'err. sw-management periods',
     &           'OUTSRF')
            return
        endif
#endif

        if (int(T) .gt. impend(imper)) goto 5
      endif

C --- add ponding to groundwater level, if gwl at soil surface:
      if (abs(gwl) .lt. 1.0d-7) then
          gwlev = pond
      else
          gwlev = gwl
      endif
      if (.NOT.HSWBD) then

C --- compose filename
        call SHIFTR (OUTNAM)
        FILNAM = OUTNAM//'.SWB'
        call SHIFTL (FILNAM)

#ifdef SWAP
        call SHIFTR (ENVSTR)

C --- open output file
        call Findunit (10,SWB)
#endif

        open (SWB, FILE=ENVSTR//FILNAM, STATUS =STASTR,
     &        ACCESS='SEQUENTIAL', iostat=ios)

C --- write header
        write (SWB,10)
        write (SWB,11)

 10     format (
     &'* Surface water system:',/,'*',/,
     &'* - GWL           : groundwater level + ponding (<0. = below ',
     &'s.s)',/,
     &'* - WLST          : target level,  aut. weir (<0. = b.s.s.) ',/,
     &'* - WLST          : weir crest,   fixed weir (<0. = b.s.s.) ',/,
     &'* - WLS           : surface water level (<0. = b.s.s.) ',/,
     &'* - WST           : storage in sw-reservoir, per unit area of',
     &'* subcatchment',/,
     &'* - D+RO+R        : drainage fluxes (>0. = into sw) + runoff +',
     &'* rapid drainage')

 11     format (
     &'* - QSUPP         : external supply to sw-reservoir',/, 
     &'* - QOUT          : outflow from sw-reservoir',/,'*',/,
     &'*DATE              GWL   WLST    WLS   WST D+RO+R  QSUPP',
     &'   QOUT D+RO+R  QSUPP   QOUT',/,
     &'*dd/mm/yyyy  nr                               inc    inc',
     &'    inc    cum    cum    cum',/,
     &'*                 [cm]   [cm]   [cm]  [cm]   [cm]   [cm]',
     &'   [cm]   [cm]   [cm]   [cm]',/,
     &'*<============><=====><=====><=====><====><=====><=====>',
     &'<=====><=====><=====><=====>')

C --- write record for initial state
        write (SWB,19) gwlev,wlstar,wls,wst,(0.),(0.),(0.),(0.),
     &   (0.),(0.)

 19     format ('*>> initial    ',3f7.1,f6.1,6f7.2)

C --- compose filename for management data
#ifdef SWAP
        if (SWSEC .eq. 2) then
#endif

#ifdef SWAPS
        if (SWSEC .eq. 1) then
#endif

          call SHIFTR (OUTNAM)
          FILNAM = OUTNAM//'.MAN'
          call SHIFTL (FILNAM)
          call SHIFTR (ENVSTR)

C --- open output file for management
#ifdef SWAP
          call Findunit (10,MAN)
#endif

          open (MAN,FILE = ENVSTR//FILNAM,STATUS =STASTR,
     &    ACCESS = 'SEQUENTIAL',iostat=ios)

          write (MAN,20)
          write (MAN,21)

 20       format (
     &'* Surface water management:',/,'*',/,
     &'* - SWMAN         : type of weir (f = fixed; a = automatic)',/
     &'* - GWL           : groundwater level + ponding (<0. = below ',
     &'s.s)',/,
     &'* - HWLMAN        : pressure head used for target level (cm)',/,
     &'* - VTAIR         : total air volume in soil profile (cm)',/
     &'* - WLST          : target level of autom. weir (<0. = b.s.s.) '
     & ,/,
     &'*              or : crest  level of fixed  weir (<0. = b.s.s.) '
     & ,/,
     &'* - WLS           : surface water level (<0. = b.s.s.) ',/,
     &'* - QOUT          : surface water outflow (<0. = supply)',/,
     &'* - NUMADJ        : number of adjustments of target level',/,
     &'* - OVERFL        : flag for overflow of aut. weir (o)',/,
     &'* - CREST         : Crest level (<0. = b.s.s.)',/,'*')
 21       format (
     &'*DATE           SWMAN     GWL HWLMAN  VTAIR   WLST    WLS   QOUT'
     &,' NUMADJ OVERFL  CREST',/,
     &'*dd/mm/yyyy  nr          [cm]   [cm]   [cm]   [cm]   [cm]   [cm]'
     &'   [cm]',/,
     &'*<============><=====><=====><=====><=====><=====><=====><=====>'
     &,'<=====><=====><=====>')

        endif
        cqdrf1  = 0
        c1wsupp = 0
        c1wout  = 0
        HSWBD   = .true.
        return
      endif

C --- water balance error (water creation)
      delbal = (wst + cwout) - (wstini + cqdrd+cruno+cqdrar + cwsupp)
      if (delbal .gt. 0.05d0) then

#ifdef SWAP
         write(6,1001) delbal
#endif

 1001    format(/' >>Error in cumulative water balance of sw:',
     &     f5.2)
      endif

C --- write output record OUTNAM.SWB
C --- conversion of daynumber to date
      call NRTODA (YEAR,DAYNR,M,D)
      write (SWB,30) D,M,YEAR,NINT(T),gwlev,wlstar,wls,wst,
     &  (cqdrd+cruno+cqdrar-cqdrf1),(cwsupp-c1wsupp),(cwout-c1wout),
     &  (cqdrd+cruno+cqdrar),cwsupp,cwout
 30   format (1x,I2,'/',I2.2,'/',2I4,3f7.1,f6.1,6f7.2)

C --- write output record OUTNAM.MAN
      if (swsec .eq. 2) then
        if (overfl) then
          spc = 'o'
        else
          spc = '-'
        endif
        if (SWMAN(imper) .eq. 1) then
          write (MAN,31) D,M,YEAR,NINT(T),gwlev,wlstar,wls,
     &    ((cwout-c1wout)-(cwsupp-c1wsupp)),numadj,spc,hbweir(imper)
        else
          write (MAN,32) D,M,YEAR,NINT(T),gwlev,hwlman,vtair,wlstar,wls,
     &    ((cwout-c1wout)-(cwsupp-c1wsupp)),numadj,spc,hbweir(imper)
        endif
 31     format (1x,I2,'/',I2.2,'/',2i4,4x,'f',2x,f7.1,2('     - '),
     &          3f7.1,i7,6x,a1,f7.1)
 32     format (1x,I2,'/',I2.2,'/',2i4,4x,'a',2x,6f7.1,i7,6x,a1,f7.1)
      endif

C --- store cumulative values
      cqdrf1  = (cqdrd+cruno+cqdrar)
      c1wsupp = cwsupp
      c1wout  = cwout

      return
      end

C ----------------------------------------------------------------------
      REAL*8 FUNCTION PRHNODE (NODE,WCON,LAYER,DISNOD,H,FLGENU,
     &                THETAR,THETAS,COFGEN,ALFAMG,HTABLE)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculate pressure head at nodal      
C                          point node from water content wcon
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NODE,LAYER(MACP)

      REAL*8  DISNOD(MACP+1)
      REAL*8  WCON,H(MACP),COFGEN(8,MAHO),HTABLE(MAHO,99),ALFAMG(MACP)
      REAL*8  THETAR(MACP),THETAS(MACP) 

      LOGICAL FLGENU(MAHO)
C ----------------------------------------------------------------------
C --- local
      INTEGER posit,lay

      REAL*8  help
C ----------------------------------------------------------------------

C --- soil layer of node
      lay = layer(node)
      if (thetas(node)-wcon.lt.1.0d-6) then

C --- saturated pressure head
        if (node.eq.1) then
          prhnode = disnod(node)
        else
          prhnode = h(node-1) + disnod(node)
        endif
      else
        if (flgenu(lay)) then

C --- pressure head according to van Genuchten
          if (wcon-thetar(node).lt.1.0d-6) then
            prhnode = -1.0d12
          else

C --- first calculate the inverse of the sorptivity
            help = (thetas(node) - thetar(node)) / 
     $             (wcon - thetar(node))
C --- raise to the power 1/m
            help = help ** (1.0d0 / cofgen(7,lay))
C --- subtract one and raise to the power 1/n 
            help = (help - 1.0d0) ** (1.0d0 / cofgen(6,lay))
C ---       divide by alpha.
            prhnode = -1.0d0 * abs(help/alfamg(node))
          endif
        else
C --- interpolated pressure head from the table
          posit   = int(max (1.0,real(wcon*100.0)))
          prhnode = htable(lay,posit) + (htable(lay,posit+1)-
     $              htable(lay,posit))*(wcon-posit*0.01d0) / 0.01d0
        endif
      endif

      return
      end

C ----------------------------------------------------------------------
      real*8 function QHTAB (wlev,imper,hqhtab,qqhtab)
C-----------------------------------------------------------------------
C     Date               : 23/08/99
C     Purpose            : calculate surface water discharge from  
C                          surface water level, using table
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C     Verschillen Swaps en Swap: geen
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER imper,itab

      REAL*8  wlev, hqhtab(10,10),qqhtab(10,10),dwl
C ----------------------------------------------------------------------

      ITAB = 2
 100  if (wlev.lt.HQHTAB(IMPER,ITAB)) then 
        ITAB = ITAB+1
        goto 100
      endif
      dwl  = (wlev-HQHTAB(IMPER,ITAB))/
     &       (HQHTAB(IMPER,ITAB-1)-HQHTAB(IMPER,ITAB))
      qhtab = QQHTAB(IMPER,ITAB) + dwl *
     &        (QQHTAB(IMPER,ITAB-1)-QQHTAB(IMPER,ITAB))  

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE SLTBAL (DT,NIRD,CIRR,ARFLUX,CPRE,QBOT,CDRAIN,CML,
     &     NUMNOD,QDRTOT,RAPDRA,CRACKC,SQBOT,SQSUR,SQRAP,SQPREC,SQIRRIG)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculation of solute balance           
C     Subroutines called : -                                           
C     Functions called   : -                                           
C     File usage         : -
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NUMNOD

      REAL*8  DT,NIRD,CIRR,ARFLUX,CPRE,QBOT,CDRAIN,CML(MACP),QDRTOT
      REAL*8  RAPDRA,CRACKC,SQBOT,SQSUR,SQRAP,SQPREC,SQIRRIG
C ----------------------------------------------------------------------

C --- add time step fluxes to total cumulative values
      sqprec  = sqprec + arflux*cpre*dt
      sqirrig = sqirrig + nird*cirr*dt

      if (qbot .gt. 0.0) then
        sqbot = sqbot + qbot*cdrain*dt
      else 
        sqbot = sqbot + qbot*cml(numnod)*dt
      endif
      sqrap = sqrap + rapdra*crackc*dt
      sqsur = sqsur + qdrtot*cdrain*dt

      return
      end


C ----------------------------------------------------------------------
      SUBROUTINE SOLUTE (CPRE,NIRD,CIRR,DT,NUMNOD,LAYER,
     & INPOLA,INPOLB,CML,THETA,DDIF,LDIS,Q,DISNOD,THETSL,
     & Z,GAMPAR,RTHETA,BEXP,DECPOT,FDEPTH,KF,FREXP,FMOBIL,
     & TSCF,QROT,DZ,KMOBIL,CIL,QDRA,CMSY,THETIM,CISY,CL,DTSOLU,
     & DTMAX,TSoil,CPOND,QTOP,WTOPLAT,ACRACK,ADSFLU,CRACKC,
     & ARFLUX,SWCRACK,CRACKW,CKWM1,RAPDRA,QCRACK,POND,CREF,QDRTOT,
     & POROS,KFSAT,DECSAT,CDRAIN,HAQUIF,DECTOT,SAMPRO,SAMCRA,SAMAQ,
     & CRALEV,DIFDES,ROTTOT,SQDRA,SWBR,SDRAIN,BDENS,SWHEA,CSURF)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculation of solute concentrations
C     Subroutines called : -                                           
C     Functions called   : -                                           
C     File usage         : -                                           
C     Differences in Swaps and Swap:
C     - Swaps uses TSoil, SWAP changed to use TSoil as well
C     - SWAPS adjustment in timestep still with SWAP method
C     - Looks like this routine never worked!!!
C     - Extra SWAP parameters
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NUMNOD,LAYER(MACP),SWCRACK,SWBR,SWHEA

      REAL*8  CPRE,NIRD,CIRR,DT,INPOLA(MACP),INPOLB(MACP),CML(MACP)
      REAL*8  DDIF,LDIS,DISNOD(MACP+1),Z(MACP)
      REAL*8  GAMPAR,RTHETA,BEXP,DECPOT,FDEPTH(MAHO),KF,FREXP
      REAL*8  FMOBIL(MACP),TSCF,QROT(MACP),DZ(MACP),KMOBIL,CIL(MACP)
      REAL*8  QDRA(5,MACP),ARFLUX,BDENS(MAHO)
      REAL*8  CMSY(MACP),CISY(MACP),CL(MACP),DTSOLU,DTMAX,CREF
      REAL*8  CPOND,QTOP,WTOPLAT,ACRACK,ADSFLU(MACP),CRACKC
      REAL*8  CRACKW,CKWM1,RAPDRA,QCRACK,POND,QDRTOT,POROS,KFSAT
      REAL*8  DECSAT,CDRAIN,HAQUIF,DECTOT,CRALEV,DIFDES,ROTTOT
      REAL*8  SAMAQ,SAMCRA,SAMPRO,SQDRA,SDRAIN,CSURF

      REAL*8  THETA(MACP),Q(MACP+1),THETIM(MAHO)
      REAL*8  THETSL(MAHO)

#ifdef SWAPS
C --- Difference in array TSoil, 0-MACP in SWAPS
      REAL*8  TSoil(0:MACP)
#endif

#ifdef SWAP
C --- TSoil defined as 1-MACP in SWAP
      REAL*8  TSoil(MACP)
#endif

      LOGICAL differ

C ----------------------------------------------------------------------
C --- local
      INTEGER level,lay,i 

      REAL*8  cmlav,thetav,ftemp,ftheta,decact,cfluxt,cfluxb
      REAL*8  cdrtot,ctrans,cdiff,crot,dispr,diffus,old,dummy,rer
      REAL*8  in,vpore,desflu(MACP+1),deswat,desconc,destot,isqdra
      REAL*8  samount
C ----------------------------------------------------------------------
      parameter (rer = 0.001d0)

      save
C ----------------------------------------------------------------------
      isqdra  = 0.0
      samount = 0.0

C --- flux at surface
      csurf = (NIRD*cirr + ARFLUX*cpre)*dt + csurf
      if (qtop.lt.-1.d-6) then

C --- For SWAP
        cpond  = csurf / (pond-qtop*dt+wtoplat)
        cfluxt = qtop*(1.0d0-acrack)*cpond*dt
	csurf  = csurf + cfluxt - wtoplat*cpond
      else
        cpond  = 0.0
        cfluxt = 0.0
      endif

C --- solute balance in cracks
      if (SWCRACK.eq.1) then
        in = wtoplat*cpond + ARFLUX*acrack*cpre*dt + destot
        if (crackw.gt.1.0d-5) then
          crackc = (crackc*ckwm1 + in)/(crackw+(rapdra+qcrack)*dt)
        else
          crackc = 0.0
        endif
      endif

C --- Flux and concentration of water flowing through clay cracks
      if (SWCRACK.eq.1) then
        destot = 0.0
        deswat = wtoplat + arflux*acrack*dt
        if (deswat .gt. 1.d-7) then
          desconc = (wtoplat*cpond + arflux*acrack*cpre*dt) / deswat
        else
          desconc = 0.0
        endif
      endif

C --- Start loop of calculations per compartment
      do  500 i = 1,numnod
        lay = layer(i)

C --- Mobile phase -----------------------------------------------------

C --- mass exchanged between compartments
          if (i .lt. numnod) then
            cmlav  = inpola(i+1) * cml(i) + inpolb(i) * cml(i+1)
            thetav = inpola(i+1)*theta(i)+inpolb(i)*theta(i+1)
            vpore  = abs(q(i+1))/thetav
            diffus = ddif*(thetav**2.33)/(thetsl(lay)*thetsl(lay))
            dispr  = diffus + ldis * vpore + 0.5 * dt * vpore * vpore
            cfluxb = (q(i+1)*cmlav +thetav * dispr 
     &               *(cml(i+1)-cml(i))/disnod(i+1))*dt
        else
          if (q(i+1).gt.0.0) then
            cfluxb = q(i+1)*cdrain*dt
          else
            cfluxb = q(i+1)*cml(i)*dt
          endif
        endif

C --- transformation of the substance
        if (swhea .eq. 1) then
	    if (TSoil(i) .lt. 35.0d0) then
            ftemp = exp(gampar*(TSoil(i)-20.0d0))
	    else
            ftemp = exp(gampar*15.0)
          endif
	  else
	    ftemp = 1.0d0
	  endif
        ftheta = min(1.0d0,(theta(i)/rtheta)**bexp)
        decact = decpot*ftemp*ftheta*fdepth(lay)
        ctrans = decact*theta(i)*cml(i)+decact*bdens(lay)
     &           *fmobil(i)*kf*cref*((cml(i)/cref)**frexp)
        dectot = dectot + ctrans*dt*dz(i)
        
C --- plant uptake and diffusion between mobile and immobile region
        crot = tscf*qrot(i)*cml(i)/dz(i)
        rottot = rottot + tscf*qrot(i)*cml(i)*dt
        if (fmobil(i).lt.0.99d0) then
          cdiff = kmobil*(cml(i)-cil(i))
        else
          cdiff = 0.0
        endif

C --- lateral drainage
        cdrtot = 0.0
        do 100 level = 1,5
          if (qdra(level,i) .gt. 0.0) then
            cdrtot = cdrtot+qdra(level,i)*cml(i)/dz(i)
          else
            cdrtot = cdrtot+qdra(level,i)*cdrain/dz(i)
          endif
 100    continue
C --- cumulative amount of solutes to lateral drainage
        isqdra  = isqdra + cdrtot*dz(i)
        sqdra   = sqdra + cdrtot*dz(i)*dt
        samount = samount + cdrtot*dz(i)

C --- desorption flux of water flowing through cracks
        if ((SWCRACK.eq.1) .and. z(i) .gt. cralev) then
          desflu(i) = deswat * (cml(i) - desconc) * difdes * dz(i)
          destot = destot + desflu(i)
        else
          desflu(i) = 0.0
        endif

C --- conservation equation for the substance
        cmsy(i) = cmsy(i)+(cfluxb-cfluxt-desflu(i))/dz(i)+
     &            (-ctrans-crot-cdrtot-cdiff)*dt +
     &            adsflu(i) * crackc * dt / dz(i)

C --- iteration procedure for calculation of cml
        differ = .true.
        if (cmsy(i).lt.1.d-6) then
          cmsy(i) = 0.0
          cml(i)  = 0.0
        else
          if (abs(frexp-1.0d0).lt.0.001d0) then
            cml(i) = cmsy(i)/(theta(i)+bdens(lay)*kf*fmobil(i))
          else 
            if (cml(i).lt.1.d-6) cml(i) = 1.d-6
 200        if (differ) then
              old    = cml(i)
              dummy  = bdens(lay)*kf*(cml(i)/cref)**(frexp-1.0d0)
              cml(i) = cmsy(i)/(theta(i)+dummy*fmobil(i))
              if (abs(cml(i)-old).lt.rer*cml(i)) differ = .false.
              goto 200
            endif
          endif
        endif

C --- Immobile phase ---------------------------------------------------

        if (fmobil(i).lt.0.99d0) then

C --- transformation of the substance
          ctrans = (decact*(1.0d0-fmobil(i))*(thetim(lay)*cil(i)+
     &             bdens(lay)*kf*cref*(cil(i)/cref)**frexp))
          dectot = dectot + ctrans*dt*dz(i)

C --- conservation equation for the substance
          cisy(i) = cisy(i)+(-ctrans+cdiff)*dt

C --- iteration procedure for calculation of cil
          differ = .true.
          if (cisy(i).lt.1.d-6) then
            cisy(i) = 0.0
            cil(i)  = 0.0
          else
            if (abs(frexp-1.0d0).lt.0.001d0) then
              cil(i) = cisy(i) / (thetim(lay) + bdens(lay)*kf) /
     &                 (1.0d0-fmobil(i))
            else
              if (cil(i).lt.1.d-6) cil(i) = 1.d-6
 300          if (differ) then
                old    = cil(i)
                dummy  = bdens(lay)*kf*(cil(i)/cref)**(frexp-1.0d0)
                cil(i) = cisy(i)/(thetim(lay)+dummy)/(1.0d0-
     &                   fmobil(i))
                if (abs(cil(i)-old).lt.rer*cil(i)) differ = .false.
                goto 300
              endif
            endif
          endif
        endif

C --- mean solute concentration in liquid
        cl(i)  = (theta(i)*cml(i)+thetim(lay)*(1.0d0-fmobil(i))
     &           *cil(i))/(theta(i)+thetim(lay)*(1.0d0-fmobil(i)))
        cfluxt = cfluxb

C --- next compartment...
 500  continue

C --- solute balance in aquifer for breakthrough curve
      if (swbr .eq. 1) then
        if (qdrtot .gt. 0.0) then
          cdrain = cdrain + dt/(poros + bdens(layer(numnod))*kfsat) * 
     &           ( (isqdra - qdrtot*cdrain)/haquif -
     &           decsat*cdrain*(poros + bdens(layer(numnod))*kfsat) )
	  else
          cdrain = cdrain + dt/(poros + bdens(layer(numnod))*kfsat) * 
     &           ( isqdra/haquif -
     &           decsat*cdrain*(poros + bdens(layer(numnod))*kfsat) )
	  endif
	endif

C --- concentration of drainage water if DIVDRA = 1
      if (qdrtot.gt.1.d-6) then
        sdrain = samount/qdrtot
      else
        sdrain = 0.0
      endif

C --- total solute amounts for balance
      sampro = 0.0
      do 520 i = 1,numnod
        sampro = sampro + (cmsy(i) + cisy(i)) * dz(i)
 520  continue
      sampro = sampro + csurf
      samcra = crackw*crackc
      samaq  = cdrain*poros*haquif

C --- determine maximum timestep
      dtsolu = dtmax
      do 550 i = 1,numnod
        diffus = ddif * (thetav**2.33)/(thetsl(lay)*thetsl(lay))

c#ifdef SWAP
        dispr  = diffus+ldis*abs(q(i))/theta(i)
c#endif

c#ifdef SWAPS
c        dispr  = real(diffus+ldis*abs(q(i))/theta(i)/dt)
c#endif

        if (dispr.lt.1.0d-8) dispr = 1.0d-8
        dummy  = dz(i)*dz(i)*theta(i)/2.0/dispr
        dtsolu = min(dtsolu,dummy)
 550  continue

      dtsolu = max(0.001d0, dtsolu)

      return
      end

C-----------------------------------------------------------------------
      REAL*8 FUNCTION THENODE (NODE,HEAD,LAYER,FLGENU,COFGEN,
     & HTABLE,THETLO,ALFAMG,THETAR,THETAS)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calc. water content at nodal point NODE from
C                          pressure head HEAD
C     Subroutines called : -
C     Functions called   : -                                           
C     File usage         : -
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global 
      INTEGER NODE,LAYER(MACP),THETLO(MAHO)

      REAL*8  HEAD,COFGEN(8,MAHO),HTABLE(MAHO,99)
      REAL*8  ALFAMG(MACP),THETAR(MACP),THETAS(MACP)

      LOGICAL FLGENU(MAHO)

C ----------------------------------------------------------------------
C --- local

      INTEGER lay,posit

      REAL*8  help
C ----------------------------------------------------------------------

C --- soil layer of the node
      lay = layer(node)

      if (head.ge.-1.0d-2) then

C ---   saturated moisture content
        thenode = thetas(node)
      else
        if (flgenu(lay)) then

C ---     soil physical data presented according to Van Genuchten
C ---     first compute |alpha * h| ** n
          help = (abs(alfamg(node)*head))**cofgen(6,lay)

C ---     add 1 and raise to the power m
          help = (1.0d0 + help) ** cofgen(7,lay)

C ---     now compute theta
          thenode = thetar(node)+(thetas(node)-thetar(node))/help    
        else

C ---     soil physical data presented in a table
          if (head.le.htable(lay,thetlo(lay))) then

C ---       h is below lowest value of table
            thenode = (0.01d0 * thetlo(lay))
          else
            posit   = thetlo(lay)
 100        posit   = posit+1
            if (head.gt.htable(lay,posit)) goto 100

C ---       linear interpolation between water contents
            thenode = 0.01d0*((posit-1)+(head-htable(lay,posit-1))/
     $                (htable(lay,posit)-htable(lay,posit-1)))
          endif
        end if
      end if

      return
      end


      SUBROUTINE TRIDAG (N,A,B,C,R,U,ErrorCode)
************************************************************************
** Date      : 15/9/99                                    
** Purpose:    Solves for a vector U a tridiagonal linear set.         *
** References:                                                         *
** Press, W.H., B.P. Flannery, S.A. Teukolsky & W.T. Vetterling, 1989. *
** Numerical Recipes in FORTRAN. Cambridge University Press, New York. *
** pp 40-41                                                            *
************************************************************************
** Input:     N -      Number of equations                             *
**            A,B,C -  Coefficients of the matrix                      *
**            R -      known vector                                    *
** Output:    U -      solved vector                                   *
************************************************************************
      IMPLICIT NONE
      Include 'Param.Fi'

C --- Global declarations
C --- (i.i) Input
      INTEGER	  ErrorCode,N
      REAL*8	  A(N), B(N), C(N), R(N)
C --- Output
      REAL*8    U(N)
C --- Parameters
      REAL*8    Small
      PARAMETER (Small = 0.3d-37)

C --- Local declarations
      INTEGER   I
      REAL*8	  Gamma(MACP), Beta
C ----------------------------------------------------------------------
C --- (1) If B(1)=0 then rewrite the equations as a set of order N-1
C     to eliminate U(2)

      If (Abs(B(1)).LT.Small) Then
        ErrorCode = 100
      else
C ---   (2) Decomposition and forward substitution
        Beta = B(1)
        U(1) = R(1) / Beta
        do I = 2, N
          Gamma(I) = C(I-1) / Beta
          Beta = B(I) - A(I)*Gamma(I)

C ---     (2.1) If Beta=0 then go to another algorithm including
C ---     elimination with pivoting
          If (Abs(Beta).LT.Small) Then
		    ErrorCode = 110
		else
            U(I) = (R(I) - A(I)*U(I-1)) / Beta
		end if
        enddo

	  if (ErrorCode .eq. 0) then
C ---     (3) Back substitution
          Do I = N-1, 1, -1
            U(I) = U(I) - Gamma(I+1)*U(I+1)
          End Do
	  end if
      end if

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE UPDATE (NUMNOD,LAYER,H,HM1,INDEKS,TAU,FLGENU,COFGEN,
     & THETAR,THETAS,ALFAMG,THETLO,THETHI,DMCH,DMCC,DIMOCA,
     & SWMOBI,FM1,PF1,SLOPFM,HTABLE,THETA)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : to check for hysteretic reversal and update      
C                          model parameters
C     Subroutines called : -
C     Functions called   : DMCNODE               
C     File usage         : -
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NUMNOD,LAYER(MACP),INDEKS(MACP),THETLO(MAHO),THETHI(MAHO)
      INTEGER SWMOBI

      REAL*8  FM1(MAHO),PF1(MAHO),SLOPFM(MAHO)

      REAL*8  H(MACP),HM1(MACP),TAU,THETAR(MACP),THETAS(MACP)
      REAL*8  THETA(MACP)  
      REAL*8  COFGEN(8,MAHO),ALFAMG(MACP),WCN
      REAL*8  DMCC(MAHO,99),DMCH(MAHO,99),DIMOCA(MACP),HTABLE(MAHO,99)

      LOGICAL FLGENU(MAHO)
C ----------------------------------------------------------------------
C --- local
      INTEGER node,lay,indtem(MACP)

      REAL*8  dmcnode,delp,sew,sed
C ----------------------------------------------------------------------

C --- check for reversal
      do 50 node = 1,numnod
        delp = hm1(node)-h(node)
        if (delp/float(indeks(node)).gt.tau.and.
     &     h(node).lt.-10.0d0 .and. h(node).gt.-1.0d3) then
          indtem(node) = -indeks(node)
        else
          indtem(node) = indeks(node)
        endif
 50   continue

      do 60 node = 2,numnod-2
        if (indtem(node).ne.indtem(node-1).and.
     &      indtem(node).ne.indtem(node+1).and.
     &      indtem(node).ne.indtem(node+2)) indtem(node)=-indtem(node)
 60   continue

C --- change parameters scanning curves
      do 100 node = 1,numnod
        if (indtem(node).eq.indeks(node)) goto 100
        lay = layer(node)

C --- wcn is water content at reversal point
        sew = (1.0d0+(cofgen(8,lay)*(-h(node)))**cofgen(6,lay))
     &        **(-cofgen(7,lay)) 
        sed = (1.0d0+(cofgen(4,lay)*(-h(node)))**cofgen(6,lay))
     &        **(-cofgen(7,lay))

        if (indeks(node).eq.1) then
          wcn = thetar(node)+(thetas(node)-thetar(node))*sew
        else
          wcn = thetar(node)+(thetas(node)-thetar(node))*sed
        endif 

C --- change index
        indeks(node) = -1*indeks(node)

C --- update alfa, thetar and thetas
        if (indeks(node).eq.1) then

C --- wetting branch
          alfamg(node) = cofgen(8,lay)
          thetas(node) = cofgen(2,lay)
          if (sew.lt.0.995d0) then
            thetar(node) = (wcn-cofgen(2,lay)*sew)/(1.0d0-sew)
          endif
        else

C --- drying branch
          alfamg(node) = cofgen(4,lay)
          thetar(node) = cofgen(1,lay)
          thetas(node) = cofgen(1,lay)+(wcn-cofgen(1,lay))/sed
        endif

C --- update capacity 
        dimoca(node) = dmcnode(node,h(node),LAYER,FLGENU,COFGEN,THETLO,
     &                 THETHI,DMCH,DMCC,THETAS,THETAR,ALFAMG,
     &                 SWMOBI,FM1,PF1,SLOPFM,HTABLE,THETA)

 100  continue

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE WATCON (VOLM1,VOLACT,NUMNOD,THETA,DZ,FMOBIL,
     & LAYER,
     & THETIM,SWCRACK,CKWM1,CRACKW,WTOPLAT,QCRACK,ARFLUX,NIRD,ACRACK,
     & RAPDRA,DT
#ifdef SWAPS
C --- Extra SWAPS variables
     & ,NSpec,SpecFrac,NRoot,thetaR,thetaS,
     & VolPart,VolRoot,VolRes,VolSat,VolPartRes,VolPartSat
#endif
     & )
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calc. water storage in the soil profile        
C     Subroutines called : -                                           
C     Functions called   : -                                           
C     File usage         : -                                           
C     Differences SWAP/SWAPS: 
C     - SWAPS: extra parameters
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NUMNOD,LAYER(MACP),SWCRACK

      REAL*8  DZ(MACP),FMOBIL(MACP),CKWM1,CRACKW,WTOPLAT,QCRACK,ARFLUX
      REAL*8  ACRACK,RAPDRA,DT,NIRD

      REAL*8  THETA(MACP),VOLM1,VOLACT,THETIM(MAHO)

#ifdef SWAPS
C --- SWAPS variables
      INTEGER NSpec,NRoot
      REAL*8  SPecFrac
      REAL*8  thetaR(MACP),thetaS(MACP)
      REAL*8  VolPart,VolRoot,VolRes,VolSat,VolPartRes,VolPartSat
#endif

C ----------------------------------------------------------------------
C --- local
      INTEGER i,lay

#ifdef SWAPS
C --- SWAPS variables
      REAL*8  cqmatri
#endif

      save
C ----------------------------------------------------------------------

c --- change of water storage in cracks
      if (SWCRACK.eq.1) then
        ckwm1  = crackw
        crackw = crackw + wtoplat + 
     &           ((arflux+nird)*acrack - qcrack - rapdra) * dt
        crackw = max (0.0d0, crackw)

#ifdef SWAPS
        cqmatri = cqmatri + qcrack*dt
#endif

      endif

C --- update soil profile water storage
      volm1  = volact
      volact = 0.0

#ifdef SWAPS
C --- SWAPS modification - specified and root zone moisture  
      VolPart    = 0.0
      VolRoot    = 0.0
      VolRes     = 0.0
      VolSat     = 0.0
      VolPartRes = 0.0
      VolPartSat = 0.0 
#endif

      do 10 i  = 1,numnod
        lay    = layer(i)
        volact = volact+(theta(i)+(1.0d0-fmobil(i))*thetim(lay))*dz(i)

#ifdef SWAPS
C --- SWAPS modification - specified, root zone, saturated and residual 
C --- moisture       
        VolRes = VolRes + thetaR(i)*dz(i)
        VolSat = VolSat + thetaS(i)*dz(i)
        if (NSpec.gt.0) Then 
           if (i.lt.NSpec) Then
              VolPart = VolPart 
     &                +theta(i)+(1.0-fmobil(i))*thetim(lay)*dz(i)
              VolPartRes = VolPartRes + thetaR(i)*dz(i)
              VolPartSat = VolPartSat + thetaS(i)*dz(i)
           elseif (i.eq.NSpec) Then
              VolPart = VolPart 
     &                + theta(i)+(1.0-fmobil(i))*thetim(lay)*dz(i)
     &                * SpecFrac
              VolPartRes = VolPartRes + thetaR(i)*dz(i)*SpecFrac
              VolPartSat = VolPartSat + thetaS(i)*dz(i)*SpecFrac
           endif  
        endif
        if (NRoot.gt.0) Then 
           if (i.LE.NRoot) Then
              VolRoot = VolRoot+theta(i)+(1.0-fmobil(i))*
     &                  thetim(lay)*dz(i) 
           endif  
        endif
#endif

 10   continue

      return
      end


C ----------------------------------------------------------------------
      subroutine WBALLEV (wls,t,wlstab,wst,sttab,dt,runots,rapdra,
     &                    qdrd,cqdrd,cwsupp,cwout,WLSOLD)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : close surface water balance using 
C                          given (input) surface water levels   

C --- Compare storage at T + drainage fluxes with storage at T+DT
C --- The difference will be either discharge or supply
C --- Update totals (of discharge, supply and qdrd)

C     Subroutines called :                          
C     Functions called   : wstlev                          
C     File usage         :
C     Differences SWAP/SWAPS: None        
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      REAL*8  wls,t,wlstab(2*MAWLS)
      REAL*8  wst,sttab(22,2),dt,runots,rapdra,qdrd,cqdrd,cwsupp
      REAL*8  cwout,AFGEN

C --- local
      REAL*8  wlsold,wstold,wstrest,wdis,wsupp,wstlev

C ----------------------------------------------------------------------
C --- wlsold gets w-level of previous time step
       wlsold = wls

C --- fetch new level from input series
       wls = AFGEN (WLSTAB,2*MAWLS,T+DT+1.0D0)

C --- determine surface water storage for level(t-dt) and level(t)
      wstold = wstlev(wlsold,sttab) 
      wst    = wstlev(wls,sttab)

C --- determine from the surface water storages and the qdrain whether
C --- supply has taken place during period (t)-(t+dt) or water has 
C --- been discharged
      wstrest = wstold + (qdrd + rapdra)*dt + runots - wst

C --- if supply was needed, set discharge to zero
      if (wstrest.le.0.0) then
        wdis  = 0.0
        wsupp = -wstrest/dt

C --- if discharge has taken place, set supply to zero
      else
        wdis  = wstrest/dt
        wsupp = 0.0
      endif

C --- cumulation of water balance terms
      cqdrd  = cqdrd  + qdrd*dt
      cwsupp = cwsupp + wsupp*dt
      cwout  = cwout  + wdis*dt

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE WLEVBAL (
#ifdef SWAP
C --- Extra SWAP parameter
     & HLEVD,
#endif
     & NRPRI,impend,nmper,swman,imper,
     & wlstar,wls,hbweir,gwl,wlsman,gwlcrit,nphase,dropr,sttab,wscap,
     & dt,runots,rapdra,wst,zbotdr,alphaw,betaw,qdrd,cqdrd,cwsupp,
     & cwout,wlsbak,osswlm, T,NUMNOD,THETAS,THETA,DZ,VCRIT,NODHD,HCRIT,
     & H,SWQHR,QQHTAB,HQHTAB,wldip,hwlman,vtair,overfl,numadj,intwl)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculate surf. water level from water balance

C --- Set target level wlstar:
C --- SWMAN = 1: HBWEIR
C --- SWMAN = 2: from table (4e #2) and within adjustment period and 
C --- taking into account the maximum drop rate. 
C --- Calculate level for maximum supply wlstara (= wlstar-wldip)

C --- Calculate storage at wlstar and at wlstara
C ---   If wlstara is above deepest bottom level then water supply 
C ---   capacity is set to maximum value otherwise both wsttara and
C ---   wsmax are set to zero

C --- Calculate max. new storage: old + incoming/outgoing fluxes
C --- 1 System falls dry: set supply to maximum,
C       set wls at bottom of deepest channel
C --- 2 System will not become full: set supply to maximum
C       calculate new level from storage
C --- 3 System becomes full under maximum supply conditions,
C ---   Determine how, first without supply
C --- 3a wlstara cannot be reached: calculate required supply,
C ---    to reach wlstara and set wls to wlstara
C --- 3b system can be filled above wlstara, however no discharge:
C ---    wls is calculated from storage and is somewhere between
C ---    wlstar and wlstara, no supply, no discharge
C --- 3c wlstar can be reached :wsupp = 0;
C ---    Now check whether there are automatic weirs for keeping the 
C ---    level at target value or that the discharge relationship
C ---    determines new level:
C --- 3c1 SWMAN=2: calculate discharge
C ---     check whether there is enough discharge capacity:
C ---     in case SWQHR = 1: calculate discap
C ---     in case SWQHR = 2: find discap from table
C ---     if sufficient capacity wls = wlstar otherwise set overflow
C --- 3c2 SWMAN=1 or overflow
C ---     Calculate highest possible discharge, if still unsufficient 
C ---     to handle present drain fluxes: stop - system overflow
C ---     otherwise start iteration procedure to determine new level,
C ---     storage and discharge:

C ***********************************************
C --- Iteration Procedure:
C --- Establish lower and upper bounds: hbweir(imper) and +100cm
C --- Calculate storage and discharge for point halfway: wsti, wdisi
C --- Calculate new storage: if higher than wsti then adjust lower
C --- bound to point halfway otherwise adapt upper bound to point halfway
C --- Continue until upper and lower bounds converge (< 0.001 cm)   
C --- Ready: update wls, wst and wdis
C ***********************************************

C     Subroutines called :                          
C     Functions called   : wstlev                          
C     File usage         :

C     Differences Swaps/Swap:
C     1 - Swaps version does not use HLEVD
C     2 - error handling
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER impend(10),nmper,swman(10),nphase(10),NUMNOD
      INTEGER NODHD(10),SWQHR,NRPRI,NUMADJ,intwl(10)

      REAL*8  wlstar,hbweir(10),gwl,wlsman(10,10)
      REAL*8  dropr(10),sttab(22,2),wscap(10),dt,runots,rapdra
      REAL*8  wls,wst,osswlm,gwlcrit(10,10)
      REAL*8  zbotdr(5),alphaw(10),betaw(10),qdrd,cqdrd,cwsupp
      REAL*8  cwout,wlsbak(4),T,DZ(MACP),VCRIT(10,10),HCRIT(10,10)
      REAL*8  QQHTAB(10,10),HQHTAB(10,10),WLDIP(10),HWLMAN
      REAL*8  VTAIR
      REAL*8  THETAS(MACP),THETA(MACP),H(MACP)

      LOGICAL overfl
     
C --- local
      INTEGER imper,iphase,NODE,Iday

      REAL*8  wlstx,wsttar,dvmax,wstmax,wsupp,wdis,wlstarb
      REAL*8  wover,discap,wlsl,wlsu,wlsi,wsti,wdisi,wstn
      REAL*8  wprod1,wprod2,oscil,wlevst,wstlev,wlstara
      REAL*8  wsttara,qhtab,rday,wsmax

#ifdef SWAP
C --- SWAP specific vars
      LOGICAL hlevd
#endif

C --- retain values of local variables
      save
C-----------------------------------------------------------------------

C --- resetting of flag for overflowing of automatic weir
      overfl = .false.

C --- memorizing previous target level
      wlstarb = wlstar

C --- determine which management period the model is in:
      imper   = 0
 100  imper   = imper + 1

#ifdef SWAP
C --- SWAP error handling
      if (imper .gt. nmper) stop 'WLEVBAL: error sw-management periods'
#endif

#ifdef SWAPS
C --- SWAPS error handling
      if (imper .gt. nmper) then ! hiero
           call SetModelStatus(100)
* gui - ma           stop 'WLEVBAL: error sw-management periods'
           call SetFatalError(9999,'error sw-management periods',
     &                 'WLEVBAL')
           return
      endif
#endif

      if (INT(T+1.0D0) .gt. impend(imper)) goto 100

C --- determine the target sw-level:
      if (swman(imper) .eq. 1) then

C --- In the case of a fixed weir the 'target level' is set to the 
C --- weir crest, for later use in calculations to determine whether 
c --- there is any outflow at all (see below):
        wlstar = hbweir(imper)
      else

C ---  For automatic weir, determine the target level of the surface 
C ---  water, dependent on groundwater level:

c ---   only adjust it if new subperiod, with length intwl(imper) has 
c       started  (or if it is is the first call (not possible in SWAPS))
        rday = (T+1.0D0)/intwl(imper)
        iday = int(rday)

#ifdef SWAP
        if (abs(rday - 1.0*iday).lt.0.00001d0 .or. .not.hlevd) then
#endif

#ifdef SWAPS
        if (abs(rday - 1.0*iday).lt.0.00001) then
#endif

          iphase = nphase(imper)
 200      if (gwl.gt.gwlcrit(imper,iphase).and.iphase.gt.1) then
            iphase = iphase-1
            goto 200
          endif

C --- compare total air volume with VCRIT, adapt iphase
          VTAIR = 0.0
          do NODE = 1,NUMNOD
            VTAIR = VTAIR + (THETAS(NODE)-THETA(NODE))
     &              *abs(DZ(NODE))
          enddo
 214      if (VTAIR.lt.VCRIT(imper,iphase).AND.IPHASE.gt.1) then
            iphase = iphase - 1
            goto 214
          endif

C --- compare H(nodhd(imper)) with HCRIT, adapt iphase 
 218      HWLMAN = H(NODHD(imper))
          if (HWLMAN.gt.HCRIT(imper,iphase).AND.IPHASE.gt.1) then
            iphase = iphase - 1
            goto 218
          endif

          wlstx = wlsman(imper,iphase)
        else

c --- use old level
          wlstx = wlstarb
        endif

C ---   if the level must drop, then do not let it drop at more than 
C ---   the specified rate:
        if (wlstx .lt. wlstar .and. dropr(imper) .gt. 0.001d0) then
          wlstar = wlstar - dropr(imper)*dt
          if (wlstar .lt. wlstx) wlstar = wlstx
        else
          wlstar = wlstx
        endif
      endif

C --- counter of adjustments
      if (abs(wlstar-wlstarb) .gt. 0.00001D0) then
         numadj = numadj + 1
      endif

C --- storage for the 'target level'
      wsttar = wstlev(wlstar,sttab)

C --- level and storage for "max. level for supply"
      wlstara = wlstar - wldip(imper)
      if (wlstara .gt. (zbotdr(1+NRPRI)+1.D-4)) then
         wsttara = wstlev(wlstara,sttab)
         wsmax   = wscap(imper)
      else
         wsttara = 0.0
         wsmax   = 0.0
      endif

C --- determine whether the system will become full (target level or
C --- level of weir crest):
      dvmax   = (qdrd + rapdra + wsmax) * dt + runots
      wstmax  = wst + dvmax

      if (wstmax .lt. 1.0D-7) then
C ---   storage decreases to zero, then the surface water system 
C ---   falls dry; set surface water supply to maximum: 
        if (wstmax .lt. -0.1D0) then

#ifdef SWAP
C --- SWAP error handling
          stop 'WLEVBAL: error algorithm for sw falling dry'
#endif

#ifdef SWAPS
C --- SWAPS error handling
          call SetModelStatus(100)
C gui - ma           stop 'WLEVBAL: error algorithm for sw falling dry'
          call SetFatalError(9999,'error algorithm for sw falling dry',
     &         'WLEVBAL')
          return
#endif

        endif
        wsupp  = wsmax
        wdis   = 0.0
        wst    = 0.0
        wls    = zbotdr(NRPRI+1)

      elseif (wstmax .ge. 0.0 .and. wstmax .lt. wsttara) then
C --- system will not become full - set supply to max. capacity
        wsupp  = wsmax
        wdis   = 0.0
        wst    = wstmax

C --- calculate new level from storage
        wls = wlevst(wst,sttab)
      else

C --- determine how system will become full: with or without needing
C     surface water supply; try first without any supply:
        dvmax  = (qdrd + rapdra) * dt + runots
        wstmax = wst + dvmax
        if (wstmax .le. wsttara) then

C --- apparently supply is needed for reaching target level, system 
C     is made full up to level wlstara, because supply is controllable:
          wsupp  = (wsttara - wst - (qdrd+rapdra)*dt-runots)/dt
          wdis   = 0.0
          wst    = wsttara
          wls    = wlstara
        elseif (wstmax .le. wsttar) then

C --- apparently drainage is more than sufficient for filling system
C     to a level above the target level FOR SUPPLY, but not enough
C     for generating discharge; so calculate level from storage:
          wsupp = 0.0
          wdis  = 0.0
          wst   = wstmax
          wls   = wlevst(wst,sttab)
        else

C --- drainage water is more than sufficient for reaching target
C     level;  now see whether there are automatic weirs for keeping
C     the level at target value, or that the discharge relationship
C     determines new level:
          wsupp = 0.
          if (swman(imper) .eq. 2) then

C --- the outflow equals the drainage flux, plus the storage
C     excess (or deficit !) in the target situation compared to
C     the actual situation: 
            wdis = (wst-wsttar + (qdrd+rapdra)*dt + runots )/dt

C --- now check whether the weir has enough discharge capacity 
C     at this water level
            if (SWQHR.eq.1) then
              wover = wls - hbweir(imper)
              discap = alphaw(imper) * (wover**betaw(imper))
            elseif (SWQHR.eq.2) then

C --- interpolate QH table
              discap = qhtab(wlstar,imper,hqhtab,qqhtab)
            endif
            if (discap .gt. wdis) then
              wls    = wlstar
              wst    = wsttar
              overfl = .false.
            else
              overfl = .true.
            endif
          endif
          if (swman(imper) .eq. 1 .or. overfl) then

C --- determine level from q-h relationship, and also account
C     for change in storage (see below)
C     check first that the system does not overflow 
            if (SWQHR.eq.1) then
              wover = sttab(1,1) - hbweir(imper)
              discap = alphaw(imper) * (wover**betaw(imper))
            elseif (SWQHR.eq.2) then
              discap = QQHTAB(imper,1)
            endif
            if (discap .lt. (qdrd+rapdra+runots/dt)) then

#ifdef SWAP
C --- SWAP error handling
              write(6,*) qdrd,rapdra,runots,dt
              stop 'LEVBAL: surface water system has overflowed !'
#endif

#ifdef SWAPS
C --- SWAPS error handling
c              write(swl,*) qdrd,rapdra,runots,dt
C gui - ma              stop 'WLEVBAL: surface water system has overflowed !'
              call SetFatalError(9999,
     &              'surface water system has overflowed !','WLEVBAL')
              call SetModelStatus(100)
              return
#endif

            endif

C --- iteration procedure for determining new level, storage, 
C     and discharge
            wlsl = hbweir(imper)
            wlsu = sttab(1,1)

C --- find storage and discharge for intermediate point
 700        wlsi   = (wlsl + wlsu) * 0.5
            wsti  = wstlev(wlsi,sttab)
            if (SWQHR.eq.1) then
              wdisi = alphaw(imper)*(wlsi-hbweir(imper))**betaw(imper)
            else
              wdisi = qhtab(wlsi,imper,hqhtab,qqhtab)
            endif
            wstn  = wst + (qdrd + rapdra - wdisi)*dt + runots
            if (wstn .lt. wsti) then
              wlsu = wlsi 
            else
              wlsl = wlsi
            endif
            if ((wlsu - wlsl) .gt. 0.001d0) then

C --- continue iteration procedure:
              goto 700
            else

C --- updating of sw-parameters:
              wls  = wlsi
              wst  = wstn
              wdis = wdisi
            endif
          endif
        endif
      endif

C --- cumulative terms:
      cqdrd  = cqdrd  + qdrd*dt
      cwsupp = cwsupp + wsupp*dt
      cwout  = cwout  + wdis*dt

C --- updating registration of last four levels, for noting oscillation
      wlsbak(1) = wlsbak(2)
      wlsbak(2) = wlsbak(3)
      wlsbak(3) = wlsbak(4)
      wlsbak(4) = wls
      wprod1 = (wlsbak(2)-wlsbak(1))*(wlsbak(3)-wlsbak(2))
      wprod2 = (wlsbak(3)-wlsbak(2))*(wlsbak(4)-wlsbak(3))
      if (wprod1.lt.0.0 .and. wprod2.lt.0.0) then
        oscil = abs(wlsbak(3)-wlsbak(2)) 
        if (oscil .gt. osswlm) then
          write(6,9997) oscil
        endif
      endif
 9997 format(/' >>sw-level oscillation of ',f4.1,' cm; advise ',
     &       'reduction of dtmax')

#ifdef SWAP
c ---------------------------------------------------------------------
c --- register that first call for target level has been
      if (.not.hlevd) then
        hlevd = .true.
      end if
#endif

      return
      end


C ----------------------------------------------------------------------
      real*8 function WLEVST (wstor,sttab)
C ----------------------------------------------------------------------
C     Date               : 23/08/99
C     Purpose            : calculate surface water level from    
C                          surface water storage, using table
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER i

      REAL*8  wstor, sttab(22,2),dwst
C ----------------------------------------------------------------------

      if (wstor .lt. sttab(22,2)) then
         STOP 'WLEVST : surface water storage below bottom of table'
      endif
      if (wstor .gt. sttab(1,2)) then
         STOP 'WLEVST : surface water storage above top of table'
      endif
      i = 0
 100  i = i+1
      if (.not.(wstor.ge.sttab(i+1,2).and.
     &          wstor.le.sttab(i,2))) goto 100
      dwst   = (wstor - sttab(i+1,2)) / (sttab(i,2) - sttab(i+1,2))
      wlevst = sttab(i+1,1) + dwst * (sttab(i,1) - sttab(i+1,1))

      return
      end


C ----------------------------------------------------------------------
      real*8 function WSTLEV (wlev,sttab)
C-----------------------------------------------------------------------
C     Date               : 23/08/99
C     Purpose            : calculate surface water storage from  
C                          surface water level
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER i

      REAL*8  wlev, sttab(22,2),dwl
C ----------------------------------------------------------------------

      if (wlev .lt. sttab(22,1)) then
         STOP 'WSTLEV : surface water level below bottom of table'
      endif
      if (wlev .gt. sttab(1,1)) then
         STOP 'WSTLEV : surface water level above top of table'
      endif

      i = 0
 100  i = i+1
      if (.not.(wlev.ge.sttab(i+1,1).and.wlev.le.sttab(i,1))) goto 100
      dwl    = (wlev - sttab(i+1,1)) / (sttab(i,1) - sttab(i+1,1))
      wstlev = sttab(i+1,2) + dwl * (sttab(i,2) - sttab(i+1,2))

      return
      end


